@@ -4,6 +4,7 @@
 #include "logger/logger.hpp"
 #include "pool/part.hpp"
 #include "util/util.hpp"
+#include "util/picture_load.hpp"
 #include <list>
 #include "nlohmann/json.hpp"
 
@@ -188,6 +189,13 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, Pool &pool, ViaPadsta
                          Logger::Domain::BOARD);
         }
     }
+    if (j.count("pictures")) {
+        const json &o = j["pictures"];
+        for (auto it = o.cbegin(); it != o.cend(); ++it) {
+            auto u = UUID(it.key());
+            load_and_log(pictures, ObjectType::PICTURE, std::forward_as_tuple(u, it.value()), Logger::Domain::BOARD);
+        }
+    }
     if (j.count("rules")) {
         try {
             rules.load_from_json(j.at("rules"));
@@ -250,13 +258,14 @@ Board Board::new_from_file(const std::string &filename, Block &block, Pool &pool
     return Board(UUID(j.at("uuid").get<std::string>()), j, block, pool, vpp);
 }
 
+
 Board::Board(const UUID &uu, Block &bl) : uuid(uu), block(&bl)
 {
     rules.add_rule(RuleID::CLEARANCE_COPPER);
     rules.add_rule(RuleID::CLEARANCE_COPPER_OTHER);
     rules.add_rule(RuleID::TRACK_WIDTH);
     rules.add_rule(RuleID::PLANE);
-    auto r = dynamic_cast<RuleTrackWidth *>(rules.get_rules(RuleID::TRACK_WIDTH).begin()->second);
+    auto r = dynamic_cast<RuleTrackWidth *>(rules.get_rules_nc(RuleID::TRACK_WIDTH).begin()->second);
     r->widths.emplace(std::piecewise_construct, std::forward_as_tuple(0), std::forward_as_tuple());
     // TBD: inner layers
     r->widths.emplace(std::piecewise_construct, std::forward_as_tuple(-100), std::forward_as_tuple());
@@ -281,10 +290,11 @@ Board::Board(const Board &brd, CopyMode copy_mode)
     : layers(brd.layers), uuid(brd.uuid), block(brd.block), name(brd.name), polygons(brd.polygons), holes(brd.holes),
       junctions(brd.junctions), tracks(brd.tracks), texts(brd.texts), lines(brd.lines), arcs(brd.arcs),
       planes(brd.planes), keepouts(brd.keepouts), dimensions(brd.dimensions), connection_lines(brd.connection_lines),
-      included_boards(brd.included_boards), board_panels(brd.board_panels), warnings(brd.warnings), rules(brd.rules),
-      fab_output_settings(brd.fab_output_settings), airwires(brd.airwires), stackup(brd.stackup), colors(brd.colors),
-      pdf_export_settings(brd.pdf_export_settings), step_export_settings(brd.step_export_settings),
-      pnp_export_settings(brd.pnp_export_settings), n_inner_layers(brd.n_inner_layers)
+      included_boards(brd.included_boards), board_panels(brd.board_panels), pictures(brd.pictures),
+      warnings(brd.warnings), rules(brd.rules), fab_output_settings(brd.fab_output_settings), airwires(brd.airwires),
+      stackup(brd.stackup), colors(brd.colors), pdf_export_settings(brd.pdf_export_settings),
+      step_export_settings(brd.step_export_settings), pnp_export_settings(brd.pnp_export_settings),
+      n_inner_layers(brd.n_inner_layers)
 {
     if (copy_mode == CopyMode::DEEP) {
         packages = brd.packages;
@@ -566,7 +576,6 @@ void Board::expand(bool careful)
     expand_packages();
 
     for (auto &it : junctions) {
-        it.second.temp = false;
         it.second.layer = 10000;
         it.second.has_via = false;
         it.second.needs_via = false;
@@ -617,13 +626,7 @@ void Board::expand(bool careful)
 
     auto params = rules.get_parameters();
     for (auto &it : vias) {
-        it.second.junction->has_via = true;
-        it.second.junction->layer = 10000;
-        it.second.padstack = *it.second.vpp_padstack;
-        ParameterSet ps_via = it.second.parameter_set;
-        ps_via.emplace(ParameterID::VIA_SOLDER_MASK_EXPANSION, params->via_solder_mask_expansion);
-        it.second.padstack.apply_parameter_set(ps_via);
-        it.second.padstack.expand_inner(n_inner_layers);
+        it.second.expand(*this);
     }
     for (auto &it : holes) {
         it.second.padstack = *it.second.pool_padstack;
@@ -681,10 +684,11 @@ void Board::expand(bool careful)
     }
 
     if (expand_flags & EXPAND_AIRWIRES)
-        update_airwires();
+        update_airwires(false, airwires_expand);
 
     expand_flags = EXPAND_ALL;
     packages_expand.clear();
+    airwires_expand.clear();
 }
 
 void Board::expand_packages()
@@ -707,12 +711,15 @@ void Board::expand_packages()
             if (it.second.alternate_package) {
                 std::set<std::string> pads_from_primary, pads_from_alt;
                 for (const auto &it_pad : it.second.pool_package->pads) {
-                    pads_from_primary.insert(it_pad.second.name);
+                    if (it_pad.second.padstack.type != Padstack::Type::MECHANICAL)
+                        pads_from_primary.insert(it_pad.second.name);
                 }
                 bool alt_valid = true;
                 for (const auto &it_pad : it.second.alternate_package->pads) {
-                    if (!pads_from_alt.insert(it_pad.second.name).second) { // duplicate pad name
-                        alt_valid = false;
+                    if (it_pad.second.padstack.type != Padstack::Type::MECHANICAL) {
+                        if (!pads_from_alt.insert(it_pad.second.name).second) { // duplicate pad name
+                            alt_valid = false;
+                        }
                     }
                 }
                 if (!alt_valid || pads_from_alt != pads_from_primary) { // alt pkg isn't pad-equal
@@ -721,15 +728,21 @@ void Board::expand_packages()
                 }
                 else {
                     it.second.package = *it.second.alternate_package;
-                    it.second.package.pads.clear(); // need to adjust pad uuids to primary package
+
+                    // need to adjust pad uuids to primary package
+                    map_erase_if(it.second.package.pads,
+                                 [](const auto &x) { return x.second.padstack.type != Padstack::Type::MECHANICAL; });
                     std::map<std::string, UUID> pad_uuids;
                     for (const auto &it_pad : it.second.pool_package->pads) {
-                        assert(pad_uuids.emplace(it_pad.second.name, it_pad.first).second); // no duplicates
+                        if (it_pad.second.padstack.type != Padstack::Type::MECHANICAL)
+                            assert(pad_uuids.emplace(it_pad.second.name, it_pad.first).second); // no duplicates
                     }
                     for (const auto &it_pad : it.second.alternate_package->pads) {
-                        auto uu = pad_uuids.at(it_pad.second.name);
-                        auto &pad = it.second.package.pads.emplace(uu, it_pad.second).first->second;
-                        pad.uuid = uu;
+                        if (it_pad.second.padstack.type != Padstack::Type::MECHANICAL) {
+                            auto uu = pad_uuids.at(it_pad.second.name);
+                            auto &pad = it.second.package.pads.emplace(uu, it_pad.second).first->second;
+                            pad.uuid = uu;
+                        }
                     }
                 }
             }
@@ -791,19 +804,34 @@ void Board::expand_packages()
     // assign nets to pads based on netlist
     for (auto &it : packages) {
         for (auto &it_pad : it.second.package.pads) {
+            it_pad.second.is_nc = false;
             if (it.second.component->part->pad_map.count(it_pad.first)) {
                 const auto &pad_map_item = it.second.component->part->pad_map.at(it_pad.first);
                 auto pin_path = UUIDPath<2>(pad_map_item.gate->uuid, pad_map_item.pin->uuid);
                 if (it.second.component->connections.count(pin_path)) {
                     const auto &conn = it.second.component->connections.at(pin_path);
                     it_pad.second.net = conn.net;
+                    if (conn.net) {
+                        it_pad.second.secondary_text = conn.net->name;
+                    }
+                    else {
+                        it_pad.second.secondary_text = "NC";
+                        it_pad.second.is_nc = true;
+                    }
                 }
                 else {
                     it_pad.second.net = nullptr;
+                    it_pad.second.secondary_text = "("
+                                                   + it.second.component->part->entity->gates.at(pin_path.at(0))
+                                                             .unit->pins.at(pin_path.at(1))
+                                                             .primary_name
+                                                   + ")";
                 }
             }
             else {
                 it_pad.second.net = nullptr;
+                it_pad.second.secondary_text = "NC";
+                it_pad.second.is_nc = true;
             }
         }
     }
@@ -1197,6 +1225,23 @@ json Board::serialize() const
             j["board_panels"][(std::string)it.first] = it.second.serialize();
         }
     }
+    if (pictures.size()) {
+        j["pictures"] = json::object();
+        for (const auto &it : pictures) {
+            j["pictures"][(std::string)it.first] = it.second.serialize();
+        }
+    }
     return j;
 }
+
+void Board::save_pictures(const std::string &dir) const
+{
+    pictures_save({&pictures}, dir, "brd");
+}
+
+void Board::load_pictures(const std::string &dir)
+{
+    pictures_load({&pictures}, dir, "brd");
+}
+
 } // namespace horizon
