@@ -19,6 +19,7 @@ public:
     BoardPackage(const UUID &uu, const json &, Block &block, Pool &pool);
     BoardPackage(const UUID &uu, Component *comp);
     BoardPackage(const UUID &uu);
+    BoardPackage(shallow_copy_t sh, const BoardPackage &other);
     UUID uuid;
     uuid_ptr<Component> component;
     const Package *alternate_package = nullptr;
@@ -32,11 +33,13 @@ public:
     bool smashed = false;
     bool omit_silkscreen = false;
     bool fixed = false;
+    bool omit_outline = false;
     std::vector<uuid_ptr<Text>> texts;
 
     std::string replace_text(const std::string &t, bool *replaced = nullptr) const;
 
     UUID get_uuid() const override;
     json serialize() const;
+    static std::vector<UUID> peek_texts(const json &j);
 };
 } // namespace horizon
