@@ -10,6 +10,36 @@ BoardRules::BoardRules()
 {
 }
 
+BoardRules::BoardRules(const BoardRules &other)
+    : rule_hole_size(other.rule_hole_size), rule_track_width(other.rule_track_width),
+      rule_clearance_copper(other.rule_clearance_copper), rule_via(other.rule_via),
+      rule_clearance_copper_other(other.rule_clearance_copper_other), rule_plane(other.rule_plane),
+      rule_diffpair(other.rule_diffpair), rule_clearance_copper_keepout(other.rule_clearance_copper_keepout),
+      rule_layer_pair(other.rule_layer_pair),
+      rule_clearance_silkscreen_exposed_copper(other.rule_clearance_silkscreen_exposed_copper),
+      rule_parameters(other.rule_parameters), rule_preflight_checks(other.rule_preflight_checks)
+{
+    update_sorted();
+}
+
+void BoardRules::operator=(const BoardRules &other)
+{
+    rule_hole_size = other.rule_hole_size;
+    rule_track_width = other.rule_track_width;
+    rule_clearance_copper = other.rule_clearance_copper;
+    rule_via = other.rule_via;
+    rule_clearance_copper_other = other.rule_clearance_copper_other;
+    rule_plane = other.rule_plane;
+    rule_diffpair = other.rule_diffpair;
+    rule_clearance_copper_keepout = other.rule_clearance_copper_keepout;
+    rule_layer_pair = other.rule_layer_pair;
+    rule_clearance_silkscreen_exposed_copper = other.rule_clearance_silkscreen_exposed_copper;
+    rule_parameters = other.rule_parameters;
+    rule_preflight_checks = other.rule_preflight_checks;
+
+    update_sorted();
+}
+
 void BoardRules::load_from_json(const json &j)
 {
     if (j.count("hole_size")) {
@@ -38,6 +68,7 @@ void BoardRules::load_from_json(const json &j)
                                           std::forward_as_tuple(u, it.value()));
         }
         fix_order(RuleID::CLEARANCE_COPPER);
+        update_sorted();
     }
     if (j.count("via")) {
         const json &o = j["via"];
@@ -92,6 +123,15 @@ void BoardRules::load_from_json(const json &j)
         }
         fix_order(RuleID::CLEARANCE_COPPER_KEEPOUT);
     }
+    if (j.count("layer_pair")) {
+        const json &o = j["layer_pair"];
+        for (auto it = o.cbegin(); it != o.cend(); ++it) {
+            auto u = UUID(it.key());
+            rule_layer_pair.emplace(std::piecewise_construct, std::forward_as_tuple(u),
+                                    std::forward_as_tuple(u, it.value()));
+        }
+        fix_order(RuleID::LAYER_PAIR);
+    }
     if (j.count("clearance_silkscreen_exposed_copper")) {
         const json &o = j["clearance_silkscreen_exposed_copper"];
         rule_clearance_silkscreen_exposed_copper = RuleClearanceSilkscreenExposedCopper(o);
@@ -127,13 +167,16 @@ void BoardRules::cleanup(const Block *block)
         it.second.match.cleanup(block);
         it.second.match_keepout.cleanup(block);
     }
+    for (auto &it : rule_layer_pair) {
+        it.second.match.cleanup(block);
+    }
     for (auto &it : rule_diffpair) {
         if (!block->net_classes.count(it.second.net_class))
             it.second.net_class = block->net_class_default->uuid;
     }
 }
 
-void BoardRules::apply(RuleID id, Board *brd, ViaPadstackProvider &vpp)
+void BoardRules::apply(RuleID id, Board *brd, ViaPadstackProvider &vpp) const
 {
     if (id == RuleID::TRACK_WIDTH) {
         for (auto &it : brd->tracks) {
@@ -202,6 +245,10 @@ json BoardRules::serialize() const
     for (const auto &it : rule_clearance_copper_keepout) {
         j["clearance_copper_keepout"][(std::string)it.first] = it.second.serialize();
     }
+    j["layer_pair"] = json::object();
+    for (const auto &it : rule_layer_pair) {
+        j["layer_pair"][(std::string)it.first] = it.second.serialize();
+    }
     j["clearance_silkscreen_exposed_copper"] = rule_clearance_silkscreen_exposed_copper.serialize();
     j["parameters"] = rule_parameters.serialize();
     return j;
@@ -219,10 +266,11 @@ std::set<RuleID> BoardRules::get_rule_ids() const
             RuleID::PLANE,
             RuleID::DIFFPAIR,
             RuleID::PREFLIGHT_CHECKS,
-            RuleID::CLEARANCE_COPPER_KEEPOUT};
+            RuleID::CLEARANCE_COPPER_KEEPOUT,
+            RuleID::LAYER_PAIR};
 }
 
-Rule *BoardRules::get_rule(RuleID id)
+const Rule *BoardRules::get_rule(RuleID id) const
 {
     if (id == RuleID::CLEARANCE_SILKSCREEN_EXPOSED_COPPER) {
         return &rule_clearance_silkscreen_exposed_copper;
@@ -236,7 +284,7 @@ Rule *BoardRules::get_rule(RuleID id)
     return nullptr;
 }
 
-Rule *BoardRules::get_rule(RuleID id, const UUID &uu)
+const Rule *BoardRules::get_rule(RuleID id, const UUID &uu) const
 {
     switch (id) {
     case RuleID::HOLE_SIZE:
@@ -255,58 +303,66 @@ Rule *BoardRules::get_rule(RuleID id, const UUID &uu)
         return &rule_diffpair.at(uu);
     case RuleID::CLEARANCE_COPPER_KEEPOUT:
         return &rule_clearance_copper_keepout.at(uu);
+    case RuleID::LAYER_PAIR:
+        return &rule_layer_pair.at(uu);
     default:
         return nullptr;
     }
 }
 
-std::map<UUID, Rule *> BoardRules::get_rules(RuleID id)
+std::map<UUID, const Rule *> BoardRules::get_rules(RuleID id) const
 {
-    std::map<UUID, Rule *> r;
+    std::map<UUID, const Rule *> r;
     switch (id) {
     case RuleID::HOLE_SIZE:
         for (auto &it : rule_hole_size) {
-            r[it.first] = &it.second;
+            r.emplace(it.first, &it.second);
         }
         break;
     case RuleID::TRACK_WIDTH:
         for (auto &it : rule_track_width) {
-            r[it.first] = &it.second;
+            r.emplace(it.first, &it.second);
         }
         break;
     case RuleID::CLEARANCE_COPPER:
         for (auto &it : rule_clearance_copper) {
-            r[it.first] = &it.second;
+            r.emplace(it.first, &it.second);
         }
         break;
 
     case RuleID::VIA:
         for (auto &it : rule_via) {
-            r[it.first] = &it.second;
+            r.emplace(it.first, &it.second);
         }
         break;
 
     case RuleID::CLEARANCE_COPPER_OTHER:
         for (auto &it : rule_clearance_copper_other) {
-            r[it.first] = &it.second;
+            r.emplace(it.first, &it.second);
         }
         break;
 
     case RuleID::PLANE:
         for (auto &it : rule_plane) {
-            r[it.first] = &it.second;
+            r.emplace(it.first, &it.second);
         }
         break;
 
     case RuleID::DIFFPAIR:
         for (auto &it : rule_diffpair) {
-            r[it.first] = &it.second;
+            r.emplace(it.first, &it.second);
         }
         break;
 
     case RuleID::CLEARANCE_COPPER_KEEPOUT:
         for (auto &it : rule_clearance_copper_keepout) {
-            r[it.first] = &it.second;
+            r.emplace(it.first, &it.second);
+        }
+        break;
+
+    case RuleID::LAYER_PAIR:
+        for (auto &it : rule_layer_pair) {
+            r.emplace(it.first, &it.second);
         }
         break;
 
@@ -350,9 +406,14 @@ void BoardRules::remove_rule(RuleID id, const UUID &uu)
         rule_clearance_copper_keepout.erase(uu);
         break;
 
+    case RuleID::LAYER_PAIR:
+        rule_layer_pair.erase(uu);
+        break;
+
     default:;
     }
     fix_order(id);
+    update_sorted();
 }
 
 Rule *BoardRules::add_rule(RuleID id)
@@ -362,54 +423,56 @@ Rule *BoardRules::add_rule(RuleID id)
     switch (id) {
     case RuleID::HOLE_SIZE:
         r = &rule_hole_size.emplace(uu, uu).first->second;
-        r->order = -1;
         break;
 
     case RuleID::TRACK_WIDTH:
         r = &rule_track_width.emplace(uu, uu).first->second;
-        r->order = -1;
         break;
 
     case RuleID::CLEARANCE_COPPER:
         r = &rule_clearance_copper.emplace(uu, uu).first->second;
-        r->order = -1;
         break;
 
     case RuleID::VIA:
         r = &rule_via.emplace(uu, uu).first->second;
-        r->order = -1;
         break;
 
     case RuleID::CLEARANCE_COPPER_OTHER:
         r = &rule_clearance_copper_other.emplace(uu, uu).first->second;
-        r->order = -1;
         break;
 
     case RuleID::PLANE:
         r = &rule_plane.emplace(uu, uu).first->second;
-        r->order = -1;
         break;
 
     case RuleID::DIFFPAIR:
         r = &rule_diffpair.emplace(uu, uu).first->second;
-        r->order = -1;
         break;
 
     case RuleID::CLEARANCE_COPPER_KEEPOUT:
         r = &rule_clearance_copper_keepout.emplace(uu, uu).first->second;
-        r->order = -1;
+        break;
+
+    case RuleID::LAYER_PAIR:
+        r = &rule_layer_pair.emplace(uu, uu).first->second;
         break;
 
     default:
         return nullptr;
     }
     fix_order(id);
+    update_sorted();
     return r;
 }
 
-uint64_t BoardRules::get_default_track_width(Net *net, int layer)
+void BoardRules::update_sorted()
 {
-    auto rules = dynamic_cast_vector<RuleTrackWidth *>(get_rules_sorted(RuleID::TRACK_WIDTH));
+    rule_sorted_clearance_copper = get_rules_sorted<const RuleClearanceCopper>(RuleID::CLEARANCE_COPPER);
+}
+
+uint64_t BoardRules::get_default_track_width(const Net *net, int layer) const
+{
+    auto rules = get_rules_sorted<RuleTrackWidth>(RuleID::TRACK_WIDTH);
     for (auto rule : rules) {
         if (rule->enabled && rule->match.match(net)) {
             if (rule->widths.count(layer)) {
@@ -422,10 +485,9 @@ uint64_t BoardRules::get_default_track_width(Net *net, int layer)
 
 static const RuleClearanceCopper fallback_clearance_copper = UUID();
 
-const RuleClearanceCopper *BoardRules::get_clearance_copper(Net *net1, Net *net2, int layer)
+const RuleClearanceCopper *BoardRules::get_clearance_copper(const Net *net1, const Net *net2, int layer) const
 {
-    auto rules = dynamic_cast_vector<RuleClearanceCopper *>(get_rules_sorted(RuleID::CLEARANCE_COPPER));
-    for (auto ru : rules) {
+    for (auto ru : rule_sorted_clearance_copper) {
         if (ru->enabled
             && ((ru->match_1.match(net1) && ru->match_2.match(net2))
                 || (ru->match_1.match(net2) && ru->match_2.match(net1)))
@@ -438,9 +500,9 @@ const RuleClearanceCopper *BoardRules::get_clearance_copper(Net *net1, Net *net2
 
 static const RuleClearanceCopperOther fallback_clearance_copper_non_copper = UUID();
 
-const RuleClearanceCopperOther *BoardRules::get_clearance_copper_other(Net *net, int layer)
+const RuleClearanceCopperOther *BoardRules::get_clearance_copper_other(const Net *net, int layer) const
 {
-    auto rules = dynamic_cast_vector<RuleClearanceCopperOther *>(get_rules_sorted(RuleID::CLEARANCE_COPPER_OTHER));
+    auto rules = get_rules_sorted<RuleClearanceCopperOther>(RuleID::CLEARANCE_COPPER_OTHER);
     for (auto ru : rules) {
         if (ru->enabled && (ru->match.match(net) && (ru->layer == layer || ru->layer == 10000))) {
             return ru;
@@ -451,9 +513,9 @@ const RuleClearanceCopperOther *BoardRules::get_clearance_copper_other(Net *net,
 
 static const RuleDiffpair diffpair_fallback = UUID();
 
-const RuleDiffpair *BoardRules::get_diffpair(NetClass *net_class, int layer)
+const RuleDiffpair *BoardRules::get_diffpair(const NetClass *net_class, int layer) const
 {
-    auto rules = dynamic_cast_vector<RuleDiffpair *>(get_rules_sorted(RuleID::DIFFPAIR));
+    auto rules = get_rules_sorted<RuleDiffpair>(RuleID::DIFFPAIR);
     for (auto ru : rules) {
         if (ru->enabled && (ru->net_class == net_class->uuid && (ru->layer == layer || ru->layer == 10000))) {
             return ru;
@@ -464,9 +526,10 @@ const RuleDiffpair *BoardRules::get_diffpair(NetClass *net_class, int layer)
 
 static const RuleClearanceCopperKeepout keepout_fallback = UUID();
 
-const RuleClearanceCopperKeepout *BoardRules::get_clearance_copper_keepout(Net *net, const KeepoutContour *contour)
+const RuleClearanceCopperKeepout *BoardRules::get_clearance_copper_keepout(const Net *net,
+                                                                           const KeepoutContour *contour) const
 {
-    auto rules = dynamic_cast_vector<RuleClearanceCopperKeepout *>(get_rules_sorted(RuleID::CLEARANCE_COPPER_KEEPOUT));
+    auto rules = get_rules_sorted<RuleClearanceCopperKeepout>(RuleID::CLEARANCE_COPPER_KEEPOUT);
     for (auto ru : rules) {
         if (ru->enabled && ru->match.match(net) && ru->match_keepout.match(contour)) {
             return ru;
@@ -475,11 +538,11 @@ const RuleClearanceCopperKeepout *BoardRules::get_clearance_copper_keepout(Net *
     return &keepout_fallback;
 }
 
-uint64_t BoardRules::get_max_clearance()
+uint64_t BoardRules::get_max_clearance() const
 {
     uint64_t max_clearance = 0;
     {
-        auto rules = dynamic_cast_vector<RuleClearanceCopper *>(get_rules_sorted(RuleID::CLEARANCE_COPPER));
+        auto rules = get_rules_sorted<RuleClearanceCopper>(RuleID::CLEARANCE_COPPER);
         for (auto ru : rules) {
             if (ru->enabled) {
                 max_clearance = std::max(max_clearance, ru->get_max_clearance());
@@ -487,7 +550,7 @@ uint64_t BoardRules::get_max_clearance()
         }
     }
     {
-        auto rules = dynamic_cast_vector<RuleClearanceCopperOther *>(get_rules_sorted(RuleID::CLEARANCE_COPPER_OTHER));
+        auto rules = get_rules_sorted<RuleClearanceCopperOther>(RuleID::CLEARANCE_COPPER_OTHER);
         for (auto ru : rules) {
             if (ru->enabled) {
                 max_clearance = std::max(max_clearance, ru->get_max_clearance());
@@ -495,8 +558,7 @@ uint64_t BoardRules::get_max_clearance()
         }
     }
     {
-        auto rules =
-                dynamic_cast_vector<RuleClearanceCopperKeepout *>(get_rules_sorted(RuleID::CLEARANCE_COPPER_KEEPOUT));
+        auto rules = get_rules_sorted<RuleClearanceCopperKeepout>(RuleID::CLEARANCE_COPPER_KEEPOUT);
         for (auto ru : rules) {
             if (ru->enabled) {
                 max_clearance = std::max(max_clearance, ru->get_max_clearance());
@@ -506,14 +568,14 @@ uint64_t BoardRules::get_max_clearance()
     return max_clearance;
 }
 
-const RuleParameters *BoardRules::get_parameters()
+const RuleParameters *BoardRules::get_parameters() const
 {
     return &rule_parameters;
 }
 
-UUID BoardRules::get_via_padstack_uuid(class Net *net)
+UUID BoardRules::get_via_padstack_uuid(const Net *net) const
 {
-    auto rules = dynamic_cast_vector<RuleVia *>(get_rules_sorted(RuleID::VIA));
+    auto rules = get_rules_sorted<RuleVia>(RuleID::VIA);
     for (auto rule : rules) {
         if (rule->enabled && rule->match.match(net)) {
             return rule->padstack;
@@ -524,9 +586,9 @@ UUID BoardRules::get_via_padstack_uuid(class Net *net)
 
 static const ParameterSet ps_empty;
 
-const ParameterSet &BoardRules::get_via_parameter_set(class Net *net)
+const ParameterSet &BoardRules::get_via_parameter_set(const Net *net) const
 {
-    auto rules = dynamic_cast_vector<RuleVia *>(get_rules_sorted(RuleID::VIA));
+    auto rules = get_rules_sorted<RuleVia>(RuleID::VIA);
     for (auto rule : rules) {
         if (rule->enabled && rule->match.match(net)) {
             return rule->parameter_set;
@@ -537,9 +599,9 @@ const ParameterSet &BoardRules::get_via_parameter_set(class Net *net)
 
 static const PlaneSettings plane_settings_default;
 
-const PlaneSettings &BoardRules::get_plane_settings(Net *net, int layer)
+const PlaneSettings &BoardRules::get_plane_settings(const Net *net, int layer) const
 {
-    auto rules = dynamic_cast_vector<RulePlane *>(get_rules_sorted(RuleID::PLANE));
+    auto rules = get_rules_sorted<RulePlane>(RuleID::PLANE);
     for (auto rule : rules) {
         if (rule->enabled && rule->match.match(net) && (rule->layer == layer || rule->layer == 10000)) {
             return rule->settings;
@@ -547,4 +609,22 @@ const PlaneSettings &BoardRules::get_plane_settings(Net *net, int layer)
     }
     return plane_settings_default;
 }
+
+int BoardRules::get_layer_pair(const Net *net, int layer) const
+{
+    auto rules = get_rules_sorted<RuleLayerPair>(RuleID::LAYER_PAIR);
+    for (auto rule : rules) {
+        if (rule->enabled && rule->match.match(net)) {
+            if (rule->layers.first == layer)
+                return rule->layers.second;
+            else if (rule->layers.second == layer)
+                return rule->layers.first;
+            else
+                return layer;
+        }
+    }
+    return layer;
+}
+
+
 } // namespace horizon
