@@ -16,8 +16,7 @@ void CanvasGL::remove_annotation(CanvasAnnotation *a)
 {
     auto layer = a->layer;
     annotations.erase(layer);
-    if (triangles.count(layer))
-        triangles.erase(layer);
+    triangles.erase(layer);
 }
 
 bool CanvasGL::layer_is_annotation(int l) const
@@ -48,9 +47,10 @@ void CanvasAnnotation::clear()
     ca->request_push();
 }
 
-void CanvasAnnotation::draw_line(const Coordf &from, const Coordf &to, ColorP color, uint64_t width)
+void CanvasAnnotation::draw_line(const Coordf &from, const Coordf &to, ColorP color, uint64_t width, bool highlight,
+                                 uint8_t color2)
 {
-    ca->draw_line(from, to, color, layer, false, width);
+    ca->add_triangle(layer, from, to, Coordf(width, NAN), color, highlight ? TriangleInfo::FLAG_HIGHLIGHT : 0, color2);
     ca->request_push();
 }
 } // namespace horizon
