@@ -9,7 +9,7 @@
 
 namespace horizon {
 
-Canvas::Canvas() : selection_filter(this), selectables(this)
+Canvas::Canvas() : selectables(*this)
 {
     layer_display[10000] = LayerDisplay(true, LayerDisplay::Mode::FILL);
     layer_display[0] = LayerDisplay(true, LayerDisplay::Mode::FILL);
@@ -18,6 +18,7 @@ Canvas::Canvas() : selection_filter(this), selectables(this)
 void Canvas::set_layer_display(int index, const LayerDisplay &ld)
 {
     layer_display[index] = ld;
+    request_push();
 }
 
 static const LayerDisplay ld_default;
@@ -44,8 +45,12 @@ void Canvas::clear()
     }
     targets.clear();
     sheet_current_uuid = UUID();
-    object_refs.clear();
+    for (auto &it : object_refs) {
+        it.second.clear();
+    }
     object_refs_current.clear();
+    object_ref_idx.clear();
+    pictures.clear();
 }
 
 void Canvas::remove_obj(const ObjectRef &r)
@@ -56,13 +61,12 @@ void Canvas::remove_obj(const ObjectRef &r)
     for (const auto &it : object_refs.at(r)) {
         auto layer = it.first;
         layers.insert(layer);
-        auto begin = triangles[layer].begin();
-        auto first = begin + it.second.first;
-        auto last = begin + it.second.second + 1;
+        auto first = it.second.first;
+        auto last = it.second.second + 1;
         assert(it.second.first < triangles[layer].size());
         assert(it.second.second < triangles[layer].size());
 
-        triangles[layer].erase(first, last);
+        triangles.at(layer).erase(first, last);
     }
 
     // fix refs that changed due to triangles being deleted
@@ -94,8 +98,8 @@ void Canvas::set_flags(const ObjectRef &r, uint8_t mask_set, uint8_t mask_clear)
     for (const auto &it : object_refs.at(r)) {
         auto layer = it.first;
         for (auto i = it.second.first; i <= it.second.second; i++) {
-            triangles[layer][i].flags |= mask_set;
-            triangles[layer][i].flags &= ~mask_clear;
+            triangles.at(layer).atm(i).second.flags |= mask_set;
+            triangles.at(layer).atm(i).second.flags &= ~mask_clear;
         }
     }
     request_push();
@@ -104,9 +108,9 @@ void Canvas::set_flags(const ObjectRef &r, uint8_t mask_set, uint8_t mask_clear)
 void Canvas::set_flags_all(uint8_t mask_set, uint8_t mask_clear)
 {
     for (auto &it : triangles) {
-        for (auto &it2 : it.second) {
-            it2.flags |= mask_set;
-            it2.flags &= ~mask_clear;
+        for (auto it2 : it.second) {
+            it2.second.flags |= mask_set;
+            it2.second.flags &= ~mask_clear;
         }
     }
     request_push();
@@ -114,37 +118,64 @@ void Canvas::set_flags_all(uint8_t mask_set, uint8_t mask_clear)
 
 void Canvas::hide_obj(const ObjectRef &r)
 {
-    set_flags(r, Triangle::FLAG_HIDDEN, 0);
+    set_flags(r, TriangleInfo::FLAG_HIDDEN, 0);
 }
 
 void Canvas::show_obj(const ObjectRef &r)
 {
-    set_flags(r, 0, Triangle::FLAG_HIDDEN);
+    set_flags(r, 0, TriangleInfo::FLAG_HIDDEN);
 }
 
 void Canvas::show_all_obj()
 {
-    set_flags_all(0, Triangle::FLAG_HIDDEN);
+    set_flags_all(0, TriangleInfo::FLAG_HIDDEN);
 }
 
-void Canvas::add_triangle(int layer, const Coordf &p0, const Coordf &p1, const Coordf &p2, ColorP color, uint8_t flags)
+void Canvas::reset_color2()
 {
-    triangles[layer].emplace_back(p0, p1, p2, color, triangle_type_current, flags, lod_current);
-    if (!fast_draw) {
-        auto idx = triangles[layer].size() - 1;
-        for (auto &ref : object_refs_current) {
-            auto &layers = object_refs[ref];
+    for (auto &[layer, tris] : triangles) {
+        for (auto [tri, tri_info] : tris) {
+            tri.color2 = 0;
+        }
+    }
+    request_push();
+}
 
-            if (layers.count(layer)) {
-                auto &idxs = layers[layer];
-                assert(idxs.second == idx - 1);
-                idxs.second = idx;
-            }
-            else {
-                auto &idxs = layers[layer];
-                idxs.first = idx;
-                idxs.second = idx;
-            }
+void Canvas::set_color2(const ObjectRef &r, uint8_t color)
+{
+    if (!object_refs.count(r))
+        return;
+    for (const auto &[layer, range] : object_refs.at(r)) {
+        const auto [first, last] = range;
+        for (auto i = first; i <= last; i++) {
+            triangles.at(layer).atm(i).first.color2 = color;
+        }
+    }
+}
+
+void Canvas::add_triangle(int layer, const Coordf &p0, const Coordf &p1, const Coordf &p2, ColorP color, uint8_t flags,
+                          uint8_t color2)
+{
+    if (group_tris) {
+        assert(group_layer == layer);
+        group_tris->emplace_back(std::forward_as_tuple(p0, p1, p2, color, lod_current, color2),
+                                 std::forward_as_tuple(triangle_type_current, flags));
+        return;
+    }
+    triangles[layer].emplace_back(std::forward_as_tuple(p0, p1, p2, color, lod_current, color2),
+                                  std::forward_as_tuple(triangle_type_current, flags));
+    auto idx = triangles[layer].size() - 1;
+    for (auto it : object_ref_idx) {
+        auto &layers = *it;
+        if (layers.count(layer)) {
+            auto &idxs = layers[layer];
+            assert(idxs.second == idx - 1);
+            idxs.second = idx;
+        }
+        else {
+            auto &idxs = layers[layer];
+            idxs.first = idx;
+            idxs.second = idx;
         }
     }
 }
@@ -231,6 +262,18 @@ int Canvas::get_overlay_layer(int layer, bool ignore_flip)
     return overlay_layers.at({layer, ignore_flip});
 }
 
+bool Canvas::is_overlay_layer(int overlay_layer, int layer) const
+{
+    for (const bool ignore_flip : {true, false}) {
+        const auto k = std::make_pair(layer, ignore_flip);
+        if (overlay_layers.count(k)) {
+            if (overlay_layers.at(k) == overlay_layer)
+                return true;
+        }
+    }
+    return false;
+}
+
 
 Color Canvas::get_layer_color(int layer) const
 {
@@ -245,16 +288,61 @@ void Canvas::set_layer_color(int layer, const Color &color)
     layer_colors[layer] = color;
 }
 
+void Canvas::object_ref_pop()
+{
+    object_refs_current.pop_back();
+    object_ref_idx.pop_back();
+    assert(group_tris == nullptr);
+}
+
+void Canvas::object_ref_push(const ObjectRef &ref)
+{
+    object_refs_current.push_back(ref);
+    object_ref_idx.push_back(&object_refs[ref]);
+    assert(group_tris == nullptr);
+}
+
+void Canvas::begin_group(int layer)
+{
+    group_layer = layer;
+    assert(group_tris == nullptr);
+    group_tris = &triangles[layer];
+    group_size = group_tris->size();
+}
+
+void Canvas::end_group()
+{
+    auto new_size = group_tris->size();
+    for (auto it : object_ref_idx) {
+        auto &layers = *it;
+        if (layers.count(group_layer)) {
+            auto &idxs = layers[group_layer];
+            idxs.second = new_size - 1;
+        }
+        else {
+            auto &idxs = layers[group_layer];
+            idxs.first = group_size;
+            idxs.second = new_size - 1;
+        }
+    }
+    group_tris = nullptr;
+}
+
 std::pair<Coordf, Coordf> Canvas::get_bbox(bool visible_only) const
 {
     Coordf a, b;
     for (const auto &it : triangles) {
         if (visible_only == false || get_layer_display(it.first).visible) {
-            for (const auto &it2 : it.second) {
-                if (it2.flags & Triangle::FLAG_GLYPH)
+            for (const auto &[it2, it_info] : it.second) {
+                if (it_info.flags & TriangleInfo::FLAG_GLYPH)
                     continue;
                 std::vector<Coordf> points = {Coordf(it2.x0, it2.y0), Coordf(it2.x1, it2.y2), Coordf(it2.x1, it2.y2)};
-                if (std::isnan(it2.y2)) { // line
+                if (std::isnan(it2.y1) && std::isnan(it2.x2) && std::isnan(it2.y2)) { // circle
+                    auto r = Coordf(it2.x1, it2.x1);
+                    a = Coordf::min(a, points.at(0) - r);
+                    b = Coordf::max(b, points.at(0) + r);
+                }
+                else if (std::isnan(it2.y2)) { // line
                     float width = it2.x2;
                     Coordf offset(width / 2, width / 2);
                     a = Coordf::min(a, points.at(0) - offset);
