@@ -3,9 +3,9 @@
 #include "common/text.hpp"
 #include "layer_display.hpp"
 #include "selectables.hpp"
-#include "selection_filter.hpp"
 #include "target.hpp"
 #include "triangle.hpp"
+#include "object_ref.hpp"
 #include "fragment_cache.hpp"
 #include "util/placement.hpp"
 #include "util/text_data.hpp"
@@ -15,6 +15,8 @@
 #include <unordered_map>
 #include <deque>
 #include <list>
+#include "picture.hpp"
+#include "util/vector_pair.hpp"
 
 namespace horizon {
 class Canvas {
@@ -44,6 +46,9 @@ public:
     void set_flags(const ObjectRef &r, uint8_t mask_set, uint8_t mask_clear);
     void set_flags_all(uint8_t mask_set, uint8_t mask_clear);
 
+    void reset_color2();
+    void set_color2(const ObjectRef &r, uint8_t color);
+
     void show_all_obj();
 
     virtual void update_markers()
@@ -52,15 +57,13 @@ public:
 
     const LayerDisplay &get_layer_display(int index) const;
     void set_layer_display(int index, const LayerDisplay &ld);
-    class SelectionFilter selection_filter;
-
     void set_layer_color(int layer, const Color &color);
 
     bool layer_is_visible(int layer) const;
 
     bool show_all_junctions_in_schematic = false;
     bool show_text_in_tracks = false;
-    bool fast_draw = false;
+    bool show_text_in_vias = false;
 
     virtual bool get_flip_view() const
     {
@@ -69,19 +72,27 @@ public:
 
     std::pair<Coordf, Coordf> get_bbox(bool visible_only = true) const;
 
-    void set_airwire_filter(class IAirwireFilter *f)
-    {
-        airwire_filter = f;
-    }
-
     static const int first_overlay_layer = 30000;
 
 protected:
-    std::map<int, std::vector<Triangle>> triangles;
-    void add_triangle(int layer, const Coordf &p0, const Coordf &p1, const Coordf &p2, ColorP co, uint8_t flg = 0);
+    std::map<int, vector_pair<Triangle, TriangleInfo>> triangles;
+    std::list<CanvasPicture> pictures;
+    void add_triangle(int layer, const Coordf &p0, const Coordf &p1, const Coordf &p2, ColorP co, uint8_t flg = 0,
+                      uint8_t color2 = 0);
 
-    std::map<ObjectRef, std::map<int, std::pair<size_t, size_t>>> object_refs;
+    using ObjectRefIdx = std::map<int, std::pair<size_t, size_t>>;
+    std::unordered_map<ObjectRef, ObjectRefIdx> object_refs;
+    void begin_group(int layer);
+    void end_group();
     std::vector<ObjectRef> object_refs_current;
+    std::vector<ObjectRefIdx *> object_ref_idx;
+    void object_ref_push(const ObjectRef &ref);
+    template <typename... Args> void object_ref_push(Args... args)
+    {
+        object_ref_push(ObjectRef(args...));
+    }
+    void object_ref_pop();
+
     void render(const class Symbol &sym, bool on_sheet = false, bool smashed = false, ColorP co = ColorP::FROM_LAYER);
     void render(const class Junction &junc, bool interactive = true, ObjectType mode = ObjectType::INVALID);
     void render(const class Line &line, bool interactive = true, ColorP co = ColorP::FROM_LAYER);
@@ -115,8 +126,8 @@ protected:
     void render(const class Dimension &dim);
     void render(const class Frame &frame, bool on_sheet = false);
     void render(const class ConnectionLine &line);
-    void render(const class Airwire &airwire);
     void render(const class BoardPanel &panel);
+    void render(const class Picture &pic);
 
     bool needs_push = true;
     virtual void request_push() = 0;
@@ -212,16 +223,20 @@ protected:
 
     UUID sheet_current_uuid;
 
-    Triangle::Type triangle_type_current = Triangle::Type::NONE;
+    TriangleInfo::Type triangle_type_current = TriangleInfo::Type::NONE;
 
-    std::map<std::pair<int, bool>, int> overlay_layers;
+    std::map<std::pair<int, bool>, int> overlay_layers; // layer, ignore_flip -> overlay layer
     int overlay_layer_current = first_overlay_layer;
     int get_overlay_layer(int layer, bool ignore_flip = false);
+    bool is_overlay_layer(int overlay_layer, int layer) const;
 
     FragmentCache fragment_cache;
 
 private:
     uint8_t lod_current = 0;
-    class IAirwireFilter *airwire_filter = nullptr;
+
+    int group_layer = 0;
+    vector_pair<Triangle, TriangleInfo> *group_tris = nullptr;
+    size_t group_size = 0;
 };
 } // namespace horizon
