@@ -13,7 +13,7 @@ void CanvasCairo::request_push()
     cr->set_line_width(.05);
     cr->set_line_cap(Cairo::LINE_CAP_ROUND);
     for (const auto &it : triangles) {
-        for (const auto &it2 : it.second) {
+        for (const auto &it2 : it.second.first) {
             if (std::isnan(it2.y2)) {
                 cr->set_line_width(std::max(it2.x2 / 1e6, .05));
                 cr->move_to(it2.x0 / 1e6, it2.y0 / 1e6);
