@@ -20,12 +20,13 @@ void CanvasGL::set_scale_and_offset(float sc, Coordf ofs)
     scale = sc;
     offset = ofs;
     update_viewmat();
+    s_signal_scale_changed.emit();
 }
 
 CanvasGL::CanvasGL()
-    : Glib::ObjectBase(typeid(CanvasGL)), Canvas::Canvas(), markers(this), grid(this), drag_selection(this),
-      selectables_renderer(this, &selectables), triangle_renderer(this, triangles), marker_renderer(this, markers),
-      p_property_work_layer(*this, "work-layer"), p_property_grid_spacing(*this, "grid-spacing"),
+    : Glib::ObjectBase(typeid(CanvasGL)), Canvas::Canvas(), markers(*this), selection_filter(*this), grid(*this),
+      drag_selection(*this), selectables_renderer(*this, selectables), triangle_renderer(*this, triangles),
+      marker_renderer(*this, markers), picture_renderer(*this), p_property_work_layer(*this, "work-layer"),
       p_property_layer_opacity(*this, "layer-opacity")
 {
     add_events(Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK | Gdk::BUTTON_MOTION_MASK | Gdk::POINTER_MOTION_MASK
@@ -38,15 +39,11 @@ CanvasGL::CanvasGL()
         work_layer = property_work_layer();
         request_push();
     });
-    property_grid_spacing().signal_changed().connect([this] {
-        grid.spacing = property_grid_spacing();
-        queue_draw();
-    });
+
     property_layer_opacity() = 100;
     property_layer_opacity().signal_changed().connect([this] { queue_draw(); });
     clarify_menu = Gtk::manage(new Gtk::Menu);
 
-    update_palette_colors();
     layer_colors = appearance.layer_colors;
 
     gesture_zoom = Gtk::GestureZoom::create(*this);
@@ -61,6 +58,28 @@ CanvasGL::CanvasGL()
     gesture_drag->set_touch_only(true);
 }
 
+void CanvasGL::set_grid_spacing(uint64_t x, uint64_t y)
+{
+    grid.spacing = Coordi(x, y);
+    queue_draw();
+}
+
+void CanvasGL::set_grid_spacing(uint64_t s)
+{
+    set_grid_spacing(s, s);
+}
+
+void CanvasGL::set_grid_origin(const Coordi &c)
+{
+    grid.origin = c;
+    queue_draw();
+}
+
+Coordi CanvasGL::get_grid_spacing() const
+{
+    return grid.spacing;
+}
+
 void CanvasGL::zoom_gesture_begin_cb(GdkEventSequence *seq)
 {
     if (pan_dragging) {
@@ -83,6 +102,7 @@ void CanvasGL::zoom_gesture_update_cb(GdkEventSequence *seq)
     set_scale(cx, cy, gesture_zoom_scale_orig * delta);
     offset = gesture_zoom_offset_orig + Coordf(cx, cy) - gesture_zoom_pos_orig;
     update_viewmat();
+    s_signal_scale_changed.emit();
     queue_draw();
 }
 
@@ -149,6 +169,8 @@ void CanvasGL::on_realize()
     GL_CHECK_ERROR
     marker_renderer.realize();
     GL_CHECK_ERROR
+    picture_renderer.realize();
+    GL_CHECK_ERROR
 
     GLint fb;
     glGetIntegerv(GL_DRAW_FRAMEBUFFER_BINDING, &fb); // save fb
@@ -203,11 +225,14 @@ bool CanvasGL::on_render(const Glib::RefPtr<Gdk::GLContext> &context)
     GL_CHECK_ERROR
     glEnable(GL_BLEND);
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    picture_renderer.render(false);
+    GL_CHECK_ERROR
     grid.render();
     GL_CHECK_ERROR
     triangle_renderer.render();
     GL_CHECK_ERROR
-
+    picture_renderer.render(true);
+    GL_CHECK_ERROR
     selectables_renderer.render();
     GL_CHECK_ERROR
     drag_selection.render();
@@ -291,13 +316,12 @@ bool CanvasGL::on_scroll_event(GdkEventScroll *scroll_event)
     return Gtk::GLArea::on_scroll_event(scroll_event);
 }
 
-Coordi CanvasGL::snap_to_grid(const Coordi &c)
+Coordi CanvasGL::snap_to_grid(const Coordi &c, unsigned int div) const
 {
-    auto sp = grid.spacing;
-    int64_t xi = round_multiple(cursor_pos.x, sp);
-    int64_t yi = round_multiple(cursor_pos.y, sp);
-    Coordi t(xi, yi);
-    return t;
+    auto c2 = c - grid.origin;
+    int64_t xi = round_multiple(c2.x, grid.spacing.x / div);
+    int64_t yi = round_multiple(c2.y, grid.spacing.y / div);
+    return grid.origin + Coordi(xi, yi);
 }
 
 void CanvasGL::cursor_move(GdkEvent *motion_event)
@@ -312,19 +336,17 @@ void CanvasGL::cursor_move(GdkEvent *motion_event)
         return;
     }
 
-    auto sp = grid.spacing;
     GdkModifierType state;
     gdk_event_get_state(motion_event, &state);
+    unsigned int div = 1;
     if (state & grid_fine_modifier) {
-        sp /= 10;
+        div = 10;
     }
 
-    int64_t xi = round_multiple(cursor_pos.x, sp);
-    int64_t yi = round_multiple(cursor_pos.y, sp);
-    Coordi t(xi, yi);
+    Coordi t = snap_to_grid(Coordi(cursor_pos.x, cursor_pos.y), div);
 
     const auto &f = std::find_if(targets.begin(), targets.end(), [t, this](const auto &a) -> bool {
-        return a.p == t && this->layer_is_visible(a.layer);
+        return a.p == t && this->layer_is_visible(a.layer) && can_snap_to_target(a);
     });
     if (f != targets.end()) {
         target_current = *f;
@@ -334,57 +356,13 @@ void CanvasGL::cursor_move(GdkEvent *motion_event)
     }
 
     if (snap_to_targets) {
-        auto target_in_selection = [this](const Target &ta) {
-            if (ta.type == ObjectType::SYMBOL_PIN) {
-                SelectableRef key(ta.path.at(0), ObjectType::SCHEMATIC_SYMBOL, ta.vertex);
-                if (selectables.items_map.count(key)
-                    && (selectables.items.at(selectables.items_map.at(key))
-                                .get_flag(horizon::Selectable::Flag::SELECTED))) {
-                    return true;
-                }
-            }
-            else if (ta.type == ObjectType::PAD) {
-                SelectableRef key(ta.path.at(0), ObjectType::BOARD_PACKAGE, ta.vertex);
-                if (selectables.items_map.count(key)
-                    && (selectables.items.at(selectables.items_map.at(key))
-                                .get_flag(horizon::Selectable::Flag::SELECTED))) {
-                    return true;
-                }
-            }
-            else if (ta.type == ObjectType::POLYGON_EDGE) {
-                SelectableRef key(ta.path.at(0), ObjectType::POLYGON_VERTEX, ta.vertex);
-                if (selectables.items_map.count(key)
-                    && (selectables.items.at(selectables.items_map.at(key))
-                                .get_flag(horizon::Selectable::Flag::SELECTED))) {
-                    return true;
-                }
-            }
-            else if (ta.type == ObjectType::DIMENSION) {
-                for (int i = 0; i < 2; i++) {
-                    SelectableRef key(ta.path.at(0), ObjectType::DIMENSION, i);
-                    if (selectables.items_map.count(key)
-                        && (selectables.items.at(selectables.items_map.at(key))
-                                    .get_flag(horizon::Selectable::Flag::SELECTED))) {
-                        return true;
-                    }
-                }
-            }
-            SelectableRef key(ta.path.at(0), ta.type, ta.vertex);
-            if (selectables.items_map.count(key)
-                && (selectables.items.at(selectables.items_map.at(key))
-                            .get_flag(horizon::Selectable::Flag::SELECTED))) {
-                return true;
-            }
-            return false;
-        };
-
-        auto dfn = [this, target_in_selection](const Target &ta) -> float {
+        auto dfn = [this](const Target &ta) -> float {
             // return inf if target in selection and tool active (selection not
             // allowed)
             if (!layer_is_visible(ta.layer))
                 return INFINITY;
 
-            if (!selection_allowed && target_in_selection(ta))
+            if (!selection_allowed && !can_snap_to_target(ta))
                 return INFINITY;
             else
                 return (cursor_pos - (Coordf)ta.p).mag_sq();
@@ -394,7 +372,7 @@ void CanvasGL::cursor_move(GdkEvent *motion_event)
                                    [dfn](const auto &a, const auto &b) { return dfn(a) < dfn(b); });
         if (mi != targets.cend()) {
             auto d = sqrt(dfn(*mi));
-            if (d < 30 / scale) {
+            if (d < appearance.snap_radius / scale) {
                 target_current = *mi;
                 t = mi->p;
             }
@@ -431,6 +409,7 @@ void CanvasGL::center_and_zoom(const Coordi &center, float sc)
     offset.x = -((center.x * (flip_view ? -1 : 1) * scale) - m_width / 2);
     offset.y = -((center.y * -scale) - m_height / 2);
     update_viewmat();
+    s_signal_scale_changed.emit();
     queue_draw();
 }
 
@@ -443,9 +422,27 @@ void CanvasGL::zoom_to_bbox(const Coordf &a, const Coordf &b)
     offset.x = -((center.x * (flip_view ? -1 : 1) * scale) - m_width / 2);
     offset.y = -((center.y * -scale) - m_height / 2);
     update_viewmat();
+    s_signal_scale_changed.emit();
     queue_draw();
 }
 
+void CanvasGL::zoom_to_bbox(const std::pair<Coordf, Coordf> &bb)
+{
+    zoom_to_bbox(bb.first, bb.second);
+}
+
+
+void CanvasGL::ensure_min_size(float w, float h)
+{
+    auto sc_x = m_width / w;
+    auto sc_y = m_height / h;
+    auto sc = std::min(sc_x, sc_y);
+    if (sc < scale) {
+        auto p = get_cursor_pos_win();
+        set_scale(p.x, p.y, sc);
+    }
+}
+
 void CanvasGL::update_viewmat()
 {
     auto scale_x = scale;
@@ -467,6 +464,7 @@ void CanvasGL::push()
         marker_renderer.push();
     if (push_filter & PF_DRAG_SELECTION)
         drag_selection.push();
+    picture_renderer.push();
     push_filter = PF_NONE;
     GL_CHECK_ERROR
 }
@@ -523,6 +521,7 @@ void CanvasGL::set_selection(const std::set<SelectableRef> &sel, bool emit)
             selectables.items.at(selectables.items_map.at(key)).set_flag(horizon::Selectable::Flag::SELECTED, true);
         }
     }
+    selectables.update_preview(sel);
     if (emit)
         s_signal_selection_changed.emit();
     request_push(PF_SELECTABLES);
@@ -543,15 +542,36 @@ void CanvasGL::select_all()
 
 std::set<SelectableRef> CanvasGL::get_selection_at(const Coordi &c)
 {
-    std::set<SelectableRef> in_selection;
+
+    std::list<std::pair<const Selectable &, const SelectableRef &>> sel;
     unsigned int i = 0;
     for (auto &it : selectables.items) {
-        if (it.inside(c, 10 / scale) && selection_filter.can_select(selectables.items_ref[i])) {
-            in_selection.insert(selectables.items_ref[i]);
+        if (it.inside(c, appearance.min_selectable_size / scale)
+            && selection_filter.can_select(selectables.items_ref[i])) {
+            // in_selection.insert(selectables.items_ref[i]);
+            sel.emplace_back(it, selectables.items_ref.at(i));
         }
         i++;
     }
-    return in_selection;
+    auto n_point = std::count_if(sel.begin(), sel.end(), [](const auto &x) { return x.first.is_point(); });
+    auto n_line = std::count_if(sel.begin(), sel.end(), [](const auto &x) { return x.first.is_line(); });
+    if (n_point == 1) { // only one point, select that one
+        auto r = std::find_if(sel.begin(), sel.end(), [](const auto &x) { return x.first.is_point(); });
+        assert(r != sel.end());
+        return {r->second};
+    }
+    else if (n_line == 1) { // only one line, select that one
+        auto r = std::find_if(sel.begin(), sel.end(), [](const auto &x) { return x.first.is_line(); });
+        assert(r != sel.end());
+        return {r->second};
+    }
+    else {
+        std::set<SelectableRef> in_selection;
+        for (const auto &[s, sr] : sel) {
+            in_selection.emplace(sr);
+        }
+        return in_selection;
+    }
 }
 
 void CanvasGL::inhibit_drag_selection()
@@ -588,10 +608,15 @@ void CanvasGL::set_highlight_on_top(bool v)
     queue_draw();
 }
 
+void CanvasGL::set_layer_mode(CanvasGL::LayerMode mode)
+{
+    layer_mode = mode;
+    queue_draw();
+}
+
 void CanvasGL::set_appearance(const Appearance &a)
 {
     appearance = a;
-    update_palette_colors();
     layer_colors = appearance.layer_colors;
     switch (a.grid_fine_modifier) {
     case Appearance::GridFineModifier::ALT:
@@ -603,7 +628,7 @@ void CanvasGL::set_appearance(const Appearance &a)
     }
     switch (a.grid_style) {
     case Appearance::GridStyle::CROSS:
-        grid.mark_size = 5;
+        grid.mark_size = 5.5;
         break;
     case Appearance::GridStyle::DOT:
         grid.mark_size = 1;
@@ -640,16 +665,6 @@ void CanvasGL::set_cursor_size(Appearance::CursorSize csize)
     set_cursor_size(size);
 }
 
-void CanvasGL::update_palette_colors()
-{
-    for (unsigned int i = 0; i < palette_colors.size(); i++) {
-        auto c = get_color(static_cast<ColorP>(i));
-        palette_colors[i][0] = c.r;
-        palette_colors[i][1] = c.g;
-        palette_colors[i][2] = c.b;
-    }
-}
-
 const Color default_color(1, 0, 1);
 
 const Color &CanvasGL::get_color(ColorP colorp) const
@@ -703,6 +718,38 @@ void CanvasGL::clear()
     request_push();
 }
 
+bool CanvasGL::can_snap_to_target(const Target &t) const
+{
+    SnapFilter k(t.type, t.path.at(0), t.vertex);
+    if (snap_filter.count(k))
+        return false;
+    k.vertex = -1;
+    if (snap_filter.count(k))
+        return false;
+    if (t.type == ObjectType::POLYGON_ARC_CENTER || t.type == ObjectType::POLYGON_EDGE
+        || t.type == ObjectType::POLYGON_VERTEX) {
+        k.type = ObjectType::POLYGON;
+        if (snap_filter.count(k))
+            return false;
+    }
+    return true;
+}
+
+bool CanvasGL::layer_is_visible(int layer) const
+{
+    if (layer_mode == LayerMode::AS_IS)
+        return Canvas::layer_is_visible(layer);
+    else
+        return layer == work_layer || layer >= 10000;
+}
+
+void CanvasGL::set_colors2(const std::vector<ColorI> &c)
+{
+    colors2 = c;
+    request_push();
+}
+
+
 static const float char_space = 1;
 
 void CanvasGL::draw_bitmap_text(const Coordf &p, float sc, const std::string &rtext, int angle, ColorP color, int layer)
@@ -712,6 +759,7 @@ void CanvasGL::draw_bitmap_text(const Coordf &p, float sc, const std::string &rt
     Coordf point = p;
     Placement tr;
     tr.set_angle(angle);
+    Coordf v = tr.transform(Coordf(1, 0));
     for (auto codepoint : text) {
         if (codepoint != ' ') {
             auto info = bitmap_font::get_glyph_info(codepoint);
@@ -730,14 +778,13 @@ void CanvasGL::draw_bitmap_text(const Coordf &p, float sc, const std::string &rt
             auto fl = reinterpret_cast<const float *>(&bits);
 
             Coordf shift(info.minx, info.miny);
-            Coordf p1(glyph_w * 1e6 * sc, 0);
 
-            add_triangle(layer, point + tr.transform(shift) * 1e6 * sc, tr.transform(p1), Coordf(aspect, *fl), color,
-                         Triangle::FLAG_GLYPH);
-            point += tr.transform(Coordf(info.advance * char_space * 1e6 * sc, 0));
+            add_triangle(layer, point + tr.transform(shift) * 1e6 * sc, v * (glyph_w * 1e6 * sc), Coordf(aspect, *fl),
+                         color, TriangleInfo::FLAG_GLYPH);
+            point += v * (info.advance * char_space * 1e6 * sc);
         }
         else {
-            point += tr.transform(Coordf(7 * char_space * 1e6 * sc, 0));
+            point += v * (7 * char_space * 1e6 * sc);
         }
     }
 }
@@ -805,6 +852,7 @@ void CanvasGL::draw_bitmap_text_box(const Placement &q, float width, float heigh
 
     text_pos.x += width / 2 - text_width / 2;
 
+    begin_group(layer);
     if (p.get_angle() > 16384 && p.get_angle() <= 49152) {
         text_pos.x *= -1;
         text_pos.y *= -1;
@@ -814,6 +862,7 @@ void CanvasGL::draw_bitmap_text_box(const Placement &q, float width, float heigh
 
         draw_bitmap_text(p.transform(text_pos), sc, s, p.get_angle(), color, layer);
     }
+    end_group();
 };
 
 // copied from
