@@ -3,13 +3,16 @@
 #include "drag_selection.hpp"
 #include "grid.hpp"
 #include "marker.hpp"
-#include "triangle.hpp"
+#include "triangle_renderer.hpp"
 #include "selectables_renderer.hpp"
 #include "util/msd_animator.hpp"
 #include <gtkmm.h>
 #include <glm/glm.hpp>
 #include "appearance.hpp"
 #include "annotation.hpp"
+#include "snap_filter.hpp"
+#include "picture_renderer.hpp"
+#include "selection_filter.hpp"
 
 namespace horizon {
 class CanvasGL : public Canvas, public Gtk::GLArea {
@@ -20,6 +23,7 @@ class CanvasGL : public Canvas, public Gtk::GLArea {
     friend MarkerRenderer;
     friend Markers;
     friend CanvasAnnotation;
+    friend PictureRenderer;
 
 public:
     CanvasGL();
@@ -45,6 +49,9 @@ public:
     void set_highlight_enabled(bool x);
     void set_highlight_on_top(bool on_top);
 
+    enum class LayerMode { AS_IS, WORK_ONLY, SHADOW_OTHER };
+    void set_layer_mode(LayerMode mode);
+
     std::set<SelectableRef> get_selection();
     void set_selection(const std::set<SelectableRef> &sel, bool emit = true);
     void select_all();
@@ -56,7 +63,7 @@ public:
     void set_selection_allowed(bool a);
     std::pair<float, Coordf> get_scale_and_offset();
     void set_scale_and_offset(float sc, Coordf ofs);
-    Coordi snap_to_grid(const Coordi &c);
+    Coordi snap_to_grid(const Coordi &c, unsigned int div = 1) const;
 
     void set_flip_view(bool fl);
     bool get_flip_view() const override;
@@ -105,21 +112,29 @@ public:
         return s_signal_can_steal_focus;
     }
 
+    typedef sigc::signal<void> type_signal_scale_changed;
+    type_signal_scale_changed signal_scale_changed()
+    {
+        return s_signal_scale_changed;
+    }
+
     void center_and_zoom(const Coordi &center, float scale = -1);
     void zoom_to_bbox(const Coordf &a, const Coordf &b);
+    void zoom_to_bbox(const std::pair<Coordf, Coordf> &bb);
+    void ensure_min_size(float w, float h);
 
     Glib::PropertyProxy<int> property_work_layer()
     {
         return p_property_work_layer.get_proxy();
     }
-    Glib::PropertyProxy<uint64_t> property_grid_spacing()
-    {
-        return p_property_grid_spacing.get_proxy();
-    }
     Glib::PropertyProxy<float> property_layer_opacity()
     {
         return p_property_layer_opacity.get_proxy();
     }
+    Glib::PropertyProxy_ReadOnly<float> property_layer_opacity() const
+    {
+        return Glib::PropertyProxy_ReadOnly<float>(this, "layer-opacity");
+    }
     Markers markers;
     void update_markers() override;
 
@@ -153,6 +168,19 @@ public:
     void remove_annotation(CanvasAnnotation *a);
     bool layer_is_annotation(int l) const;
 
+    std::set<SnapFilter> snap_filter;
+
+    void set_grid_spacing(uint64_t x, uint64_t y);
+    void set_grid_spacing(uint64_t s);
+    Coordi get_grid_spacing() const;
+    void set_grid_origin(const Coordi &c);
+
+    SelectionFilter selection_filter;
+
+    bool layer_is_visible(int layer) const;
+
+    void set_colors2(const std::vector<ColorI> &c);
+
 protected:
     void push() override;
     void request_push() override;
@@ -175,7 +203,6 @@ private:
     Coord<float> cursor_pos;
     Coord<int64_t> cursor_pos_grid;
     bool cursor_external = false;
-    bool warped = false;
 
     GLuint renderbuffer;
     GLuint stencilrenderbuffer;
@@ -202,6 +229,8 @@ private:
 
     MarkerRenderer marker_renderer;
 
+    PictureRenderer picture_renderer;
+
     void pan_drag_begin(GdkEventButton *button_event);
     void pan_drag_end(GdkEventButton *button_event);
     void pan_drag_move(GdkEventMotion *motion_event);
@@ -217,7 +246,6 @@ private:
 
     bool selection_allowed = true;
     Glib::Property<int> p_property_work_layer;
-    Glib::Property<uint64_t> p_property_grid_spacing;
     Glib::Property<float> p_property_layer_opacity;
 
     Gtk::Menu *clarify_menu;
@@ -227,8 +255,7 @@ private:
     bool highlight_on_top = false;
     Appearance appearance;
 
-    void update_palette_colors();
-    std::array<std::array<float, 4>, static_cast<size_t>(ColorP::N_COLORS)> palette_colors;
+    LayerMode layer_mode = LayerMode::AS_IS;
 
     bool drag_selection_inhibited = false;
 
@@ -264,6 +291,10 @@ private:
     void drag_gesture_begin_cb(GdkEventSequence *seq);
     void drag_gesture_update_cb(GdkEventSequence *seq);
 
+    bool can_snap_to_target(const Target &t) const;
+
+    std::vector<ColorI> colors2;
+
 protected:
     void on_size_allocate(Gtk::Allocation &alloc) override;
     void on_realize() override;
@@ -281,5 +312,6 @@ protected:
     type_signal_grid_mul_changed s_signal_grid_mul_changed;
     type_signal_request_display_name s_signal_request_display_name;
     type_signal_can_steal_focus s_signal_can_steal_focus;
+    type_signal_scale_changed s_signal_scale_changed;
 };
 } // namespace horizon
