@@ -7,7 +7,7 @@
 #include <glm/gtc/type_ptr.hpp>
 
 namespace horizon {
-DragSelection::DragSelection(class CanvasGL *c) : ca(c), active(0), box(this), line(this)
+DragSelection::DragSelection(class CanvasGL &c) : ca(c), active(0), box(ca), line(ca)
 {
 }
 
@@ -107,13 +107,13 @@ void DragSelection::Box::render()
 {
     glUseProgram(program);
     glBindVertexArray(vao);
-    glUniformMatrix3fv(screenmat_loc, 1, GL_FALSE, glm::value_ptr(parent->ca->screenmat));
-    glUniformMatrix3fv(viewmat_loc, 1, GL_FALSE, glm::value_ptr(parent->ca->viewmat));
-    glUniform1f(scale_loc, parent->ca->scale);
+    glUniformMatrix3fv(screenmat_loc, 1, GL_FALSE, glm::value_ptr(ca.screenmat));
+    glUniformMatrix3fv(viewmat_loc, 1, GL_FALSE, glm::value_ptr(ca.viewmat));
+    glUniform1f(scale_loc, ca.scale);
     glUniform2f(a_loc, sel_a.x, sel_a.y);
     glUniform2f(b_loc, sel_b.x, sel_b.y);
     glUniform1i(fill_loc, fill);
-    auto co = ca->get_color(ColorP::SELECTION_BOX);
+    auto co = ca.get_color(ColorP::SELECTION_BOX);
     gl_color_to_uniform_3f(color_loc, co);
 
     glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
@@ -126,10 +126,10 @@ void DragSelection::Line::render()
 {
     glUseProgram(program);
     glBindVertexArray(vao);
-    glUniformMatrix3fv(screenmat_loc, 1, GL_FALSE, glm::value_ptr(parent->ca->screenmat));
-    glUniformMatrix3fv(viewmat_loc, 1, GL_FALSE, glm::value_ptr(parent->ca->viewmat));
-    glUniform1f(scale_loc, parent->ca->scale);
-    auto co = ca->get_color(ColorP::SELECTION_LINE);
+    glUniformMatrix3fv(screenmat_loc, 1, GL_FALSE, glm::value_ptr(ca.screenmat));
+    glUniformMatrix3fv(viewmat_loc, 1, GL_FALSE, glm::value_ptr(ca.viewmat));
+    glUniform1f(scale_loc, ca.scale);
+    auto co = ca.get_color(ColorP::SELECTION_LINE);
     gl_color_to_uniform_3f(color_loc, co);
 
 
@@ -145,7 +145,7 @@ void DragSelection::Line::push()
         return;
     glBindBuffer(GL_ARRAY_BUFFER, vbo);
     auto n_vertices = vertices.size();
-    if (parent->ca->selection_tool == CanvasGL::SelectionTool::PAINT) {
+    if (ca.selection_tool == CanvasGL::SelectionTool::PAINT) {
         glBufferData(GL_ARRAY_BUFFER, sizeof(Vertex) * n_vertices, vertices.data(), GL_STREAM_DRAW);
     }
     else {
@@ -171,10 +171,10 @@ void DragSelection::render()
 {
     if (active != 2)
         return;
-    if (ca->selection_tool == CanvasGL::SelectionTool::BOX)
+    if (ca.selection_tool == CanvasGL::SelectionTool::BOX)
         box.render();
 
-    if (is_line_sel(ca->selection_tool))
+    if (is_line_sel(ca.selection_tool))
         line.render();
 }
 
@@ -182,11 +182,11 @@ void DragSelection::drag_begin(GdkEventButton *button_event)
 {
     if (button_event->state & Gdk::SHIFT_MASK)
         return;
-    if (!ca->selection_allowed)
+    if (!ca.selection_allowed)
         return;
     if (button_event->type == GDK_2BUTTON_PRESS) {
         active = 0;
-        ca->drag_selection_inhibited = false;
+        ca.drag_selection_inhibited = false;
         return;
     }
     gdouble x, y;
@@ -194,18 +194,18 @@ void DragSelection::drag_begin(GdkEventButton *button_event)
     if (button_event->button == 1) { // inside of grid and middle mouse button
         active = 1;
         sel_o = Coordf(x, y);
-        if (!is_line_sel(ca->selection_tool)) {
-            box.sel_a = ca->screen2canvas(sel_o);
+        if (!is_line_sel(ca.selection_tool)) {
+            box.sel_a = ca.screen2canvas(sel_o);
             box.sel_b = box.sel_a;
         }
         else {
             line.vertices.clear();
-            auto c = ca->screen2canvas(sel_o);
+            auto c = ca.screen2canvas(sel_o);
             line.vertices.emplace_back(c.x, c.y);
             line.path.clear();
             line.path.emplace_back(c.x, c.y);
         }
-        ca->queue_draw();
+        ca.queue_draw();
     }
 }
 
@@ -213,39 +213,39 @@ void DragSelection::drag_move(GdkEventMotion *motion_event)
 {
     gdouble x, y;
     gdk_event_get_coords((GdkEvent *)motion_event, &x, &y);
-    if (ca->drag_selection_inhibited && active) {
+    if (ca.drag_selection_inhibited && active) {
         active = 0;
         return;
     }
 
     if (active == 1) {
-        if (is_line_sel(ca->selection_tool)) {
+        if (is_line_sel(ca.selection_tool)) {
             if (ABS(sel_o.x - x) > 10 || ABS(sel_o.y - y) > 10) {
                 active = 2;
-                ca->set_selection_mode(CanvasGL::SelectionMode::NORMAL);
+                ca.set_selection_mode(CanvasGL::SelectionMode::NORMAL);
             }
         }
         else {
             if (ABS(sel_o.x - x) > 10 && ABS(sel_o.y - y) > 10) {
                 active = 2;
-                ca->set_selection_mode(CanvasGL::SelectionMode::NORMAL);
+                ca.set_selection_mode(CanvasGL::SelectionMode::NORMAL);
             }
         }
     }
     else if (active == 2) {
-        if (!is_line_sel(ca->selection_tool)) {
-            box.sel_b = ca->screen2canvas(Coordf(x, y));
+        if (!is_line_sel(ca.selection_tool)) {
+            box.sel_b = ca.screen2canvas(Coordf(x, y));
             box.update();
         }
         else {
-            auto c = ca->screen2canvas(Coordf(x, y));
+            auto c = ca.screen2canvas(Coordf(x, y));
             line.vertices.emplace_back(c.x, c.y);
             line.path.emplace_back(c.x, c.y);
             line.update();
-            ca->request_push(CanvasGL::PF_DRAG_SELECTION);
-            ca->request_push(CanvasGL::PF_SELECTABLES);
+            ca.request_push(CanvasGL::PF_DRAG_SELECTION);
+            ca.request_push(CanvasGL::PF_SELECTABLES);
         }
-        ca->queue_draw();
+        ca.queue_draw();
     }
 }
 
@@ -254,7 +254,7 @@ void DragSelection::drag_end(GdkEventButton *button_event)
     if (button_event->button == 1) { // inside of grid and middle mouse button {
         bool toggle = button_event->state & Gdk::CONTROL_MASK;
         if (active == 2) {
-            for (auto &it : ca->selectables.items) {
+            for (auto &it : ca.selectables.items) {
                 if (it.get_flag(Selectable::Flag::PRELIGHT)) {
                     if (toggle)
                         it.set_flag(Selectable::Flag::SELECTED, !it.get_flag(Selectable::Flag::SELECTED));
@@ -267,50 +267,43 @@ void DragSelection::drag_end(GdkEventButton *button_event)
                 }
                 it.set_flag(Selectable::Flag::PRELIGHT, false);
             }
-            ca->request_push(CanvasGL::PF_DRAG_SELECTION);
-            ca->request_push(CanvasGL::PF_SELECTABLES);
-            ca->s_signal_selection_changed.emit();
+            ca.selectables.update_preview(ca.get_selection());
+            ca.request_push(CanvasGL::PF_DRAG_SELECTION);
+            ca.request_push(CanvasGL::PF_SELECTABLES);
+            ca.s_signal_selection_changed.emit();
         }
         else if (active == 1) {
             std::cout << "click select" << std::endl;
-            if (ca->selection_mode == CanvasGL::SelectionMode::HOVER) { // just select what was
-                                                                        // selecte by hover select
-                ca->set_selection_mode(CanvasGL::SelectionMode::NORMAL);
-                ca->s_signal_selection_changed.emit();
+            if (ca.selection_mode == CanvasGL::SelectionMode::HOVER) { // just select what was
+                                                                       // selecte by hover select
+                ca.set_selection_mode(CanvasGL::SelectionMode::NORMAL);
+                ca.selectables.update_preview(ca.get_selection());
+                ca.s_signal_selection_changed.emit();
             }
             else {
                 std::set<SelectableRef> selection;
-                selection = ca->get_selection();
+                selection = ca.get_selection();
                 gdouble x, y;
                 gdk_event_get_coords((GdkEvent *)button_event, &x, &y);
-                auto c = ca->screen2canvas({(float)x, (float)y});
-                std::vector<unsigned int> in_selection;
-                {
-                    unsigned int i = 0;
-                    for (auto &it : ca->selectables.items) {
-                        it.set_flag(Selectable::Flag::PRELIGHT, false);
-                        it.set_flag(Selectable::Flag::SELECTED, false);
-                        if (it.inside(c, 10 / ca->scale)
-                            && ca->selection_filter.can_select(ca->selectables.items_ref[i])) {
-                            in_selection.push_back(i);
-                        }
-                        i++;
-                    }
+                auto c = ca.screen2canvas({(float)x, (float)y});
+                for (auto &it : ca.selectables.items) {
+                    it.set_flag(Selectable::Flag::PRELIGHT, false);
+                    it.set_flag(Selectable::Flag::SELECTED, false);
                 }
-                if (in_selection.size() > 1) {
-                    ca->set_selection(selection, false);
-                    for (const auto it : ca->clarify_menu->get_children()) {
-                        ca->clarify_menu->remove(*it);
-                    }
-                    for (auto i : in_selection) {
-                        const auto sr = ca->selectables.items_ref[i];
+                auto sel_from_canvas = ca.get_selection_at(Coordi(c.x, c.y));
 
+                if (sel_from_canvas.size() > 1) {
+                    ca.set_selection(selection, false);
+                    for (const auto it : ca.clarify_menu->get_children()) {
+                        ca.clarify_menu->remove(*it);
+                    }
+                    for (const auto sr : sel_from_canvas) {
                         std::string text = object_descriptions.at(sr.type).name;
-                        auto display_name = ca->s_signal_request_display_name.emit(sr.type, sr.uuid);
+                        auto display_name = ca.s_signal_request_display_name.emit(sr.type, sr.uuid);
                         if (display_name.size()) {
                             text += " " + display_name;
                         }
-                        auto layers = ca->layer_provider->get_layers();
+                        auto layers = ca.layer_provider->get_layers();
                         if (layers.count(sr.layer)) {
                             text += " (" + layers.at(sr.layer).name + ")";
                         }
@@ -332,24 +325,24 @@ void DragSelection::drag_end(GdkEventButton *button_event)
                                 else {
                                     sel.insert(sr);
                                 }
-                                ca->set_selection(sel);
+                                ca.set_selection(sel);
                             }
                             else {
-                                ca->set_selection({sr}, false);
+                                ca.set_selection({sr}, false);
                             }
 #ifdef G_OS_WIN32 // work around a bug(?) in intel(?) GPU drivers on windows
-                            Glib::signal_idle().connect_once([this] { ca->queue_draw(); });
+                            Glib::signal_idle().connect_once([this] { ca.queue_draw(); });
 #endif
                         });
                         la->signal_deselect().connect([this, selection, toggle] {
                             if (toggle) {
-                                ca->set_selection(selection, false);
+                                ca.set_selection(selection, false);
                             }
                             else {
-                                ca->set_selection({}, false);
+                                ca.set_selection({}, false);
                             }
 #ifdef G_OS_WIN32 // work around a bug(?) in intel(?) GPU drivers on windows
-                            Glib::signal_idle().connect_once([this] { ca->queue_draw(); });
+                            Glib::signal_idle().connect_once([this] { ca.queue_draw(); });
 #endif
                         });
                         la->signal_activate().connect([this, sr, selection, toggle] {
@@ -361,23 +354,23 @@ void DragSelection::drag_end(GdkEventButton *button_event)
                                 else {
                                     sel.insert(sr);
                                 }
-                                ca->set_selection(sel);
+                                ca.set_selection(sel);
                             }
                             else {
-                                ca->set_selection({sr}, true);
+                                ca.set_selection({sr}, true);
                             }
                         });
                         la->show();
-                        ca->clarify_menu->append(*la);
+                        ca.clarify_menu->append(*la);
                     }
 #if GTK_CHECK_VERSION(3, 22, 0)
-                    ca->clarify_menu->popup_at_pointer((GdkEvent *)button_event);
+                    ca.clarify_menu->popup_at_pointer((GdkEvent *)button_event);
 #else
-                    ca->clarify_menu->popup(0, gtk_get_current_event_time());
+                    ca.clarify_menu->popup(0, gtk_get_current_event_time());
 #endif
                 }
-                else if (in_selection.size() == 1) {
-                    auto sel = ca->selectables.items_ref[in_selection.front()];
+                else if (sel_from_canvas.size() == 1) {
+                    auto sel = *sel_from_canvas.begin();
                     if (toggle) {
                         if (selection.count(sel)) {
                             selection.erase(sel);
@@ -385,23 +378,23 @@ void DragSelection::drag_end(GdkEventButton *button_event)
                         else {
                             selection.insert(sel);
                         }
-                        ca->set_selection(selection);
+                        ca.set_selection(selection);
                     }
                     else {
-                        ca->set_selection({sel});
+                        ca.set_selection({sel});
                     }
                 }
-                else if (in_selection.size() == 0) {
+                else if (sel_from_canvas.size() == 0) {
                     if (toggle)
-                        ca->set_selection(selection);
+                        ca.set_selection(selection);
                     else
-                        ca->set_selection({});
+                        ca.set_selection({});
                 }
             }
         }
         active = 0;
-        ca->queue_draw();
-        ca->drag_selection_inhibited = false;
+        ca.queue_draw();
+        ca.drag_selection_inhibited = false;
     }
 }
 
@@ -412,10 +405,10 @@ void DragSelection::Box::update()
     float ymin = std::min(sel_a.y, sel_b.y);
     float ymax = std::max(sel_a.y, sel_b.y);
     unsigned int i = 0;
-    for (auto &it : ca->selectables.items) {
+    for (auto &it : ca.selectables.items) {
         it.set_flag(Selectable::Flag::PRELIGHT, false);
-        if (ca->selection_filter.can_select(ca->selectables.items_ref[i])) {
-            auto sq = ca->selection_qualifier;
+        if (ca.selection_filter.can_select(ca.selectables.items_ref[i])) {
+            auto sq = ca.selection_qualifier;
 
             if (sq == CanvasGL::SelectionQualifier::AUTO) {
                 if (sel_a.x < sel_b.x)
@@ -468,17 +461,17 @@ void DragSelection::Box::update()
         }
         i++;
     }
-    ca->request_push(CanvasGL::PF_DRAG_SELECTION);
-    ca->request_push(CanvasGL::PF_SELECTABLES);
+    ca.request_push(CanvasGL::PF_DRAG_SELECTION);
+    ca.request_push(CanvasGL::PF_SELECTABLES);
 }
 
 void DragSelection::Line::update()
 {
     unsigned int i = 0;
-    for (auto &it : ca->selectables.items) {
+    for (auto &it : ca.selectables.items) {
         it.set_flag(Selectable::Flag::PRELIGHT, false);
-        if (ca->selection_filter.can_select(ca->selectables.items_ref[i])) {
-            if (ca->selection_tool == CanvasGL::SelectionTool::PAINT) {
+        if (ca.selection_filter.can_select(ca.selectables.items_ref[i])) {
+            if (ca.selection_tool == CanvasGL::SelectionTool::PAINT) {
                 ClipperLib::Path sel(4);
                 auto corners = it.get_corners();
                 for (size_t j = 0; j < 4; j++) {
@@ -497,15 +490,15 @@ void DragSelection::Line::update()
                 }
             }
 
-            else if (ca->selection_tool == CanvasGL::SelectionTool::LASSO) {
-                if (ca->selection_qualifier == CanvasGL::SelectionQualifier::INCLUDE_ORIGIN) {
+            else if (ca.selection_tool == CanvasGL::SelectionTool::LASSO) {
+                if (ca.selection_qualifier == CanvasGL::SelectionQualifier::INCLUDE_ORIGIN) {
                     ClipperLib::IntPoint pt(it.x, it.y);
                     if (ClipperLib::PointInPolygon(pt, path) != 0) {
                         it.set_flag(Selectable::Flag::PRELIGHT, true);
                     }
                 }
-                else if (ca->selection_qualifier == CanvasGL::SelectionQualifier::INCLUDE_BOX
-                         || ca->selection_qualifier == CanvasGL::SelectionQualifier::TOUCH_BOX) {
+                else if (ca.selection_qualifier == CanvasGL::SelectionQualifier::INCLUDE_BOX
+                         || ca.selection_qualifier == CanvasGL::SelectionQualifier::TOUCH_BOX) {
                     ClipperLib::Path sel(4);
                     auto corners = it.get_corners();
                     for (size_t j = 0; j < 4; j++) {
@@ -517,7 +510,7 @@ void DragSelection::Line::update()
                     clipper.AddPath(path, ClipperLib::ptClip, true);
 
                     ClipperLib::Paths isect;
-                    if (ca->selection_qualifier == CanvasGL::SelectionQualifier::INCLUDE_BOX) {
+                    if (ca.selection_qualifier == CanvasGL::SelectionQualifier::INCLUDE_BOX) {
                         clipper.Execute(ClipperLib::ctDifference, isect); // isect = sel-path
 
                         if (isect.size() == 0) {
@@ -536,7 +529,7 @@ void DragSelection::Line::update()
         }
         i++;
     }
-    ca->request_push(CanvasGL::PF_DRAG_SELECTION);
-    ca->request_push(CanvasGL::PF_SELECTABLES);
+    ca.request_push(CanvasGL::PF_DRAG_SELECTION);
+    ca.request_push(CanvasGL::PF_SELECTABLES);
 }
 } // namespace horizon
