@@ -7,15 +7,17 @@ class Grid {
     friend class CanvasGL;
 
 public:
-    Grid(class CanvasGL *c);
+    Grid(const class CanvasGL &c);
     void realize();
     void render();
     void render_cursor(Coord<int64_t> &coord);
     enum class Style { CROSS, DOT, GRID };
 
 private:
-    CanvasGL *ca;
-    int64_t spacing;
+    const CanvasGL &ca;
+    Coordi spacing;
+    Coordi origin;
+    unsigned int major = 0;
     float mark_size;
     unsigned int mul = 0;
 
