@@ -23,11 +23,10 @@
 #include "selectables.hpp"
 #include "selection_filter.hpp"
 #include "target.hpp"
-#include "triangle.hpp"
 #include "util/placement.hpp"
 #include "util/util.hpp"
-#include "iairwire_filter.hpp"
 #include "board/board_panel.hpp"
+#include "common/picture.hpp"
 #include <algorithm>
 #include <ctime>
 #include <sstream>
@@ -56,7 +55,7 @@ void Canvas::render(const Junction &junc, bool interactive, ObjectType mode)
 
     auto layer = layer_display.count(junc.layer) ? junc.layer : 10000;
 
-    object_refs_current.emplace_back(ObjectType::JUNCTION, junc.uuid);
+    object_ref_push(ObjectType::JUNCTION, junc.uuid);
     if (draw) {
         if (junc.connection_count == 2) {
             if (show_all_junctions_in_schematic)
@@ -73,13 +72,11 @@ void Canvas::render(const Junction &junc, bool interactive, ObjectType mode)
             draw_cross(junc.position, 0.25_mm, c);
         }
     }
-    object_refs_current.pop_back();
+    object_ref_pop();
 
     if (interactive) {
         selectables.append(junc.uuid, ObjectType::JUNCTION, junc.position, 0, layer);
-        if (!junc.temp) {
-            targets.emplace_back(junc.uuid, ObjectType::JUNCTION, transform.transform(junc.position), 0, layer);
-        }
+        targets.emplace_back(junc.uuid, ObjectType::JUNCTION, transform.transform(junc.position), 0, layer);
     }
 }
 
@@ -87,7 +84,7 @@ void Canvas::render(const PowerSymbol &sym)
 {
     auto c = ColorP::FROM_LAYER;
     transform.shift = sym.junction->position;
-    object_refs_current.emplace_back(ObjectType::POWER_SYMBOL, sym.uuid);
+    object_ref_push(ObjectType::POWER_SYMBOL, sym.uuid);
 
     auto style = sym.net->power_symbol_style;
     img_auto_line = img_mode;
@@ -184,27 +181,23 @@ void Canvas::render(const PowerSymbol &sym)
     transform.reset();
     img_auto_line = false;
 
-    object_refs_current.pop_back();
+    object_ref_pop();
 }
 
 ObjectRef Canvas::add_line(const std::deque<Coordi> &pts, int64_t width, ColorP color, int layer)
 {
+    if (pts.size() < 2) {
+        return ObjectRef();
+    }
     auto uu = UUID::random();
     ObjectRef sr(ObjectType::LINE, uu);
-    object_refs_current.push_back(sr);
-    triangle_type_current = Triangle::Type::TRACK_PREVIEW;
-    if (pts.size() >= 2) {
-        for (size_t i = 1; i < pts.size(); i++) {
-            auto pt1 = pts.at(i - 1);
-            auto pt2 = pts.at(i);
-            draw_line(pt1, pt2, color, layer, false, width);
-        }
+    object_ref_push(sr);
+    for (size_t i = 1; i < pts.size(); i++) {
+        auto pt1 = pts.at(i - 1);
+        auto pt2 = pts.at(i);
+        draw_line(pt1, pt2, color, layer, false, width);
     }
-    else {
-        object_refs[sr];
-    }
-    triangle_type_current = Triangle::Type::NONE;
-    object_refs_current.pop_back();
+    object_ref_pop();
     request_push();
     return sr;
 }
@@ -214,9 +207,9 @@ void Canvas::render(const Line &line, bool interactive, ColorP co)
     img_line(line.from->position, line.to->position, line.width, line.layer);
     if (img_mode)
         return;
-    triangle_type_current = Triangle::Type::GRAPHICS;
+    triangle_type_current = TriangleInfo::Type::GRAPHICS;
     draw_line(line.from->position, line.to->position, co, line.layer, true, line.width);
-    triangle_type_current = Triangle::Type::NONE;
+    triangle_type_current = TriangleInfo::Type::NONE;
     if (interactive) {
         selectables.append_line(line.uuid, ObjectType::LINE, line.from->position, line.to->position, line.width, 0,
                                 line.layer);
@@ -243,9 +236,9 @@ void Canvas::render(const LineNet &line)
     img_line(line.from.get_position(), line.to.get_position(), width, 0);
     if (img_mode)
         return;
-    object_refs_current.emplace_back(ObjectType::LINE_NET, line.uuid);
+    object_ref_push(ObjectType::LINE_NET, line.uuid);
     draw_line(line.from.get_position(), line.to.get_position(), c, 0, true, width);
-    object_refs_current.pop_back();
+    object_ref_pop();
     selectables.append_line(line.uuid, ObjectType::LINE_NET, line.from.get_position(), line.to.get_position(), width);
 }
 
@@ -267,20 +260,20 @@ void Canvas::render(const Track &track, bool interactive)
 
     auto center = (track.from.get_position() + track.to.get_position()) / 2;
     if (interactive)
-        object_refs_current.emplace_back(ObjectType::TRACK, track.uuid);
+        object_ref_push(ObjectType::TRACK, track.uuid);
     draw_line(track.from.get_position(), track.to.get_position(), c, layer, true, width);
     if (track.locked) {
         auto ol = get_overlay_layer(layer);
         draw_lock(center, 0.7 * track.width, ColorP::TEXT_OVERLAY, ol, true);
     }
     if (interactive && show_text_in_tracks && width > 0 && track.net && track.net->name.size() && track.from.is_junc()
-        && track.to.is_junc()) {
+        && track.to.is_junc() && (!show_text_in_vias || (!track.from.junc->has_via && !track.to.junc->has_via))) {
         auto overlay_layer = get_overlay_layer(track.layer, true);
         set_lod_size(width);
         auto vec = (track.from.get_position() - track.to.get_position());
         auto length = sqrt(vec.mag_sq());
         Placement p(center);
-        p.set_angle(atan2(vec.y, vec.x) * (32768 / M_PI));
+        p.set_angle_rad(atan2(vec.y, vec.x));
         if (get_flip_view()) {
             p.shift.x *= -1;
             p.invert_angle();
@@ -289,17 +282,12 @@ void Canvas::render(const Track &track, bool interactive)
         set_lod_size(-1);
     }
     if (interactive)
-        object_refs_current.pop_back();
+        object_ref_pop();
     if (interactive)
         selectables.append_line(track.uuid, ObjectType::TRACK, track.from.get_position(), track.to.get_position(),
                                 track.width, 0, track.layer);
 }
 
-void Canvas::render(const Airwire &airwire)
-{
-    draw_line(airwire.from.get_position(), airwire.to.get_position(), ColorP::AIRWIRE, 10000, true, 0);
-}
-
 void Canvas::render(const ConnectionLine &line)
 {
     if (img_mode)
@@ -594,9 +582,9 @@ static int64_t sq(int64_t x)
 
 void Canvas::render(const Arc &arc, bool interactive, ColorP co)
 {
-    Coordf a(arc.from->position);   // ,b,c;
-    Coordf b(arc.to->position);     // ,b,c;
-    Coordf c(arc.center->position); // ,b,c;
+    Coordf a(arc.from->position); // ,b,c;
+    Coordf b(arc.to->position);   // ,b,c;
+    Coordf c = project_onto_perp_bisector(a, b, arc.center->position);
     float radius0 = sqrt(sq(c.x - a.x) + sq(c.y - a.y));
     float radius1 = sqrt(sq(c.x - b.x) + sq(c.y - b.y));
     float a0 = atan2f(a.y - c.y, a.x - c.x);
@@ -610,9 +598,9 @@ void Canvas::render(const Arc &arc, bool interactive, ColorP co)
 void Canvas::render(const SchematicSymbol &sym)
 {
     transform = sym.placement;
-    object_refs_current.emplace_back(ObjectType::SCHEMATIC_SYMBOL, sym.uuid);
+    object_ref_push(ObjectType::SCHEMATIC_SYMBOL, sym.uuid);
     render(sym.symbol, true, sym.smashed, ColorP::FROM_LAYER);
-    object_refs_current.pop_back();
+    object_ref_pop();
     for (const auto &it : sym.symbol.pins) {
         targets.emplace_back(UUIDPath<2>(sym.uuid, it.second.uuid), ObjectType::SYMBOL_PIN,
                              transform.transform(it.second.position));
@@ -648,10 +636,10 @@ void Canvas::render(const Text &text, bool interactive, ColorP co)
     }
 
     img_patch_type(PatchType::TEXT);
-    triangle_type_current = Triangle::Type::TEXT;
+    triangle_type_current = TriangleInfo::Type::TEXT;
     auto extents = draw_text0(transform.shift, text.size, text.overridden ? text.text_override : text.text, angle, rev,
                               text.origin, co, text.layer, text.width, true, text.font, false, transform.mirror);
-    triangle_type_current = Triangle::Type::NONE;
+    triangle_type_current = TriangleInfo::Type::NONE;
     // img_text_extents(text, extents);
     img_patch_type(PatchType::OTHER);
     transform_restore();
@@ -690,7 +678,7 @@ void Canvas::render(const NetLabel &label)
         txt += " [" + join(label.on_sheets, ",") + "]";
     }
 
-    object_refs_current.emplace_back(ObjectType::NET_LABEL, label.uuid);
+    object_ref_push(ObjectType::NET_LABEL, label.uuid);
     if (label.style == NetLabel::Style::FLAG) {
 
         std::pair<Coordf, Coordf> extents;
@@ -706,7 +694,7 @@ void Canvas::render(const NetLabel &label)
         selectables.append(label.uuid, ObjectType::NET_LABEL, label.junction->position + Coordi(0, 1000000),
                            extents.first, extents.second);
     }
-    object_refs_current.pop_back();
+    object_ref_pop();
 }
 void Canvas::render(const BusLabel &label)
 {
@@ -748,8 +736,7 @@ void Canvas::render(const BusRipper &ripper)
     }
     auto extents = draw_text0(connector_pos + Coordi(0, 0.5_mm), 1.5_mm, ripper.bus_member->name, angle, false,
                               TextOrigin::BASELINE, c);
-    if (!ripper.temp)
-        targets.emplace_back(ripper.uuid, ObjectType::BUS_RIPPER, connector_pos);
+    targets.emplace_back(ripper.uuid, ObjectType::BUS_RIPPER, connector_pos);
     selectables.append(ripper.uuid, ObjectType::BUS_RIPPER, connector_pos, extents.first, extents.second);
 }
 
@@ -775,12 +762,15 @@ void Canvas::render(const Polygon &ipoly, bool interactive, ColorP co)
     if (!layer_display.count(poly.layer))
         return;
     if (auto plane = dynamic_cast<Plane *>(poly.usage.ptr)) {
-        triangle_type_current = Triangle::Type::PLANE;
+        triangle_type_current = TriangleInfo::Type::PLANE_FILL;
         auto tris = fragment_cache.get_triangles(*plane);
+        object_ref_push(ObjectType::PLANE, plane->uuid);
+        begin_group(poly.layer);
         for (const auto &tri : tris) {
             add_triangle(poly.layer, transform.transform(tri[0]), transform.transform(tri[1]),
                          transform.transform(tri[2]), co);
         }
+        triangle_type_current = TriangleInfo::Type::PLANE_OUTLINE;
         for (const auto &frag : plane->fragments) {
             ColorP co_orphan = co;
             if (frag.orphan == true)
@@ -800,10 +790,12 @@ void Canvas::render(const Polygon &ipoly, bool interactive, ColorP co)
                           poly.layer);
             }
         }
-        triangle_type_current = Triangle::Type::NONE;
+        end_group();
+        triangle_type_current = TriangleInfo::Type::NONE;
+        object_ref_pop();
     }
     else { // normal polygon
-        triangle_type_current = Triangle::Type::POLYGON;
+        triangle_type_current = TriangleInfo::Type::POLYGON;
         TPPLPoly po;
         po.Init(poly.vertices.size());
         po.SetHole(false);
@@ -819,6 +811,7 @@ void Canvas::render(const Polygon &ipoly, bool interactive, ColorP co)
         TPPLPartition part;
         po.SetOrientation(TPPL_CCW);
         part.Triangulate_EC(&po, &outpolys);
+        begin_group(poly.layer);
         for (auto &tri : outpolys) {
             assert(tri.GetNumPoints() == 3);
             Coordf p0 = transform.transform(coordf_from_pt(tri[0]));
@@ -830,37 +823,48 @@ void Canvas::render(const Polygon &ipoly, bool interactive, ColorP co)
             draw_line(poly.vertices[i].position, poly.vertices[(i + 1) % poly.vertices.size()].position, co,
                       poly.layer);
         }
-        triangle_type_current = Triangle::Type::NONE;
+        end_group();
+        triangle_type_current = TriangleInfo::Type::NONE;
     }
 
-    if (interactive && !ipoly.temp) {
+    if (interactive) {
+        selectables.group_begin();
         const Polygon::Vertex *v_last = nullptr;
-        size_t i = 0;
-        for (const auto &it : ipoly.vertices) {
-            if (v_last) {
-                auto center = (v_last->position + it.position) / 2;
-                if (v_last->type != Polygon::Vertex::Type::ARC) {
-                    selectables.append_line(poly.uuid, ObjectType::POLYGON_EDGE, v_last->position, it.position, 0,
-                                            i - 1, poly.layer);
+        {
+            size_t i = 0;
+            for (const auto &it : ipoly.vertices) {
+                if (v_last) {
+                    auto center = (v_last->position + it.position) / 2;
+                    if (v_last->type != Polygon::Vertex::Type::ARC) {
+                        selectables.append_line(poly.uuid, ObjectType::POLYGON_EDGE, v_last->position, it.position, 0,
+                                                i - 1, poly.layer);
 
-                    targets.emplace_back(poly.uuid, ObjectType::POLYGON_EDGE, center, i - 1, poly.layer);
+                        targets.emplace_back(poly.uuid, ObjectType::POLYGON_EDGE, center, i - 1, poly.layer);
+                    }
                 }
+                v_last = &it;
+                i++;
             }
-            selectables.append(poly.uuid, ObjectType::POLYGON_VERTEX, it.position, i, poly.layer);
-            targets.emplace_back(poly.uuid, ObjectType::POLYGON_VERTEX, it.position, i, poly.layer);
-            if (it.type == Polygon::Vertex::Type::ARC) {
-                selectables.append(poly.uuid, ObjectType::POLYGON_ARC_CENTER, it.arc_center, i, poly.layer);
-                targets.emplace_back(poly.uuid, ObjectType::POLYGON_ARC_CENTER, it.arc_center, i, poly.layer);
+            if (ipoly.vertices.back().type != Polygon::Vertex::Type::ARC) {
+                auto center = (ipoly.vertices.front().position + ipoly.vertices.back().position) / 2;
+                targets.emplace_back(poly.uuid, ObjectType::POLYGON_EDGE, center, i - 1, poly.layer);
+                selectables.append_line(poly.uuid, ObjectType::POLYGON_EDGE, poly.vertices.front().position,
+                                        ipoly.vertices.back().position, 0, i - 1, poly.layer);
             }
-            v_last = &it;
-            i++;
         }
-        if (ipoly.vertices.back().type != Polygon::Vertex::Type::ARC) {
-            auto center = (ipoly.vertices.front().position + ipoly.vertices.back().position) / 2;
-            targets.emplace_back(poly.uuid, ObjectType::POLYGON_EDGE, center, i - 1, poly.layer);
-            selectables.append_line(poly.uuid, ObjectType::POLYGON_EDGE, poly.vertices.front().position,
-                                    ipoly.vertices.back().position, 0, i - 1, poly.layer);
+        {
+            size_t i = 0;
+            for (const auto &it : ipoly.vertices) {
+                selectables.append(poly.uuid, ObjectType::POLYGON_VERTEX, it.position, i, poly.layer);
+                targets.emplace_back(poly.uuid, ObjectType::POLYGON_VERTEX, it.position, i, poly.layer);
+                if (it.type == Polygon::Vertex::Type::ARC) {
+                    selectables.append(poly.uuid, ObjectType::POLYGON_ARC_CENTER, it.arc_center, i, poly.layer);
+                    targets.emplace_back(poly.uuid, ObjectType::POLYGON_ARC_CENTER, it.arc_center, i, poly.layer);
+                }
+                i++;
+            }
         }
+        selectables.group_end();
     }
 }
 
@@ -877,10 +881,10 @@ void Canvas::render(const Shape &shape, bool interactive)
         targets.emplace_back(shape.uuid, ObjectType::SHAPE, shape.placement.shift, 0, shape.layer);
     }
     if (shape.form == Shape::Form::CIRCLE) {
-        auto r = shape.params.at(0);
+        auto r = shape.params.at(0) / 2;
         transform_save();
         transform.accumulate(shape.placement);
-        draw_line(Coordf(0, 0), Coordf(.1e3, 0), ColorP::FROM_LAYER, shape.layer, true, r);
+        add_triangle(shape.layer, transform.shift, Coordf(r, NAN), Coordf(NAN, NAN), ColorP::FROM_LAYER);
         transform_restore();
     }
     else if (shape.form == Shape::Form::RECTANGLE) {
@@ -889,7 +893,7 @@ void Canvas::render(const Shape &shape, bool interactive)
         auto w = shape.params.at(0);
         auto h = shape.params.at(1);
         add_triangle(shape.layer, transform.transform(Coordf(-w / 2, 0)), transform.transform(Coordf(w / 2, 0)),
-                     Coordf(h, NAN), ColorP::FROM_LAYER, Triangle::FLAG_BUTT);
+                     Coordf(h, NAN), ColorP::FROM_LAYER, TriangleInfo::FLAG_BUTT);
         transform_restore();
     }
     else if (shape.form == Shape::Form::OBROUND) {
@@ -962,9 +966,7 @@ void Canvas::render(const Symbol &sym, bool on_sheet, bool smashed, ColorP co)
         for (const auto &it : sym.junctions) {
             auto &junc = it.second;
             selectables.append(junc.uuid, ObjectType::JUNCTION, junc.position, 0, 10000, true);
-            if (!junc.temp) {
-                targets.emplace_back(junc.uuid, ObjectType::JUNCTION, transform.transform(junc.position));
-            }
+            targets.emplace_back(junc.uuid, ObjectType::JUNCTION, transform.transform(junc.position));
         }
     }
 
@@ -975,9 +977,9 @@ void Canvas::render(const Symbol &sym, bool on_sheet, bool smashed, ColorP co)
     if (object_refs_current.size() && object_refs_current.back().type == ObjectType::SCHEMATIC_SYMBOL) {
         auto sym_uuid = object_refs_current.back().uuid;
         for (const auto &it : sym.pins) {
-            object_refs_current.emplace_back(ObjectType::SYMBOL_PIN, it.second.uuid, sym_uuid);
+            object_ref_push(ObjectType::SYMBOL_PIN, it.second.uuid, sym_uuid);
             render(it.second, !on_sheet, co);
-            object_refs_current.pop_back();
+            object_ref_pop();
         }
     }
     else {
@@ -1038,6 +1040,9 @@ void Canvas::render(const Sheet &sheet)
     for (const auto &it : sheet.arcs) {
         render(it.second);
     }
+    for (const auto &it : sheet.pictures) {
+        render(it.second);
+    }
 }
 
 void Canvas::render(const Padstack &padstack, bool interactive)
@@ -1065,6 +1070,7 @@ void Canvas::render_pad_overlay(const Pad &pad)
     auto bb = pad.padstack.get_bbox(true); // only copper
     auto a = bb.first;
     auto b = bb.second;
+    transform.accumulate(Placement((a + b) / 2));
 
     auto pad_width = abs(b.x - a.x);
     auto pad_height = abs(b.y - a.y);
@@ -1092,10 +1098,10 @@ void Canvas::render_pad_overlay(const Pad &pad)
 
     set_lod_size(std::min(pad_height, pad_width));
     for (const auto overlay_layer : text_layers) {
-        if (pad.net && pad.net->name.size() > 0) {
+        if (pad.secondary_text.size()) {
             draw_bitmap_text_box(tr, pad_width, pad_height, pad.name, ColorP::TEXT_OVERLAY, overlay_layer,
                                  TextBoxMode::UPPER);
-            draw_bitmap_text_box(tr, pad_width, pad_height, pad.net->name, ColorP::TEXT_OVERLAY, overlay_layer,
+            draw_bitmap_text_box(tr, pad_width, pad_height, pad.secondary_text, ColorP::TEXT_OVERLAY, overlay_layer,
                                  TextBoxMode::LOWER);
         }
         else {
@@ -1107,16 +1113,26 @@ void Canvas::render_pad_overlay(const Pad &pad)
     transform_restore();
 }
 
+static int get_target_layer_for_padstack_type(Padstack::Type type)
+{
+    if (type == Padstack::Type::TOP) {
+        return BoardLayers::TOP_COPPER;
+    }
+    else if (type == Padstack::Type::BOTTOM) {
+        return BoardLayers::BOTTOM_COPPER;
+    }
+    else {
+        return 10000;
+    }
+}
+
 void Canvas::render(const Package &pkg, bool interactive, bool smashed, bool omit_silkscreen, bool omit_outline)
 {
     if (interactive) {
         for (const auto &it : pkg.junctions) {
             auto &junc = it.second;
             selectables.append(junc.uuid, ObjectType::JUNCTION, junc.position, 0, junc.layer);
-            if (!junc.temp) {
-                targets.emplace_back(junc.uuid, ObjectType::JUNCTION, transform.transform(junc.position), 0,
-                                     junc.layer);
-            }
+            targets.emplace_back(junc.uuid, ObjectType::JUNCTION, transform.transform(junc.position), 0, junc.layer);
         }
     }
 
@@ -1140,18 +1156,18 @@ void Canvas::render(const Package &pkg, bool interactive, bool smashed, bool omi
     if (object_refs_current.size() && object_refs_current.back().type == ObjectType::BOARD_PACKAGE) {
         auto pkg_uuid = object_refs_current.back().uuid;
         for (const auto &it : pkg.pads) {
-            object_refs_current.emplace_back(ObjectType::PAD, it.second.uuid, pkg_uuid);
+            object_ref_push(ObjectType::PAD, it.second.uuid, pkg_uuid);
             render_pad_overlay(it.second);
             render(it.second);
-            object_refs_current.pop_back();
+            object_ref_pop();
         }
     }
     else if (interactive) {
         for (const auto &it : pkg.pads) {
-            object_refs_current.emplace_back(ObjectType::PAD, it.second.uuid);
+            object_ref_push(ObjectType::PAD, it.second.uuid);
             render_pad_overlay(it.second);
             render(it.second);
-            object_refs_current.pop_back();
+            object_ref_pop();
         }
     }
     else {
@@ -1169,13 +1185,14 @@ void Canvas::render(const Package &pkg, bool interactive, bool smashed, bool omi
     }
 
     if (interactive) {
-        for (const auto &it : pkg.pads) {
+        for (const auto &[pad_uuid, pad] : pkg.pads) {
             transform_save();
-            transform.accumulate(it.second.placement);
-            auto bb = it.second.padstack.get_bbox();
-            selectables.append(it.second.uuid, ObjectType::PAD, {0, 0}, bb.first, bb.second);
+            transform.accumulate(pad.placement);
+            auto bb = pad.padstack.get_bbox();
+            selectables.append(pad.uuid, ObjectType::PAD, {0, 0}, bb.first, bb.second);
             transform_restore();
-            targets.emplace_back(it.second.uuid, ObjectType::PAD, it.second.placement.shift);
+            int target_layer = get_target_layer_for_padstack_type(pad.padstack.type);
+            targets.emplace_back(pad.uuid, ObjectType::PAD, pad.placement.shift, 0, target_layer);
         }
         for (const auto &it : pkg.warnings) {
             render(it);
@@ -1183,6 +1200,9 @@ void Canvas::render(const Package &pkg, bool interactive, bool smashed, bool omi
         for (const auto &it : pkg.dimensions) {
             render(it.second);
         }
+        for (const auto &it : pkg.pictures) {
+            render(it.second);
+        }
     }
 }
 
@@ -1236,18 +1256,19 @@ void Canvas::render(const BoardPackage &pkg, bool interactive)
                            pkg.flip ? BoardLayers::BOTTOM_PACKAGE : BoardLayers::TOP_PACKAGE);
         targets.emplace_back(pkg.uuid, ObjectType::BOARD_PACKAGE, pkg.placement.shift);
 
-        for (const auto &it : pkg.package.pads) {
-            targets.emplace_back(UUIDPath<2>(pkg.uuid, it.first), ObjectType::PAD,
-                                 transform.transform(it.second.placement.shift));
+        for (const auto &[pad_uuid, pad] : pkg.package.pads) {
+            int target_layer = get_target_layer_for_padstack_type(pad.padstack.type);
+            targets.emplace_back(UUIDPath<2>(pkg.uuid, pad.uuid), ObjectType::PAD,
+                                 transform.transform(pad.placement.shift), 0, target_layer);
         }
     }
     if (interactive)
-        object_refs_current.emplace_back(ObjectType::BOARD_PACKAGE, pkg.uuid);
+        object_ref_push(ObjectType::BOARD_PACKAGE, pkg.uuid);
 
     render(pkg.package, false, pkg.smashed, pkg.omit_silkscreen, pkg.omit_outline);
 
     if (interactive)
-        object_refs_current.pop_back();
+        object_ref_pop();
 
     transform.reset();
     transform_restore();
@@ -1267,15 +1288,24 @@ void Canvas::render(const Via &via, bool interactive)
     img_net(via.junction->net);
     img_patch_type(PatchType::VIA);
     if (interactive)
-        object_refs_current.emplace_back(ObjectType::VIA, via.uuid);
+        object_ref_push(ObjectType::VIA, via.uuid);
     render(via.padstack, false);
     if (via.locked) {
         auto ol = get_overlay_layer(BoardLayers::TOP_COPPER);
         draw_lock({0, 0}, 0.7 * std::min(std::abs(bb.second.x - bb.first.x), std::abs(bb.second.y - bb.first.y)),
                   ColorP::TEXT_OVERLAY, ol, true);
     }
+    if (show_text_in_vias && interactive && via.junction->net && via.junction->net->name.size()) {
+        auto size = (bb.second.x - bb.first.x) * 1.2;
+        set_lod_size(size);
+        Placement p(via.junction->position);
+        draw_bitmap_text_box(p, size, size, via.junction->net->name, ColorP::TEXT_OVERLAY, 10000, TextBoxMode::FULL);
+        set_lod_size(-1);
+    }
+
+
     if (interactive)
-        object_refs_current.pop_back();
+        object_ref_pop();
     img_net(nullptr);
     img_patch_type(PatchType::OTHER);
     transform_restore();
@@ -1294,10 +1324,10 @@ void Canvas::render(const BoardHole &hole, bool interactive)
     else
         img_patch_type(PatchType::HOLE_NPTH);
     if (interactive)
-        object_refs_current.emplace_back(ObjectType::BOARD_HOLE, hole.uuid);
+        object_ref_push(ObjectType::BOARD_HOLE, hole.uuid);
     render(hole.padstack, false);
     if (interactive)
-        object_refs_current.pop_back();
+        object_ref_pop();
     img_net(nullptr);
     img_patch_type(PatchType::OTHER);
     transform_restore();
@@ -1305,8 +1335,6 @@ void Canvas::render(const BoardHole &hole, bool interactive)
 
 void Canvas::render(const class Dimension &dim)
 {
-    typedef Coord<double> Coordd;
-
     Coordd p0 = dim.p0;
     Coordd p1 = dim.p1;
 
@@ -1330,6 +1358,7 @@ void Canvas::render(const class Dimension &dim)
 
     ColorP co = ColorP::DIMENSION;
 
+    selectables.group_begin();
     if (v.mag_sq()) {
 
         auto q0 = p0 + wn * dim.label_distance;
@@ -1348,7 +1377,6 @@ void Canvas::render(const class Dimension &dim)
 
         Coordf text_pos = (q0 + v / 2) - (vn * text_width / 2);
         auto angle = atan2(v.y, v.x);
-        int angle_i = (angle / M_PI) * 32768;
 
         if ((text_width + 2 * dim.label_size) > length) {
             text_pos = q1;
@@ -1370,7 +1398,7 @@ void Canvas::render(const class Dimension &dim)
         transform_save();
         transform.reset();
         transform.shift = Coordi(q0.x, q0.y);
-        transform.set_angle(angle_i);
+        transform.set_angle_rad(angle);
         draw_line({0, 0}, Coordf(arrow_mul * dim.label_size, dim.label_size / 2), co, 10000, true, 0);
         draw_line({0, 0}, Coordf(arrow_mul * dim.label_size, (int64_t)dim.label_size / -2), co, 10000, true, 0);
 
@@ -1379,6 +1407,7 @@ void Canvas::render(const class Dimension &dim)
         draw_line({0, 0}, Coordf(-arrow_mul * dim.label_size, (int64_t)dim.label_size / -2), co, 10000, true, 0);
         transform_restore();
 
+        int angle_i = (angle / M_PI) * 32768;
         auto real_text_bb = draw_text0(Coordi(text_pos.x, text_pos.y), dim.label_size, s,
                                        get_flip_view() ? (32768 - angle_i) : angle_i, get_flip_view(),
                                        TextOrigin::CENTER, co, 10000, 0, true);
@@ -1389,11 +1418,10 @@ void Canvas::render(const class Dimension &dim)
 
     selectables.append(dim.uuid, ObjectType::DIMENSION, dim.p0, 0);
     selectables.append(dim.uuid, ObjectType::DIMENSION, dim.p1, 1);
+    selectables.group_end();
 
-    if (!dim.temp) {
-        targets.emplace_back(dim.uuid, ObjectType::DIMENSION, dim.p0, 0);
-        targets.emplace_back(dim.uuid, ObjectType::DIMENSION, dim.p1, 1);
-    }
+    targets.emplace_back(dim.uuid, ObjectType::DIMENSION, dim.p0, 0);
+    targets.emplace_back(dim.uuid, ObjectType::DIMENSION, dim.p1, 1);
 }
 
 void Canvas::render(const Board &brd, bool interactive, PanelMode mode, OutlineMode outline_mode)
@@ -1408,17 +1436,6 @@ void Canvas::render(const Board &brd, bool interactive, PanelMode mode, OutlineM
             render(it.second, true, ObjectType::BOARD);
         }
     }
-    if (!img_mode && interactive) {
-        for (const auto &it_net : brd.airwires) {
-            if (airwire_filter == nullptr || airwire_filter->airwire_is_visible(it_net.first)) {
-                object_refs_current.emplace_back(ObjectType::AIRWIRE, it_net.first);
-                for (const auto &it : it_net.second) {
-                    render(it);
-                }
-                object_refs_current.pop_back();
-            }
-        }
-    }
     for (const auto &it : brd.polygons) {
         if (outline_mode == OutlineMode::OMIT && it.second.layer == BoardLayers::L_OUTLINE)
             continue;
@@ -1479,6 +1496,9 @@ void Canvas::render(const Board &brd, bool interactive, PanelMode mode, OutlineM
         for (const auto &it : brd.warnings) {
             render(it);
         }
+        for (const auto &it : brd.pictures) {
+            render(it.second);
+        }
     }
 
     clock_t end = clock();
@@ -1507,9 +1527,7 @@ void Canvas::render(const Frame &fr, bool on_sheet)
         for (const auto &it : fr.junctions) {
             auto &junc = it.second;
             selectables.append(junc.uuid, ObjectType::JUNCTION, junc.position, 0, 10000, true);
-            if (!junc.temp) {
-                targets.emplace_back(junc.uuid, ObjectType::JUNCTION, transform.transform(junc.position));
-            }
+            targets.emplace_back(junc.uuid, ObjectType::JUNCTION, transform.transform(junc.position));
         }
     }
     for (const auto &it : fr.lines) {
@@ -1533,4 +1551,29 @@ void Canvas::render(const Frame &fr, bool on_sheet)
     draw_line(Coordf(0, fr.height), Coordf(), c);
 }
 
+void Canvas::render(const Picture &pic)
+{
+    if (!pic.data && !img_mode) {
+        draw_error(pic.placement.shift, 2e5, "Image " + (std::string)pic.data_uuid + " not found");
+        selectables.append_angled(pic.uuid, ObjectType::PICTURE, pic.placement.shift, pic.placement.shift,
+                                  Coordf(1_mm, 1_mm), 0);
+        return;
+    }
+    pictures.emplace_back();
+    auto &x = pictures.back();
+    x.angle = pic.placement.get_angle() / 65536.0 * 2 * M_PI;
+    x.x = pic.placement.shift.x;
+    x.y = pic.placement.shift.y;
+    x.px_size = pic.px_size;
+    x.data = pic.data;
+    x.on_top = pic.on_top;
+    x.opacity = pic.opacity;
+    float w = pic.data->width * pic.px_size;
+    float h = pic.data->height * pic.px_size;
+    if (img_mode)
+        return;
+    selectables.append_angled(pic.uuid, ObjectType::PICTURE, pic.placement.shift, pic.placement.shift, {w, h},
+                              pic.placement.get_angle_rad());
+}
+
 } // namespace horizon
