@@ -1,11 +1,12 @@
 #include "selectables.hpp"
 #include "canvas.hpp"
+#include <assert.h>
 
 namespace horizon {
 
 Selectable::Selectable(const Coordf &center, const Coordf &box_center, const Coordf &box_dim, float a, bool always)
     : x(center.x), y(center.y), c_x(box_center.x), c_y(box_center.y), width(std::abs(box_dim.x)),
-      height(std::abs(box_dim.y)), angle(a), flags(always ? 4 : 0)
+      height(std::abs(box_dim.y)), angle(a), flags(always ? static_cast<int>(Flag::ALWAYS) : 0)
 {
 }
 
@@ -15,10 +16,10 @@ bool Selectable::inside(const Coordf &c, float expand) const
     float a = -angle;
     float dx = d.x * cos(a) - d.y * sin(a);
     float dy = d.x * sin(a) + d.y * cos(a);
-    float w = width / 2;
-    float h = height / 2;
+    float w = std::max(width, expand) / 2;
+    float h = std::max(height, expand) / 2;
 
-    return (dx >= -w - expand) && (dx <= w + expand) && (dy >= -h - expand) && (dy <= h + expand);
+    return (dx >= -w) && (dx <= w) && (dy >= -h) && (dy <= h);
 }
 
 float Selectable::area() const
@@ -31,6 +32,21 @@ float Selectable::area() const
         return width * height;
 }
 
+bool Selectable::is_line() const
+{
+    return (width == 0) != (height == 0);
+}
+
+bool Selectable::is_point() const
+{
+    return width == 0 && height == 0;
+}
+
+bool Selectable::is_box() const
+{
+    return width > 0 && height > 0;
+}
+
 static void rotate(float &x, float &y, float a)
 {
     float ix = x;
@@ -72,20 +88,20 @@ bool Selectable::get_flag(Selectable::Flag f) const
     return flags & mask;
 }
 
-Selectables::Selectables(class Canvas *c) : ca(c)
+Selectables::Selectables(const Canvas &c) : ca(c)
 {
 }
 
 void Selectables::append(const UUID &uu, ObjectType ot, const Coordf &center, const Coordf &a, const Coordf &b,
                          unsigned int vertex, int layer, bool always)
 {
-    Placement tr = ca->transform;
+    Placement tr = ca.transform;
     if (tr.mirror)
         tr.invert_angle();
     tr.mirror = false;
-    auto box_center = ca->transform.transform((b + a) / 2);
+    auto box_center = ca.transform.transform((b + a) / 2);
     auto box_dim = b - a;
-    append_angled(uu, ot, center, box_center, box_dim, (tr.get_angle() * M_PI) / 32768.0, vertex, layer, always);
+    append_angled(uu, ot, center, box_center, box_dim, tr.get_angle_rad(), vertex, layer, always);
 }
 
 void Selectables::append(const UUID &uu, ObjectType ot, const Coordf &center, unsigned int vertex, int layer,
@@ -99,8 +115,9 @@ void Selectables::append_angled(const UUID &uu, ObjectType ot, const Coordf &cen
 {
     items_map.emplace(std::piecewise_construct, std::forward_as_tuple(uu, ot, vertex, layer),
                       std::forward_as_tuple(items.size()));
-    items.emplace_back(ca->transform.transform(center), box_center, box_dim, angle, always);
+    items.emplace_back(ca.transform.transform(center), box_center, box_dim, angle, always);
     items_ref.emplace_back(uu, ot, vertex, layer);
+    items_group.push_back(group_current);
 }
 
 void Selectables::append_line(const UUID &uu, ObjectType ot, const Coordf &p0, const Coordf &p1, float width,
@@ -115,10 +132,42 @@ void Selectables::append_line(const UUID &uu, ObjectType ot, const Coordf &p0, c
 }
 
 
+void Selectables::update_preview(const std::set<SelectableRef> &sel)
+{
+    std::set<int> groups;
+    for (const auto &it : sel) {
+        if (items_map.count(it)) {
+            auto idx = items_map.at(it);
+            auto group = items_group.at(idx);
+            if (group != -1)
+                groups.insert(group);
+        }
+    }
+    for (size_t i = 0; i < items.size(); i++) {
+        auto group = items_group.at(i);
+        items.at(i).set_flag(Selectable::Flag::PREVIEW, groups.count(group));
+    }
+}
+
+void Selectables::group_begin()
+{
+    assert(group_current == -1);
+    group_current = group_max;
+}
+
+void Selectables::group_end()
+{
+    assert(group_current != -1);
+    group_current = -1;
+    group_max++;
+}
+
 void Selectables::clear()
 {
     items.clear();
     items_ref.clear();
+    items_group.clear();
     items_map.clear();
+    group_max = 0;
 }
 } // namespace horizon
