@@ -1,12 +1,11 @@
 #include "selection_filter.hpp"
 #include "board/board_layers.hpp"
-#include "canvas.hpp"
+#include "canvas_gl.hpp"
 
 namespace horizon {
 bool SelectionFilter::can_select(const SelectableRef &sel) const
 {
-    if (!ca->layer_is_visible(sel.layer) && sel.layer != 10000 && sel.layer != BoardLayers::TOP_PACKAGE
-        && sel.layer != BoardLayers::BOTTOM_PACKAGE)
+    if (!ca.layer_is_visible(sel.layer) && sel.layer != 10000 && sel.type != ObjectType::BOARD_PACKAGE)
         return false;
     ObjectType type = sel.type;
     if (type == ObjectType::POLYGON_ARC_CENTER || sel.type == ObjectType::POLYGON_EDGE
