@@ -55,9 +55,10 @@ enum class ObjectType {
     KEEPOUT,
     CONNECTION_LINE,
     AIRWIRE,
-    BOARD_PANEL
+    BOARD_PANEL,
+    PICTURE
 };
-enum class PatchType { OTHER, TRACK, PAD, PAD_TH, VIA, PLANE, HOLE_PTH, HOLE_NPTH, BOARD_EDGE, TEXT };
+enum class PatchType { OTHER, TRACK, PAD, PAD_TH, VIA, PLANE, HOLE_PTH, HOLE_NPTH, BOARD_EDGE, TEXT, N_TYPES };
 
 extern const LutEnumStr<PatchType> patch_type_lut;
 extern const LutEnumStr<ObjectType> object_type_lut;
@@ -211,6 +212,7 @@ public:
 
 typedef Coord<float> Coordf;
 typedef Coord<int64_t> Coordi;
+typedef Coord<double> Coordd;
 
 class Color {
 public:
@@ -231,6 +233,28 @@ public:
     }
 };
 
+struct ColorI {
+    uint8_t r;
+    uint8_t g;
+    uint8_t b;
+
+    bool operator<(const ColorI &other) const
+    {
+        return hashify() < other.hashify();
+    }
+
+    Color to_color() const
+    {
+        return Color::new_from_int(r, g, b);
+    }
+
+private:
+    uint32_t hashify() const
+    {
+        return r | (g << 8) | (b << 16);
+    }
+};
+
 constexpr int64_t operator"" _mm(long double i)
 {
     return i * 1e6;
