@@ -60,6 +60,7 @@ public:
     {
         return m_pool;
     }
+    virtual ObjectType get_object_type() const = 0;
 
     /**
      * Expands the non-working document.
@@ -69,7 +70,6 @@ public:
     ToolResponse tool_begin(ToolID tool_id, const ToolArgs &args, class ImpInterface *imp, bool transient = false);
     ToolResponse tool_update(const ToolArgs &args);
     std::pair<bool, bool> tool_can_begin(ToolID tool_id, const std::set<SelectableRef> &selection);
-    bool tool_handles_esc();
     void save();
     void autosave();
     virtual void delete_autosave() = 0;
@@ -118,6 +118,8 @@ public:
     std::set<SelectableRef> &get_tool_selection();
     Pool *m_pool;
 
+    std::set<InToolActionID> get_tool_actions() const;
+
     bool get_needs_save() const;
     void set_needs_save();
 
