@@ -12,7 +12,7 @@ namespace horizon {
 class CoreBoard : public Core, public DocumentBoard {
 public:
     CoreBoard(const std::string &board_filename, const std::string &block_filename, const std::string &via_dir,
-              Pool &pool);
+              const std::string &pictures_dir, Pool &pool);
 
     class Block *get_block() override;
     class LayerProvider *get_layer_provider() override;
@@ -61,6 +61,11 @@ public:
 
     const std::string &get_filename() const override;
 
+    ObjectType get_object_type() const override
+    {
+        return ObjectType::BOARD;
+    }
+
 private:
     ViaPadstackProvider via_padstack_provider;
 
@@ -77,7 +82,7 @@ private:
 
     std::string m_board_filename;
     std::string m_block_filename;
-    std::string m_via_dir;
+    std::string m_pictures_dir;
 
     class HistoryItem : public Core::HistoryItem {
     public:
