@@ -33,23 +33,23 @@ bool CoreFrame::has_object_type(ObjectType ty) const
     return false;
 }
 
-std::map<UUID, Polygon> *CoreFrame::get_polygon_map(bool work)
+std::map<UUID, Polygon> *CoreFrame::get_polygon_map()
 {
     return &frame.polygons;
 }
-std::map<UUID, Junction> *CoreFrame::get_junction_map(bool work)
+std::map<UUID, Junction> *CoreFrame::get_junction_map()
 {
     return &frame.junctions;
 }
-std::map<UUID, Text> *CoreFrame::get_text_map(bool work)
+std::map<UUID, Text> *CoreFrame::get_text_map()
 {
     return &frame.texts;
 }
-std::map<UUID, Line> *CoreFrame::get_line_map(bool work)
+std::map<UUID, Line> *CoreFrame::get_line_map()
 {
     return &frame.lines;
 }
-std::map<UUID, Arc> *CoreFrame::get_arc_map(bool work)
+std::map<UUID, Arc> *CoreFrame::get_arc_map()
 {
     return &frame.arcs;
 }
@@ -75,17 +75,6 @@ Frame *CoreFrame::get_frame()
     return &frame;
 }
 
-void CoreFrame::commit()
-{
-    set_needs_save(true);
-}
-
-void CoreFrame::revert()
-{
-    history_load(history_current);
-    reverted = true;
-}
-
 CoreFrame::HistoryItem::HistoryItem(const Frame &fr) : frame(fr)
 {
 }
