@@ -22,6 +22,11 @@ public:
 
     const std::string &get_filename() const override;
 
+    ObjectType get_object_type() const override
+    {
+        return ObjectType::FRAME;
+    }
+
 private:
     std::map<UUID, Polygon> *get_polygon_map() override;
     std::map<UUID, Junction> *get_junction_map() override;
