@@ -7,13 +7,16 @@
 #include "util/util.hpp"
 #include <giomm/file.h>
 #include <glibmm/fileutils.h>
+#include <glibmm/miscutils.h>
 
 namespace horizon {
 CorePackage::CorePackage(const std::string &filename, Pool &pool)
-    : package(Package::new_from_file(filename, pool)), m_filename(filename), rules(package.rules),
+    : package(Package::new_from_file(filename, pool)), m_filename(filename),
+      m_pictures_dir(Glib::build_filename(Glib::path_get_dirname(filename), "pictures")), rules(package.rules),
       parameter_program_code(package.parameter_program.get_code()), parameter_set(package.parameter_set),
       models(package.models), default_model(package.default_model)
 {
+    package.load_pictures(m_pictures_dir);
     rebuild();
     m_pool = &pool;
 }
@@ -32,6 +35,7 @@ bool CorePackage::has_object_type(ObjectType ty) const
     case ObjectType::ARC:
     case ObjectType::KEEPOUT:
     case ObjectType::DIMENSION:
+    case ObjectType::PICTURE:
         return true;
         break;
     default:;
@@ -83,6 +87,10 @@ std::map<UUID, Dimension> *CorePackage::get_dimension_map()
 {
     return &package.dimensions;
 }
+std::map<UUID, Picture> *CorePackage::get_picture_map()
+{
+    return &package.pictures;
+}
 std::map<UUID, Hole> *CorePackage::get_hole_map()
 {
     return nullptr;
@@ -260,6 +268,7 @@ void CorePackage::save(const std::string &suffix)
 
     json j = package.serialize();
     save_json_to_file(m_filename + suffix, j);
+    package.save_pictures(m_pictures_dir);
 }
 
 
