@@ -45,6 +45,11 @@ public:
 
     const std::string &get_filename() const override;
 
+    ObjectType get_object_type() const override
+    {
+        return ObjectType::PACKAGE;
+    }
+
 private:
     std::map<UUID, Junction> *get_junction_map() override;
     std::map<UUID, Line> *get_line_map() override;
@@ -54,9 +59,11 @@ private:
     std::map<UUID, Hole> *get_hole_map() override;
     std::map<UUID, Keepout> *get_keepout_map() override;
     std::map<UUID, Dimension> *get_dimension_map() override;
+    std::map<UUID, Picture> *get_picture_map() override;
 
     Package package;
     std::string m_filename;
+    std::string m_pictures_dir;
 
     PackageRules rules;
 
