@@ -34,74 +34,74 @@ bool CoreSymbol::has_object_type(ObjectType ty) const
     return false;
 }
 
-std::map<UUID, Text> *CoreSymbol::get_text_map(bool work)
+std::map<UUID, Text> *CoreSymbol::get_text_map()
 {
     return &sym.texts;
 }
 
-std::map<UUID, Polygon> *CoreSymbol::get_polygon_map(bool work)
+std::map<UUID, Polygon> *CoreSymbol::get_polygon_map()
 {
     return &sym.polygons;
 }
 
-Junction *CoreSymbol::get_junction(const UUID &uu, bool work)
+Junction *CoreSymbol::get_junction(const UUID &uu)
 {
     return sym.get_junction(uu);
 }
-Line *CoreSymbol::get_line(const UUID &uu, bool work)
+Line *CoreSymbol::get_line(const UUID &uu)
 {
     return &sym.lines.at(uu);
 }
-SymbolPin *CoreSymbol::get_symbol_pin(const UUID &uu, bool work)
+SymbolPin *CoreSymbol::get_symbol_pin(const UUID &uu)
 {
     return sym.get_symbol_pin(uu);
 }
-Arc *CoreSymbol::get_arc(const UUID &uu, bool work)
+Arc *CoreSymbol::get_arc(const UUID &uu)
 {
     return &sym.arcs.at(uu);
 }
 
-Junction *CoreSymbol::insert_junction(const UUID &uu, bool work)
+Junction *CoreSymbol::insert_junction(const UUID &uu)
 {
     auto x = sym.junctions.emplace(std::make_pair(uu, uu));
     return &(x.first->second);
 }
-void CoreSymbol::delete_junction(const UUID &uu, bool work)
+void CoreSymbol::delete_junction(const UUID &uu)
 {
     sym.junctions.erase(uu);
 }
 
-Line *CoreSymbol::insert_line(const UUID &uu, bool work)
+Line *CoreSymbol::insert_line(const UUID &uu)
 {
     auto x = sym.lines.emplace(std::make_pair(uu, uu));
     return &(x.first->second);
 }
-void CoreSymbol::delete_line(const UUID &uu, bool work)
+void CoreSymbol::delete_line(const UUID &uu)
 {
     sym.lines.erase(uu);
 }
 
-Arc *CoreSymbol::insert_arc(const UUID &uu, bool work)
+Arc *CoreSymbol::insert_arc(const UUID &uu)
 {
     auto x = sym.arcs.emplace(std::make_pair(uu, uu));
     return &(x.first->second);
 }
-void CoreSymbol::delete_arc(const UUID &uu, bool work)
+void CoreSymbol::delete_arc(const UUID &uu)
 {
     sym.arcs.erase(uu);
 }
 
-void CoreSymbol::delete_symbol_pin(const UUID &uu, bool work)
+void CoreSymbol::delete_symbol_pin(const UUID &uu)
 {
     sym.pins.erase(uu);
 }
-SymbolPin *CoreSymbol::insert_symbol_pin(const UUID &uu, bool work)
+SymbolPin *CoreSymbol::insert_symbol_pin(const UUID &uu)
 {
     auto x = sym.pins.emplace(std::make_pair(uu, uu));
     return &(x.first->second);
 }
 
-std::vector<Line *> CoreSymbol::get_lines(bool work)
+std::vector<Line *> CoreSymbol::get_lines()
 {
     std::vector<Line *> r;
     for (auto &it : sym.lines) {
@@ -110,7 +110,7 @@ std::vector<Line *> CoreSymbol::get_lines(bool work)
     return r;
 }
 
-std::vector<Arc *> CoreSymbol::get_arcs(bool work)
+std::vector<Arc *> CoreSymbol::get_arcs()
 {
     std::vector<Arc *> r;
     for (auto &it : sym.arcs) {
@@ -119,7 +119,7 @@ std::vector<Arc *> CoreSymbol::get_arcs(bool work)
     return r;
 }
 
-std::vector<const Pin *> CoreSymbol::get_pins(bool work)
+std::vector<const Pin *> CoreSymbol::get_pins()
 {
     std::vector<const Pin *> r;
     for (auto &it : sym.unit->pins) {
@@ -296,38 +296,7 @@ std::pair<Coordi, Coordi> CoreSymbol::get_bbox()
     return bb;
 }
 
-bool CoreSymbol::can_search_for_object_type(ObjectType ty) const
-{
-    switch (ty) {
-    case ObjectType::SYMBOL_PIN:
-        return true;
-        break;
-    default:;
-    }
-
-    return false;
-}
-
-std::list<Core::SearchResult> CoreSymbol::search(const SearchQuery &q)
-{
-    std::list<Core::SearchResult> results;
-    if (q.get_query().size() == 0)
-        return results;
-    if (q.types.count(ObjectType::SYMBOL_PIN)) {
-        for (const auto &it : sym.pins) {
-            if (q.contains(it.second.name)) {
-                results.emplace_back(ObjectType::SYMBOL_PIN, it.first);
-                auto &x = results.back();
-                x.location = it.second.position;
-                x.selectable = true;
-            }
-        }
-    }
-    sort_search_results(results, q);
-    return results;
-}
-
-Symbol *CoreSymbol::get_symbol(bool work)
+Symbol *CoreSymbol::get_symbol()
 {
     return &sym;
 }
@@ -348,17 +317,6 @@ void CoreSymbol::set_pin_display_mode(Symbol::PinDisplayMode mode)
     sym.expand(pin_display_mode);
 }
 
-void CoreSymbol::commit()
-{
-    set_needs_save(true);
-}
-
-void CoreSymbol::revert()
-{
-    history_load(history_current);
-    reverted = true;
-}
-
 const std::string &CoreSymbol::get_filename() const
 {
     return m_filename;
