@@ -44,7 +44,6 @@
 #include "tools/tool_place_text.hpp"
 #include "tools/tool_place_via.hpp"
 #include "tools/tool_rotate_arbitrary.hpp"
-#include "tools/tool_route_track.hpp"
 #include "tools/tool_route_track_interactive.hpp"
 #include "tools/tool_set_diffpair.hpp"
 #include "tools/tool_set_via_net.hpp"
@@ -77,6 +76,8 @@
 #include "tools/tool_smash_package_outline.hpp"
 #include "tools/tool_resize_symbol.hpp"
 #include "tools/tool_round_off_vertex.hpp"
+#include "tools/tool_swap_gates.hpp"
+#include "tools/tool_place_picture.hpp"
 
 namespace horizon {
 
@@ -173,6 +174,7 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
         return std::make_unique<ToolEnterDatum>(this, tool_id);
 
     case ToolID::PLACE_HOLE:
+    case ToolID::PLACE_HOLE_SLOT:
         return std::make_unique<ToolPlaceHole>(this, tool_id);
 
     case ToolID::PLACE_PAD:
@@ -194,9 +196,6 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::PLACE_VIA:
         return std::make_unique<ToolPlaceVia>(this, tool_id);
 
-    case ToolID::ROUTE_TRACK:
-        return std::make_unique<ToolRouteTrack>(this, tool_id);
-
     case ToolID::DRAG_KEEP_SLOPE:
         return std::make_unique<ToolDragKeepSlope>(this, tool_id);
 
@@ -205,6 +204,8 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
         return std::make_unique<ToolSmash>(this, tool_id);
 
     case ToolID::PLACE_SHAPE:
+    case ToolID::PLACE_SHAPE_OBROUND:
+    case ToolID::PLACE_SHAPE_RECTANGLE:
         return std::make_unique<ToolPlaceShape>(this, tool_id);
 
     case ToolID::EDIT_SHAPE:
@@ -368,6 +369,12 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::ROUND_OFF_VERTEX:
         return std::make_unique<ToolRoundOffVertex>(this, tool_id);
 
+    case ToolID::SWAP_GATES:
+        return std::make_unique<ToolSwapGates>(this, tool_id);
+
+    case ToolID::PLACE_PICTURE:
+        return std::make_unique<ToolPlacePicture>(this, tool_id);
+
     default:
         return nullptr;
     }
