@@ -36,19 +36,21 @@ enum class ToolID {
     ENTER_DATUM,
     MOVE_EXACTLY,
     PLACE_HOLE,
+    PLACE_HOLE_SLOT,
     PLACE_PAD,
     PASTE,
     ASSIGN_PART,
     MAP_PACKAGE,
     DRAW_TRACK,
     PLACE_VIA,
-    ROUTE_TRACK,
     DRAG_KEEP_SLOPE,
     ADD_PART,
     ANNOTATE,
     SMASH,
     UNSMASH,
     PLACE_SHAPE,
+    PLACE_SHAPE_RECTANGLE,
+    PLACE_SHAPE_OBROUND,
     EDIT_SHAPE,
     IMPORT_DXF,
     MANAGE_NET_CLASSES,
@@ -140,6 +142,8 @@ enum class ToolID {
     SMASH_PANEL_OUTLINE,
     SMASH_PACKAGE_OUTLINE,
     RESIZE_SYMBOL,
-    ROUND_OFF_VERTEX
+    ROUND_OFF_VERTEX,
+    SWAP_GATES,
+    PLACE_PICTURE,
 };
 } // namespace horizon
