@@ -14,6 +14,15 @@ public:
     {
         return true;
     }
+    std::set<InToolActionID> get_actions() const override
+    {
+        using I = InToolActionID;
+        return {
+                I::LMB,
+                I::CANCEL,
+                I::RMB,
+        };
+    }
 
 private:
     class Junction *temp = 0;
