@@ -4,7 +4,6 @@
 #include "imp/imp_interface.hpp"
 #include "tool_helper_move.hpp"
 #include <sstream>
-#include <gdk/gdkkeysyms.h>
 
 namespace horizon {
 
@@ -23,7 +22,7 @@ ToolResponse ToolDrawLineNet::begin(const ToolArgs &args)
     std::cout << "tool draw net line junction\n";
 
     temp_junc_head = doc.c->insert_junction(UUID::random());
-    temp_junc_head->temp = true;
+    imp->set_snap_filter({{ObjectType::JUNCTION, temp_junc_head->uuid}});
     temp_junc_head->position = args.coords;
     selection.clear();
 
@@ -31,16 +30,27 @@ ToolResponse ToolDrawLineNet::begin(const ToolArgs &args)
     return ToolResponse();
 }
 
+void ToolDrawLineNet::set_snap_filter()
+{
+    std::set<SnapFilter> sf;
+    if (temp_junc_head)
+        sf.emplace(ObjectType::JUNCTION, temp_junc_head->uuid);
+    if (temp_junc_mid)
+        sf.emplace(ObjectType::JUNCTION, temp_junc_mid->uuid);
+    imp->set_snap_filter(sf);
+}
+
 void ToolDrawLineNet::restart(const Coordi &c)
 {
     doc.c->get_schematic()->expand(); // gets rid of warnings...
     temp_junc_head = doc.c->insert_junction(UUID::random());
-    temp_junc_head->temp = true;
     temp_junc_head->position = c;
+    temp_junc_mid = nullptr;
     temp_line_head = nullptr;
     temp_line_mid = nullptr;
     component_floating = nullptr;
     net_label = nullptr;
+    set_snap_filter();
     update_tip();
 }
 
@@ -67,23 +77,29 @@ Junction *ToolDrawLineNet::make_temp_junc(const Coordi &c)
 {
     Junction *j = doc.r->insert_junction(UUID::random());
     j->position = c;
-    j->temp = true;
     return j;
 }
 
 void ToolDrawLineNet::update_tip()
 {
+    std::vector<ActionLabelInfo> actions;
+    actions.reserve(12);
+    actions.emplace_back(InToolActionID::LMB, "connect");
+    actions.emplace_back(InToolActionID::RMB);
+    actions.emplace_back(InToolActionID::PLACE_JUNCTION);
+    actions.emplace_back(InToolActionID::COMMIT, "finish");
+    actions.emplace_back(InToolActionID::POSTURE);
+    actions.emplace_back(InToolActionID::ARBITRARY_ANGLE_MODE);
+
     std::stringstream ss;
-    ss << "<b>LMB:</b>place junction/connect <b>RMB:</b>finish and delete last "
-          "segment <b>space:</b>place junction <b>⏎:</b>finish <b>/:</b>line "
-          "posture <b>a:</b>arbitrary angle ";
     if (net_label) {
-        ss << "<b>+-:</b>change label size <b>s:</b>set label size ";
+        actions.emplace_back(InToolActionID::NET_LABEL_SIZE_INC, InToolActionID::NET_LABEL_SIZE_DEC, "label size");
+        actions.emplace_back(InToolActionID::ENTER_SIZE, "enter label size");
+        actions.emplace_back(InToolActionID::ROTATE, "rotate label");
     }
     if (temp_line_head) {
         if (temp_line_head->net) {
-            ss << "<b>b:</b>toggle label <b>r:</b>rotate label";
-            ss << " <i>";
+            actions.emplace_back(InToolActionID::TOGGLE_NET_LABEL);
             if (temp_line_head->net->name.size()) {
                 ss << "drawing net \"" << temp_line_head->net->name << "\"";
             }
@@ -92,20 +108,17 @@ void ToolDrawLineNet::update_tip()
             }
         }
         else if (temp_line_head->bus) {
-            ss << " <i>";
             ss << "drawing bus \"" << temp_line_head->bus->name << "\"";
         }
         else {
-            ss << " <i>";
             ss << "drawing no net";
         }
     }
     else {
-        ss << " <i>";
         ss << "select starting point";
     }
-    ss << "</i>";
     imp->tool_bar_set_tip(ss.str());
+    imp->tool_bar_set_actions(actions);
 }
 
 void ToolDrawLineNet::apply_settings()
@@ -120,8 +133,9 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
     if (args.type == ToolEventType::MOVE) {
         move_temp_junc(args.coords);
     }
-    else if (args.type == ToolEventType::CLICK) {
-        if (args.button == 1) {
+    else if (args.type == ToolEventType::ACTION) {
+        switch (args.action) {
+        case InToolActionID::LMB: {
             Junction *ju = nullptr;
             SchematicSymbol *sym = nullptr;
             uuid_ptr<SymbolPin> pin = nullptr;
@@ -177,23 +191,14 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
                     for (auto it : doc.c->get_net_lines()) {
                         if (it->coord_on_line(temp_junc_head->position)) {
                             if (it != temp_line_head && it != temp_line_mid) {
-                                bool is_temp_line = false;
-                                for (const auto &it_ft : {it->from, it->to}) {
-                                    if (it_ft.is_junc()) {
-                                        if (it_ft.junc->temp)
-                                            is_temp_line = true;
-                                    }
-                                }
-                                if (!is_temp_line) {
-                                    imp->tool_bar_flash("split net line");
-                                    auto li = doc.c->get_sheet()->split_line_net(it, ju);
-                                    net = li->net;
-                                    bus = li->bus;
-                                    ju->net = li->net;
-                                    ju->bus = li->bus;
-                                    ju->connection_count = 3;
-                                    break;
-                                }
+                                imp->tool_bar_flash("split net line");
+                                auto li = doc.c->get_sheet()->split_line_net(it, ju);
+                                net = li->net;
+                                bus = li->bus;
+                                ju->net = li->net;
+                                ju->bus = li->bus;
+                                ju->connection_count = 3;
+                                break;
                             }
                         }
                     }
@@ -204,6 +209,7 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
                 temp_junc_mid->bus = bus;
                 temp_junc_head->net = net;
                 temp_junc_head->bus = bus;
+                set_snap_filter();
             }
             else { // already have net line
                 component_floating = nullptr;
@@ -289,31 +295,22 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
                     for (auto it : doc.c->get_net_lines()) {
                         if (it->coord_on_line(temp_junc_head->position)) {
                             if (it != temp_line_head && it != temp_line_mid) {
-                                bool is_temp_line = false;
-                                for (const auto &it_ft : {it->from, it->to}) {
-                                    if (it_ft.is_junc()) {
-                                        if (it_ft.junc->temp)
-                                            is_temp_line = true;
-                                    }
-                                }
-                                if (!is_temp_line) {
-                                    imp->tool_bar_flash("split net line");
-                                    net = it->net;
-                                    bus = it->bus;
+                                imp->tool_bar_flash("split net line");
+                                net = it->net;
+                                bus = it->bus;
 
-                                    auto do_merge =
-                                            merge_bus_net(temp_line_head->net, temp_line_head->bus, it->net, it->bus);
-                                    if (do_merge) {
-                                        doc.c->get_sheet()->split_line_net(it, ju);
-                                        restart(args.coords);
-                                        return ToolResponse();
-                                    }
-                                    else {
-                                        return ToolResponse(); // bus-net
-                                                               // illegal
-                                    }
-                                    break;
+                                auto do_merge =
+                                        merge_bus_net(temp_line_head->net, temp_line_head->bus, it->net, it->bus);
+                                if (do_merge) {
+                                    doc.c->get_sheet()->split_line_net(it, ju);
+                                    restart(args.coords);
+                                    return ToolResponse();
                                 }
+                                else {
+                                    return ToolResponse(); // bus-net
+                                                           // illegal
+                                }
+                                break;
                             }
                         }
                     }
@@ -326,6 +323,7 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
                     temp_junc_head = make_temp_junc(args.coords);
                     temp_junc_head->net = net;
                     temp_junc_head->bus = bus;
+                    set_snap_filter();
                 }
             }
             temp_line_mid = doc.c->insert_line_net(UUID::random());
@@ -350,8 +348,9 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
             temp_line_head->bus = bus;
             temp_line_head->from.connect(temp_junc_mid);
             temp_line_head->to.connect(temp_junc_head);
-        }
-        else if (args.button == 3) {
+        } break;
+
+        case InToolActionID::RMB:
             if (temp_line_head) {
                 doc.c->delete_line_net(temp_line_head->uuid);
                 doc.c->delete_line_net(temp_line_mid->uuid);
@@ -365,10 +364,9 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
                 doc.r->delete_junction(temp_junc_head->uuid);
                 return ToolResponse::commit();
             }
-        }
-    }
-    else if (args.type == ToolEventType::KEY) {
-        if (args.key == GDK_KEY_Escape) {
+            break;
+
+        case InToolActionID::CANCEL:
             if (temp_line_head) {
                 doc.c->delete_line_net(temp_line_head->uuid);
                 doc.c->delete_line_net(temp_line_mid->uuid);
@@ -378,8 +376,8 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
             }
             doc.r->delete_junction(temp_junc_head->uuid);
             return ToolResponse::commit();
-        }
-        else if (args.key == GDK_KEY_space) {
+
+        case InToolActionID::PLACE_JUNCTION: {
             Junction *ju = temp_junc_head;
             temp_junc_mid = make_temp_junc(args.coords);
             temp_junc_mid->net = ju->net;
@@ -400,11 +398,14 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
             temp_line_head->bus = ju->bus;
             temp_line_head->from.connect(temp_junc_mid);
             temp_line_head->to.connect(temp_junc_head);
-        }
-        else if (args.key == GDK_KEY_Return) {
+
+            set_snap_filter();
+        } break;
+
+        case InToolActionID::COMMIT:
             return ToolResponse::commit();
-        }
-        else if (args.key == GDK_KEY_slash) {
+
+        case InToolActionID::POSTURE:
             switch (bend_mode) {
             case BendMode::XY:
                 bend_mode = BendMode::YX;
@@ -416,8 +417,14 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
                 bend_mode = BendMode::XY;
             }
             move_temp_junc(args.coords);
-        }
-        else if (args.key == GDK_KEY_b) {
+            break;
+
+        case InToolActionID::ARBITRARY_ANGLE_MODE:
+            bend_mode = BendMode::ARB;
+            move_temp_junc(args.coords);
+            break;
+
+        case InToolActionID::TOGGLE_NET_LABEL:
             if (temp_junc_head && !temp_junc_head->bus) {
                 if (!net_label) {
                     auto uu = UUID::random();
@@ -430,27 +437,29 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
                     net_label = nullptr;
                 }
             }
-        }
-        else if (args.key == GDK_KEY_plus || args.key == GDK_KEY_equal) {
+            break;
+
+        case InToolActionID::NET_LABEL_SIZE_INC:
             if (net_label)
                 step_net_label_size(true);
-        }
-        else if (args.key == GDK_KEY_minus) {
+            break;
+
+        case InToolActionID::NET_LABEL_SIZE_DEC:
             if (net_label)
                 step_net_label_size(false);
-        }
-        else if (args.key == GDK_KEY_s) {
+            break;
+
+        case InToolActionID::ENTER_SIZE:
             if (net_label)
                 ask_net_label_size();
-        }
-        else if (args.key == GDK_KEY_r) {
+            break;
+
+        case InToolActionID::ROTATE:
             if (net_label)
-                net_label->orientation =
-                        ToolHelperMove::transform_orientation(net_label->orientation, args.key == GDK_KEY_r);
-        }
-        else if (args.key == GDK_KEY_a) {
-            bend_mode = BendMode::ARB;
-            move_temp_junc(args.coords);
+                net_label->orientation = ToolHelperMove::transform_orientation(net_label->orientation, true);
+            break;
+
+        default:;
         }
     }
     update_tip();
