@@ -11,9 +11,24 @@ public:
     ToolResponse begin(const ToolArgs &args) override;
     ToolResponse update(const ToolArgs &args) override;
     bool can_begin() override;
-    bool handles_esc() override
+
+    std::set<InToolActionID> get_actions() const override
     {
-        return true;
+        using I = InToolActionID;
+        return {
+                I::LMB,
+                I::CANCEL,
+                I::RMB,
+                I::PLACE_JUNCTION,
+                I::COMMIT,
+                I::POSTURE,
+                I::ARBITRARY_ANGLE_MODE,
+                I::NET_LABEL_SIZE_INC,
+                I::NET_LABEL_SIZE_DEC,
+                I::ENTER_SIZE,
+                I::ROTATE,
+                I::TOGGLE_NET_LABEL,
+        };
     }
 
 private:
@@ -34,5 +49,6 @@ private:
 
     Junction *make_temp_junc(const Coordi &c);
     void apply_settings() override;
+    void set_snap_filter();
 };
 } // namespace horizon
