@@ -11,6 +11,13 @@ public:
     ToolResponse begin(const ToolArgs &args) override;
     ToolResponse update(const ToolArgs &args) override;
     bool can_begin() override;
+    std::set<InToolActionID> get_actions() const override
+    {
+        using I = InToolActionID;
+        return {
+                I::LMB, I::CANCEL, I::RMB, I::FLIP_ARC, I::TOGGLE_ARC, I::RESTRICT,
+        };
+    }
 
 private:
     Polygon *temp = nullptr;
@@ -20,5 +27,6 @@ private:
     ArcMode arc_mode = ArcMode::OFF;
     void update_tip();
     void update_vertex(const Coordi &c);
+    void set_snap_filter();
 };
 } // namespace horizon
