@@ -13,10 +13,17 @@ public:
     {
         return true;
     }
+    std::set<InToolActionID> get_actions() const override
+    {
+        using I = InToolActionID;
+        return {
+                I::LMB,
+                I::CANCEL,
+                I::RMB,
+        };
+    }
 
 private:
-    enum class Mode { CENTER, CORNER };
-
     std::array<class Junction *, 4> junctions;
 
     void update_junctions(const Coordi &c);
