@@ -1,6 +1,7 @@
 #include "tool_helper_map_symbol.hpp"
 #include "document/idocument_schematic.hpp"
 #include "schematic/schematic.hpp"
+#include "canvas/canvas_gl.hpp"
 #include "imp/imp_interface.hpp"
 #include "util/sqlite.hpp"
 
@@ -19,10 +20,11 @@ const Symbol *ToolHelperMapSymbol::get_symbol_for_unit(const UUID &unit_uu, bool
     }
     if (auto_selected)
         *auto_selected = false;
-    bool r = false;
     if (n != 1) {
-        std::tie(r, selected_symbol) = imp->dialogs.select_symbol(doc.r->get_pool(), unit_uu);
-        if (!r) {
+        if (auto r = imp->dialogs.select_symbol(doc.r->get_pool(), unit_uu)) {
+            selected_symbol = *r;
+        }
+        else {
             return nullptr;
         }
     }
@@ -46,7 +48,11 @@ SchematicSymbol *ToolHelperMapSymbol::map_symbol(Component *comp, const Gate *ga
     SchematicSymbol *schsym = doc.c->insert_schematic_symbol(UUID::random(), sym);
     schsym->component = comp;
     schsym->gate = gate;
-    doc.c->get_sheet()->expand_symbols(*doc.c->get_schematic());
+    auto bb = schsym->symbol.get_bbox(true);
+    auto sz = bb.second - bb.first;
+    imp->get_canvas()->ensure_min_size(sz.x * 1.5, sz.y * 1.5);
+
+    doc.c->get_sheet()->expand_symbol(schsym->uuid, *doc.c->get_schematic());
 
     return schsym;
 }
