@@ -10,12 +10,14 @@
 #include "document/idocument_symbol.hpp"
 #include "pool/symbol.hpp"
 #include "board/board_layers.hpp"
+#include "imp/imp_interface.hpp"
 
 namespace horizon {
 void ToolHelperMove::move_init(const Coordi &c)
 {
     last = c;
     origin = c;
+    imp->set_snap_filter_from_selection(selection);
 }
 
 static Coordi *get_dim_coord(Dimension *dim, int vertex)
@@ -74,6 +76,9 @@ void ToolHelperMove::move_do(const Coordi &delta)
         case ObjectType::BOARD_PANEL:
             doc.b->get_board()->board_panels.at(it.uuid).placement.shift += delta;
             break;
+        case ObjectType::PICTURE:
+            doc.r->get_picture(it.uuid)->placement.shift += delta;
+            break;
         case ObjectType::DIMENSION: {
             auto dim = doc.r->get_dimension(it.uuid);
             if (it.vertex < 2) {
@@ -325,6 +330,14 @@ void ToolHelperMove::move_mirror_or_rotate(const Coordi &center, bool rotate)
             }
         } break;
 
+        case ObjectType::PICTURE: {
+            auto pic = doc.r->get_picture(it.uuid);
+            transform(pic->placement.shift, center, rotate);
+            if (rotate) {
+                pic->placement.inc_angle_deg(-90);
+            }
+        } break;
+
         case ObjectType::SHAPE: {
             Shape *shape = &doc.a->get_padstack()->shapes.at(it.uuid);
             transform(shape->placement.shift, center, rotate);
