@@ -14,6 +14,7 @@
 #include <iostream>
 #include <gtkmm.h>
 #include "core/tool_id.hpp"
+#include "util/picture_util.hpp"
 
 namespace horizon {
 
@@ -70,7 +71,11 @@ public:
     ToolDataPaste(const json &j) : paste_data(j)
     {
     }
+    ToolDataPaste(Glib::RefPtr<Gdk::Pixbuf> pb) : pixbuf(pb)
+    {
+    }
     json paste_data;
+    Glib::RefPtr<Gdk::Pixbuf> pixbuf;
 };
 
 ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canvas)
@@ -243,6 +248,7 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
             for (auto &it_txt : x->texts) {
                 it_txt = doc.r->get_text(text_xlat.at(it_txt.uuid));
             }
+            sheet->expand_symbol(u, *doc.c->get_schematic());
             x->placement.shift += shift;
             selection.emplace(u, ObjectType::SCHEMATIC_SYMBOL);
         }
@@ -435,6 +441,13 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
 ToolResponse ToolPaste::begin(const ToolArgs &args)
 {
     update_tip();
+    imp->tool_bar_set_actions({
+            {InToolActionID::LMB},
+            {InToolActionID::RMB},
+            {InToolActionID::ROTATE},
+            {InToolActionID::MIRROR},
+            {InToolActionID::RESTRICT},
+    });
     if (auto data = dynamic_cast<ToolDataPaste *>(args.data.get())) {
         paste_data = data->paste_data;
         return begin_paste(paste_data, args.coords);
@@ -448,12 +461,21 @@ ToolResponse ToolPaste::begin(const ToolArgs &args)
         return begin_paste(paste_data, args.coords);
     }
     else {
-        auto ref_clipboard = Gtk::Clipboard::get();
-        ref_clipboard->request_contents("imp-buffer", [this](const Gtk::SelectionData &sel_data) {
-            auto td = std::make_unique<ToolDataPaste>(nullptr);
-            if (sel_data.gobj() && sel_data.get_data_type() == "imp-buffer")
-                td->paste_data = json::parse(sel_data.get_data_as_string());
-            imp->tool_update_data(std::move(td));
+        Gtk::Clipboard::get()->request_contents("imp-buffer", [this](const Gtk::SelectionData &sel_data) {
+            if (sel_data.gobj() && sel_data.get_data_type() == "imp-buffer") {
+                auto td = std::make_unique<ToolDataPaste>(json::parse(sel_data.get_data_as_string()));
+                imp->tool_update_data(std::move(td));
+            }
+            else if (doc.r->has_object_type(ObjectType::PICTURE)) {
+                Gtk::Clipboard::get()->request_image([this](const Glib::RefPtr<Gdk::Pixbuf> &pb) {
+                    auto td = std::make_unique<ToolDataPaste>(pb);
+                    imp->tool_update_data(std::move(td));
+                });
+            }
+            else {
+                auto td = std::make_unique<ToolDataPaste>(nullptr);
+                imp->tool_update_data(std::move(td));
+            }
         });
         return ToolResponse();
     }
@@ -461,40 +483,53 @@ ToolResponse ToolPaste::begin(const ToolArgs &args)
 
 void ToolPaste::update_tip()
 {
-    if (paste_data == nullptr) { // wait for data
+    if (paste_data == nullptr && pic == nullptr) { // wait for data
         imp->tool_bar_set_tip("waiting for paste data");
-        return;
     }
-    auto delta = get_delta();
-
-    std::string s =
-            "<b>LMB:</b>place <b>RMB:</b>cancel <b>r:</b>rotate "
-            "<b>e:</b>mirror <b>/:</b>restrict <i>"
-            + coord_to_string(delta, true) + " " + restrict_mode_to_string() + "</i> ";
-    imp->tool_bar_set_tip(s);
+    else {
+        auto delta = get_delta();
+        imp->tool_bar_set_tip(coord_to_string(delta, true) + " " + restrict_mode_to_string());
+    }
 }
 
 ToolResponse ToolPaste::update(const ToolArgs &args)
 {
-    if (paste_data == nullptr) { // wait for data
+    if (paste_data == nullptr && pic == nullptr) { // wait for data
         if (args.type == ToolEventType::DATA) {
             auto data = dynamic_cast<ToolDataPaste *>(args.data.get());
-            if (data->paste_data == nullptr) {
+            if (data->paste_data != nullptr) {
+                paste_data = data->paste_data;
+                return begin_paste(paste_data, args.coords);
+            }
+            else if (data->pixbuf) {
+                pic = doc.r->insert_picture(UUID::random());
+                pic->placement.shift = args.coords;
+                pic->data = picture_data_from_pixbuf(data->pixbuf);
+                pic->data_uuid = pic->data->uuid;
+                float width = 10_mm;
+                pic->px_size = width / pic->data->width;
+                selection.clear();
+                selection.emplace(pic->uuid, ObjectType::PICTURE);
+                update_tip();
+                move_init(args.coords);
+            }
+            else {
                 imp->tool_bar_flash("Empty Buffer");
                 return ToolResponse::end();
             }
-            paste_data = data->paste_data;
-            return begin_paste(paste_data, args.coords);
         }
     }
     else {
         if (args.type == ToolEventType::MOVE) {
             move_do_cursor(args.coords);
             update_tip();
-            return ToolResponse::fast();
+            return ToolResponse();
         }
-        else if (args.type == ToolEventType::CLICK || (is_transient && args.type == ToolEventType::CLICK_RELEASE)) {
-            if (args.button == 1) {
+        else if (args.type == ToolEventType::ACTION) {
+            if (args.action == InToolActionID::LMB || (is_transient && args.action == InToolActionID::LMB_RELEASE)) {
+                if (pic) {
+                    return ToolResponse::commit();
+                }
                 merge_selected_junctions();
                 for (const auto &it : selection) {
                     if (it.type == ObjectType::SCHEMATIC_SYMBOL) {
@@ -508,21 +543,15 @@ ToolResponse ToolPaste::update(const ToolArgs &args)
                 return ToolResponse::next(ToolResponse::Result::COMMIT, tool_id,
                                           std::make_unique<ToolDataPaste>(paste_data));
             }
-            else {
+            else if (any_of(args.action, {InToolActionID::RMB, InToolActionID::CANCEL})) {
                 return ToolResponse::revert();
             }
-        }
-        else if (args.type == ToolEventType::KEY) {
-            if (args.key == GDK_KEY_Escape) {
-                return ToolResponse::revert();
-            }
-            else if (args.key == GDK_KEY_slash) {
+            else if (args.action == InToolActionID::RESTRICT) {
                 cycle_restrict_mode();
                 move_do_cursor(args.coords);
             }
-            else if (args.key == GDK_KEY_r || args.key == GDK_KEY_e) {
-                bool rotate = args.key == GDK_KEY_r;
-                move_mirror_or_rotate(args.coords, rotate);
+            else if (any_of(args.action, {InToolActionID::ROTATE, InToolActionID::MIRROR})) {
+                move_mirror_or_rotate(args.coords, args.action == InToolActionID::ROTATE);
             }
         }
     }
