@@ -8,6 +8,16 @@ public:
     ToolResponse begin(const ToolArgs &args) override;
     ToolResponse update(const ToolArgs &args) override;
     bool can_begin() override;
+    std::set<InToolActionID> get_actions() const override
+    {
+        using I = InToolActionID;
+        return {
+                I::LMB,
+                I::CANCEL,
+                I::RMB,
+                I::ROTATE,
+        };
+    }
 
 private:
     const class IncludedBoard *inc_board = nullptr;
