@@ -10,6 +10,13 @@ class ToolPlacePowerSymbol : public ToolPlaceJunction {
 public:
     ToolPlacePowerSymbol(IDocument *c, ToolID tid);
     bool can_begin() override;
+    std::set<InToolActionID> get_actions() const override
+    {
+        using I = InToolActionID;
+        return {
+                I::LMB, I::CANCEL, I::RMB, I::ROTATE, I::MIRROR,
+        };
+    }
 
 protected:
     void create_attached() override;
