@@ -25,7 +25,7 @@ ToolRotateArbitrary::ToolRotateArbitrary(IDocument *c, ToolID tid) : ToolBase(c,
 void ToolRotateArbitrary::expand_selection()
 {
     std::set<SelectableRef> new_sel;
-    std::set<SelectableRef> pkgs_fixed;
+    std::set<SelectableRef> sel_remove;
     for (const auto &it : selection) {
         switch (it.type) {
         case ObjectType::LINE: {
@@ -59,7 +59,7 @@ void ToolRotateArbitrary::expand_selection()
         case ObjectType::BOARD_PACKAGE: {
             BoardPackage *pkg = &doc.b->get_board()->packages.at(it.uuid);
             if (pkg->fixed) {
-                pkgs_fixed.insert(it);
+                sel_remove.insert(it);
             }
             else {
                 for (const auto &itt : pkg->texts) {
@@ -72,10 +72,23 @@ void ToolRotateArbitrary::expand_selection()
         }
     }
     selection.insert(new_sel.begin(), new_sel.end());
-    if (pkgs_fixed.size() && imp)
+
+    if (sel_remove.size() && imp)
         imp->tool_bar_flash("can't move fixed package");
+
+    if (doc.c) {
+        for (const auto &it : selection) {
+            if (it.type == ObjectType::JUNCTION) {
+                const auto ju = doc.r->get_junction(it.uuid);
+                if (ju->net) {
+                    sel_remove.insert(it);
+                }
+            }
+        }
+    }
+
     for (auto it = selection.begin(); it != selection.end();) {
-        if (pkgs_fixed.count(*it))
+        if (sel_remove.count(*it))
             it = selection.erase(it);
         else
             ++it;
@@ -107,40 +120,44 @@ ToolRotateArbitrary::~ToolRotateArbitrary()
 
 bool ToolRotateArbitrary::can_begin()
 {
-    if (doc.c) // dont' rotate arbitraryily in schematic
-        return false;
     expand_selection();
     return selection.size() > 0;
 }
 
 void ToolRotateArbitrary::update_tip()
 {
+    std::vector<ActionLabelInfo> actions;
+    actions.reserve(8);
+
+
     if (state == State::ORIGIN) {
-        imp->tool_bar_set_tip("<b>LMB:</b>set origin <b>RMB:</b>cancel");
+        actions.emplace_back(InToolActionID::LMB, "set origin");
+        actions.emplace_back(InToolActionID::RMB);
     }
     else if (state == State::REF) {
-        imp->tool_bar_set_tip("<b>LMB:</b>set ref <b>RMB:</b>cancel");
+        actions.emplace_back(InToolActionID::LMB, "set ref");
+        actions.emplace_back(InToolActionID::RMB);
     }
     else if (state == State::ROTATE) {
-        std::stringstream ss;
-        ss << "<b>LMB:</b>finish <b>RMB:</b>cancel <b>s:</b>toggle snap <b>Enter:</b>enter angle <i>";
-        ss << "Angle: " << angle_to_string(iangle, false);
+        actions.emplace_back(InToolActionID::LMB, "finish");
+        actions.emplace_back(InToolActionID::RMB);
+        actions.emplace_back(InToolActionID::TOGGLE_ANGLE_SNAP, "toggle snap");
+        actions.emplace_back(InToolActionID::ENTER_DATUM, "enter angle");
+
+        std::string s = "Angle: " + angle_to_string(iangle, false);
         if (snap)
-            ss << " (snapped)";
-        ss << "</i>";
-        imp->tool_bar_set_tip(ss.str());
+            s += " (snapped)";
+        imp->tool_bar_set_tip(s);
     }
     else if (state == State::SCALE) {
+        actions.emplace_back(InToolActionID::LMB, "finish");
+        actions.emplace_back(InToolActionID::RMB);
         std::stringstream ss;
-        ss << "<b>LMB:</b>finish <b>RMB:</b>cancel <i>";
         ss << "Scale: " << scale;
-        ss << "</i>";
         imp->tool_bar_set_tip(ss.str());
     }
-    // auto delta = last-origin;
 
-    // imp->tool_bar_set_tip("<b>LMB:</b>place <b>RMB:</b>cancel <b>r:</b>rotate
-    // <b>e:</b>mirror "+coord_to_string(delta, true));
+    imp->tool_bar_set_actions(actions);
 }
 
 void ToolRotateArbitrary::save_placements()
@@ -277,13 +294,15 @@ ToolResponse ToolRotateArbitrary::update(const ToolArgs &args)
             apply_placements_rotation(iangle);
         }
         update_tip();
-        return ToolResponse::fast();
+        return ToolResponse();
     }
-    else if (args.type == ToolEventType::CLICK) {
-        if (args.button == 1) {
+    else if (args.type == ToolEventType::ACTION) {
+        switch (args.action) {
+        case InToolActionID::LMB:
             if (tool_id == ToolID::ROTATE_ARBITRARY) {
                 if (state == State::ORIGIN) {
                     state = State::ROTATE;
+                    imp->set_snap_filter_from_selection(selection);
                     update_tip();
                 }
                 else {
@@ -298,30 +317,31 @@ ToolResponse ToolRotateArbitrary::update(const ToolArgs &args)
                 }
                 else if (state == State::REF) {
                     state = State::SCALE;
+                    imp->set_snap_filter_from_selection(selection);
                     update_tip();
                 }
                 else {
                     return ToolResponse::commit();
                 }
             }
-        }
-        else {
+            break;
+
+        case InToolActionID::RMB:
+        case InToolActionID::CANCEL:
             return ToolResponse::revert();
-        }
-    }
-    else if (args.type == ToolEventType::KEY) {
-        if (args.key == GDK_KEY_s) {
+
+        case InToolActionID::TOGGLE_ANGLE_SNAP:
             snap ^= true;
-        }
-        else if (args.key == GDK_KEY_Return) {
-            auto r = imp->dialogs.ask_datum_angle("Enter angle", 0);
-            if (r.first) {
-                apply_placements_rotation(r.second);
+            break;
+
+        case InToolActionID::ENTER_DATUM: {
+            if (auto r = imp->dialogs.ask_datum_angle("Enter angle", 0)) {
+                apply_placements_rotation(*r);
                 return ToolResponse::commit();
             }
-        }
-        else if (args.key == GDK_KEY_Escape) {
-            return ToolResponse::revert();
+        } break;
+
+        default:;
         }
     }
     return ToolResponse();
