@@ -11,7 +11,11 @@
 #include "router/router/pns_meander_placer_base.h"
 #include "util/util.hpp"
 #include "core/tool_id.hpp"
+#include "util/selection_util.hpp"
 #include <iostream>
+#include <iomanip>
+#include "nlohmann/json.hpp"
+#include "core/tool_data_window.hpp"
 
 namespace horizon {
 
@@ -40,6 +44,30 @@ ToolRouteTrackInteractive::ToolRouteTrackInteractive(IDocument *c, ToolID tid) :
 {
 }
 
+json ToolRouteTrackInteractive::Settings::serialize() const
+{
+    json j;
+    j["effort"] = effort;
+    j["remove_loops"] = remove_loops;
+    return j;
+}
+
+void ToolRouteTrackInteractive::Settings::load_from_json(const json &j)
+{
+    effort = j.value("effort", 1);
+    remove_loops = j.value("remove_loops", true);
+}
+
+void ToolRouteTrackInteractive::apply_settings()
+{
+    if (!router)
+        return;
+    PNS::ROUTING_SETTINGS routing_settings(router->Settings());
+    routing_settings.SetRemoveLoops(settings.remove_loops);
+    routing_settings.SetOptimizerEffort(static_cast<PNS::PNS_OPTIMIZATION_EFFORT>(settings.effort));
+    router->LoadSettings(routing_settings);
+}
+
 bool ToolRouteTrackInteractive::is_tune() const
 {
     switch (tool_id) {
@@ -56,16 +84,25 @@ bool ToolRouteTrackInteractive::can_begin()
 {
     if (!doc.b)
         return false;
+    auto via = get_via(selection);
+    auto track = get_track(selection);
     switch (tool_id) {
     case ToolID::DRAG_TRACK_INTERACTIVE:
+        if (via && track)
+            return false;
+        else if (via)
+            return via->junction->net;
+        else if (track)
+            return track->net;
+        else
+            return false;
+
     case ToolID::TUNE_TRACK:
-        return get_track(selection);
+        return track && track->net;
 
     case ToolID::TUNE_DIFFPAIR:
-    case ToolID::TUNE_DIFFPAIR_SKEW: {
-        auto track = get_track(selection);
+    case ToolID::TUNE_DIFFPAIR_SKEW:
         return track && track->net && track->net->diffpair;
-    }
 
     default:
         return true;
@@ -86,18 +123,18 @@ bool ToolRouteTrackInteractive::is_specific()
 
 Track *ToolRouteTrackInteractive::get_track(const std::set<SelectableRef> &sel)
 {
-    Track *track = nullptr;
-    for (const auto &it : sel) {
-        if (it.type == ObjectType::TRACK) {
-            if (track == nullptr) {
-                track = &doc.b->get_board()->tracks.at(it.uuid);
-            }
-            else {
-                return nullptr;
-            }
-        }
-    }
-    return track;
+    if (sel_count_type(sel, ObjectType::TRACK) == 1)
+        return &doc.b->get_board()->tracks.at(sel_find_one(sel, ObjectType::TRACK).uuid);
+    else
+        return nullptr;
+}
+
+Via *ToolRouteTrackInteractive::get_via(const std::set<SelectableRef> &sel)
+{
+    if (sel_count_type(sel, ObjectType::VIA) == 1)
+        return &doc.b->get_board()->vias.at(sel_find_one(sel, ObjectType::VIA).uuid);
+    else
+        return nullptr;
 }
 
 ToolResponse ToolRouteTrackInteractive::begin(const ToolArgs &args)
@@ -147,24 +184,35 @@ ToolResponse ToolRouteTrackInteractive::begin(const ToolArgs &args)
     router->ClearWorld();
     router->SyncWorld();
 
-    PNS::ROUTING_SETTINGS settings;
-    settings.SetShoveVias(false);
+    PNS::ROUTING_SETTINGS routing_settings;
+    routing_settings.SetShoveVias(false);
 
     PNS::SIZES_SETTINGS sizes_settings;
 
-    router->LoadSettings(settings);
+    router->LoadSettings(routing_settings);
     router->UpdateSizes(sizes_settings);
+    apply_settings();
 
     imp->canvas_update();
     update_tip();
 
     if (tool_id == ToolID::DRAG_TRACK_INTERACTIVE) {
-        Track *track = get_track(args.selection);
-        if (!track) {
-            return ToolResponse::end();
+        auto track = get_track(args.selection);
+        auto via = get_via(args.selection);
+        const PNS::PNS_HORIZON_PARENT_ITEM *parent = nullptr;
+        Net *net = nullptr;
+        if (track) {
+            parent = iface->get_parent(track);
+            net = track->net;
         }
-        auto parent = iface->get_parent(track);
-        wrapper->m_startItem = router->GetWorld()->FindItemByParent(parent, iface->get_net_code(track->net.uuid));
+        else if (via) {
+            parent = iface->get_parent(via);
+            net = via->junction->net;
+        }
+        else
+            return ToolResponse::end();
+
+        wrapper->m_startItem = router->GetWorld()->FindItemByParent(parent, iface->get_net_code(net->uuid));
 
         VECTOR2I p0(args.coords.x, args.coords.y);
         if (!router->StartDragging(p0, wrapper->m_startItem, PNS::DM_ANY))
@@ -180,8 +228,13 @@ ToolResponse ToolRouteTrackInteractive::begin(const ToolArgs &args)
         VECTOR2I p0(args.coords.x, args.coords.y);
         if (!router->StartRouting(p0, wrapper->m_startItem, 0))
             return ToolResponse::end();
-        router->Move(p0, NULL);
         meander_placer = dynamic_cast<PNS::MEANDER_PLACER_BASE *>(router->Placer());
+        if (auto ref = imp->get_length_tuning_ref()) {
+            PNS::MEANDER_SETTINGS meander_settings = meander_placer->MeanderSettings();
+            meander_settings.m_targetLength = ref;
+            meander_placer->UpdateSettings(meander_settings);
+        }
+        router->Move(p0, NULL);
     }
     update_tip();
     return ToolResponse();
@@ -568,8 +621,8 @@ ToolResponse ToolRouteTrackInteractive::update(const ToolArgs &args)
             wrapper->updateEndItem(args);
             router->Move(wrapper->m_endSnapPoint, wrapper->m_endItem);
         }
-        else if (args.type == ToolEventType::CLICK) {
-            if (args.button == 1) {
+        else if (args.type == ToolEventType::ACTION) {
+            if ((args.action == InToolActionID::LMB) || (is_transient && args.action == InToolActionID::LMB_RELEASE)) {
                 wrapper->updateEndItem(args);
                 if (router->FixRoute(wrapper->m_endSnapPoint, wrapper->m_endItem)) {
                     router->StopRouting();
@@ -579,51 +632,60 @@ ToolResponse ToolRouteTrackInteractive::update(const ToolArgs &args)
                     return ToolResponse::commit();
                 }
             }
-            else if (args.button == 3) {
+            else if (any_of(args.action, {InToolActionID::RMB, InToolActionID::CANCEL})) {
                 return ToolResponse::revert();
             }
         }
-        else if (args.type == ToolEventType::KEY) {
-            if (args.key == GDK_KEY_Escape) {
-                board->expand_flags =
-                        static_cast<Board::ExpandFlags>(Board::EXPAND_PROPAGATE_NETS | Board::EXPAND_AIRWIRES);
-                return ToolResponse::commit();
-            }
-        }
     }
     else if (is_tune()) {
         if (args.type == ToolEventType::MOVE) {
             canvas->set_cursor_pos(args.coords);
             router->Move(VECTOR2I(args.coords.x, args.coords.y), NULL);
         }
-        else if (args.type == ToolEventType::CLICK) {
-            if (args.button == 1) {
+        else if (args.type == ToolEventType::ACTION) {
+            PNS::MEANDER_SETTINGS meander_settings = meander_placer->MeanderSettings();
+
+            switch (args.action) {
+            case InToolActionID::LMB:
                 if (router->FixRoute(VECTOR2I(args.coords.x, args.coords.y), NULL)) {
                     router->StopRouting();
                     board->expand_flags =
                             static_cast<Board::ExpandFlags>(Board::EXPAND_PROPAGATE_NETS | Board::EXPAND_AIRWIRES);
                     return ToolResponse::commit();
                 }
-            }
-            else if (args.button == 3) {
-                board->expand_flags =
-                        static_cast<Board::ExpandFlags>(Board::EXPAND_PROPAGATE_NETS | Board::EXPAND_AIRWIRES);
-                return ToolResponse::commit();
-            }
-        }
-        else if (args.type == ToolEventType::KEY) {
-            PNS::MEANDER_SETTINGS settings = meander_placer->MeanderSettings();
-            if (args.key == GDK_KEY_l) {
-                auto r = imp->dialogs.ask_datum("Target length", settings.m_targetLength);
-                if (r.first) {
-                    settings.m_targetLength = r.second;
-                    meander_placer->UpdateSettings(settings);
+                break;
+
+            case InToolActionID::RMB:
+            case InToolActionID::CANCEL:
+                return ToolResponse::revert();
+
+            case InToolActionID::LENGTH_TUNING_LENGTH: {
+                if (auto r = imp->dialogs.ask_datum("Target length", meander_settings.m_targetLength)) {
+                    meander_settings.m_targetLength = *r;
+                    meander_placer->UpdateSettings(meander_settings);
                     router->Move(VECTOR2I(args.coords.x, args.coords.y), NULL);
                 }
-            }
-            else if (args.key == GDK_KEY_less || args.key == GDK_KEY_greater) {
+            } break;
+
+            case InToolActionID::LENGTH_TUNING_SPACING_INC:
+            case InToolActionID::LENGTH_TUNING_SPACING_DEC: {
                 int dir = 0;
-                if (args.key == GDK_KEY_less) {
+                if (args.action == InToolActionID::LENGTH_TUNING_SPACING_DEC) {
+                    dir = -1;
+                }
+                else {
+                    dir = 1;
+                }
+                meander_placer->SpacingStep(dir);
+                imp->tool_bar_flash("Meander spacing: " + dim_to_string(meander_placer->MeanderSettings().m_spacing)
+                                    + " <i>" + meander_placer->TuningInfo() + "</i>");
+                router->Move(VECTOR2I(args.coords.x, args.coords.y), NULL);
+            } break;
+
+            case InToolActionID::LENGTH_TUNING_AMPLITUDE_INC:
+            case InToolActionID::LENGTH_TUNING_AMPLITUDE_DEC: {
+                int dir = 0;
+                if (args.action == InToolActionID::LENGTH_TUNING_AMPLITUDE_DEC) {
                     dir = -1;
                 }
                 else {
@@ -634,24 +696,9 @@ ToolResponse ToolRouteTrackInteractive::update(const ToolArgs &args)
                                     + dim_to_string(meander_placer->MeanderSettings().m_maxAmplitude) + " <i>"
                                     + meander_placer->TuningInfo() + "</i>");
                 router->Move(VECTOR2I(args.coords.x, args.coords.y), NULL);
-            }
-            else if (args.key == GDK_KEY_comma || args.key == GDK_KEY_period) {
-                int dir = 0;
-                if (args.key == GDK_KEY_comma) {
-                    dir = -1;
-                }
-                else {
-                    dir = 1;
-                }
-                meander_placer->SpacingStep(dir);
-                imp->tool_bar_flash("Meander spacing: " + dim_to_string(meander_placer->MeanderSettings().m_spacing)
-                                    + " <i>" + meander_placer->TuningInfo() + "</i>");
-                router->Move(VECTOR2I(args.coords.x, args.coords.y), NULL);
-            }
-            else if (args.key == GDK_KEY_Escape) {
-                board->expand_flags =
-                        static_cast<Board::ExpandFlags>(Board::EXPAND_PROPAGATE_NETS | Board::EXPAND_AIRWIRES);
-                return ToolResponse::commit();
+            } break;
+
+            default:;
             }
         }
     }
@@ -660,19 +707,10 @@ ToolResponse ToolRouteTrackInteractive::update(const ToolArgs &args)
             if (args.type == ToolEventType::MOVE) {
                 wrapper->updateStartItem(args);
             }
-            else if (args.type == ToolEventType::KEY) {
-                if (args.key == GDK_KEY_s) {
-                    shove ^= true;
-                }
-                else if (args.key == GDK_KEY_Escape) {
-                    board->expand_flags =
-                            static_cast<Board::ExpandFlags>(Board::EXPAND_PROPAGATE_NETS | Board::EXPAND_AIRWIRES);
-                    return ToolResponse::commit();
-                }
-            }
-            else if (args.type == ToolEventType::CLICK) {
-                wrapper->updateStartItem(args);
-                if (args.button == 1) {
+            else if (args.type == ToolEventType::ACTION) {
+                switch (args.action) {
+                case InToolActionID::LMB:
+                    wrapper->updateStartItem(args);
                     for (const auto &it : board->keepouts) {
                         if (keepout_hit(it.second, {wrapper->m_startSnapPoint.x, wrapper->m_startSnapPoint.y},
                                         PNS::PNS_HORIZON_IFACE::layer_from_router(wrapper->getStartLayer()))) {
@@ -684,11 +722,19 @@ ToolResponse ToolRouteTrackInteractive::update(const ToolArgs &args)
                     if (!wrapper->prepareInteractive()) {
                         return ToolResponse::end();
                     }
-                }
-                else if (args.button == 3) {
+                    break;
+
+                case InToolActionID::ROUTER_MODE:
+                    shove ^= true;
+                    break;
+
+                case InToolActionID::RMB:
+                case InToolActionID::CANCEL:
                     board->expand_flags =
                             static_cast<Board::ExpandFlags>(Board::EXPAND_PROPAGATE_NETS | Board::EXPAND_AIRWIRES);
                     return ToolResponse::commit();
+
+                default:;
                 }
             }
         }
@@ -697,10 +743,13 @@ ToolResponse ToolRouteTrackInteractive::update(const ToolArgs &args)
                 wrapper->updateEndItem(args);
                 router->Move(wrapper->m_endSnapPoint, wrapper->m_endItem);
             }
-            else if (args.type == ToolEventType::CLICK) {
-                if (args.button == 1) {
+            else if (args.type == ToolEventType::ACTION) {
+                switch (args.action) {
+                case InToolActionID::LMB: {
                     wrapper->updateEndItem(args);
 
+                    const bool was_placing_via = router->IsPlacingVia();
+
                     if (router->FixRoute(wrapper->m_endSnapPoint, wrapper->m_endItem)) {
                         router->StopRouting();
                         imp->canvas_update();
@@ -712,13 +761,26 @@ ToolResponse ToolRouteTrackInteractive::update(const ToolArgs &args)
                     }
                     imp->canvas_update();
 
+
+                    if (was_placing_via) {
+                        auto layer = PNS::PNS_HORIZON_IFACE::layer_from_router(router->GetCurrentLayer());
+                        auto nets = router->GetCurrentNets();
+                        const Net *net = nullptr;
+                        for (auto x : nets) {
+                            net = iface->get_net_for_code(x);
+                        }
+                        layer = rules->get_layer_pair(net, layer);
+                        router->SwitchLayer(PNS::PNS_HORIZON_IFACE::layer_to_router(layer));
+                    }
+
                     // Synchronize the indicated layer
                     imp->set_work_layer(PNS::PNS_HORIZON_IFACE::layer_from_router(router->GetCurrentLayer()));
                     wrapper->updateEndItem(args);
                     router->Move(wrapper->m_endSnapPoint, wrapper->m_endItem);
                     wrapper->m_startItem = NULL;
-                }
-                else if (args.button == 3) {
+                } break;
+
+                case InToolActionID::RMB:
                     router->StopRouting();
                     imp->canvas_update();
                     state = State::START;
@@ -726,6 +788,26 @@ ToolResponse ToolRouteTrackInteractive::update(const ToolArgs &args)
                     imp->update_highlights();
                     update_tip();
                     return ToolResponse();
+
+                case InToolActionID::CANCEL:
+                    router->StopRouting();
+                    board->expand_flags =
+                            static_cast<Board::ExpandFlags>(Board::EXPAND_PROPAGATE_NETS | Board::EXPAND_AIRWIRES);
+                    return ToolResponse::commit();
+
+                case InToolActionID::POSTURE:
+                    router->FlipPosture();
+                    wrapper->updateEndItem(args);
+                    router->Move(wrapper->m_endSnapPoint, wrapper->m_endItem);
+                    break;
+
+                case InToolActionID::TOGGLE_VIA:
+                    router->ToggleViaPlacement();
+                    wrapper->updateEndItem(args);
+                    router->Move(wrapper->m_endSnapPoint, wrapper->m_endItem);
+                    break;
+
+                default:;
                 }
             }
             else if (args.type == ToolEventType::LAYER_CHANGE) {
@@ -735,37 +817,20 @@ ToolResponse ToolRouteTrackInteractive::update(const ToolArgs &args)
                     router->Move(wrapper->m_endSnapPoint, wrapper->m_endItem);
                 }
             }
-            else if (args.type == ToolEventType::KEY) {
-                if (args.key == GDK_KEY_slash) {
-                    router->FlipPosture();
-                    wrapper->updateEndItem(args);
-                    router->Move(wrapper->m_endSnapPoint, wrapper->m_endItem);
-                }
-                else if (args.key == GDK_KEY_v) {
-                    router->ToggleViaPlacement();
-                    wrapper->updateEndItem(args);
-                    router->Move(wrapper->m_endSnapPoint, wrapper->m_endItem);
-                }
-                else if (args.key == GDK_KEY_Escape) {
-                    router->StopRouting();
-                    board->expand_flags =
-                            static_cast<Board::ExpandFlags>(Board::EXPAND_PROPAGATE_NETS | Board::EXPAND_AIRWIRES);
-                    return ToolResponse::commit();
-                }
-            }
         }
-        if (args.type == ToolEventType::KEY) {
-            if (args.key == GDK_KEY_w) {
+        if (args.type == ToolEventType::ACTION) {
+            switch (args.action) {
+            case InToolActionID::ENTER_WIDTH: {
                 PNS::SIZES_SETTINGS sz(router->Sizes());
-                auto r = imp->dialogs.ask_datum("Track width", sz.TrackWidth());
-                if (r.first) {
-                    sz.SetTrackWidth(r.second);
+                if (auto r = imp->dialogs.ask_datum("Track width", sz.TrackWidth())) {
+                    sz.SetTrackWidth(*r);
                     sz.SetWidthFromRules(false);
                     router->UpdateSizes(sz);
                     router->Move(wrapper->m_endSnapPoint, wrapper->m_endItem);
                 }
-            }
-            else if (args.key == GDK_KEY_W) {
+            } break;
+
+            case InToolActionID::TRACK_WIDTH_DEFAULT: {
                 auto nets = router->GetCurrentNets();
                 Net *net = nullptr;
                 for (auto x : nets) {
@@ -779,17 +844,32 @@ ToolResponse ToolRouteTrackInteractive::update(const ToolArgs &args)
                     router->UpdateSizes(sz);
                     router->Move(wrapper->m_endSnapPoint, wrapper->m_endItem);
                 }
-            }
-            else if (args.key == GDK_KEY_o) {
-                auto r = imp->dialogs.ask_datum("Routing offset", .1_mm);
-                if (r.first) {
-                    iface->set_override_routing_offset(r.second);
+            } break;
+
+            case InToolActionID::CLEARANCE_OFFSET: {
+                if (auto r = imp->dialogs.ask_datum("Routing offset", .1_mm)) {
+                    iface->set_override_routing_offset(*r);
                     router->Move(wrapper->m_endSnapPoint, wrapper->m_endItem);
                 }
-            }
-            else if (args.key == GDK_KEY_O) {
+            } break;
+
+            case InToolActionID::CLEARANCE_OFFSET_DEFAULT: {
                 iface->set_override_routing_offset(-1);
                 router->Move(wrapper->m_endSnapPoint, wrapper->m_endItem);
+            } break;
+
+            case InToolActionID::ROUTER_SETTINGS:
+                imp->dialogs.show_router_settings_window(settings);
+                break;
+
+            default:;
+            }
+        }
+        else if (args.type == ToolEventType::DATA) {
+            if (auto data = dynamic_cast<const ToolDataWindow *>(args.data.get())) {
+                if (data->event == ToolDataWindow::Event::UPDATE) {
+                    apply_settings();
+                }
             }
         }
     }
@@ -797,30 +877,64 @@ ToolResponse ToolRouteTrackInteractive::update(const ToolArgs &args)
     return ToolResponse();
 }
 
+static std::string format_length(double l)
+{
+    l /= 1e6;
+    std::stringstream ss;
+    ss.imbue(get_locale());
+    ss << l;
+    ss << " mm";
+    return ss.str();
+}
+
 void ToolRouteTrackInteractive::update_tip()
 {
     std::stringstream ss;
+    std::vector<ActionLabelInfo> actions;
+    actions.reserve(12);
     if (tool_id == ToolID::DRAG_TRACK_INTERACTIVE) {
-        imp->tool_bar_set_tip("<b>LMB:</b>place <b>RMB:</b>cancel");
+        imp->tool_bar_set_actions({
+                {InToolActionID::LMB},
+                {InToolActionID::RMB},
+        });
         return;
     }
     if (is_tune()) {
         if (meander_placer) {
-            ss << "<b>LMB:</b>place <b>RMB:</b>cancel <b>l:</b>set length <b>&lt;&gt;:</b>+/- amplitude <b>,.:</b>+/- "
-                  "spacing";
-            ss << " <i>";
-            ss << meander_placer->TuningInfo();
-            ss << "</i>";
-            imp->tool_bar_set_tip(ss.str());
+            imp->tool_bar_set_actions({
+                    {InToolActionID::LMB},
+                    {InToolActionID::RMB},
+                    {InToolActionID::LENGTH_TUNING_LENGTH},
+                    {InToolActionID::LENGTH_TUNING_AMPLITUDE_DEC, InToolActionID::LENGTH_TUNING_AMPLITUDE_INC,
+                     "amplitude"},
+                    {InToolActionID::LENGTH_TUNING_SPACING_DEC, InToolActionID::LENGTH_TUNING_SPACING_INC, "spacing"},
+            });
+            imp->tool_bar_set_tip(meander_placer->TuningInfo());
         }
         return;
     }
     if (state == State::ROUTING) {
-        ss << "<b>LMB:</b>place junction/connect <b>RMB:</b>finish and delete "
-              "last segment <b>/:</b>track posture <b>v:</b>toggle via "
-              "<b>w:</b>track width <b>W:</b>default track width "
-              "<b>o:</b>clearance offset <b>O:</b>default cl. offset"
-              "<i> ";
+        actions.emplace_back(InToolActionID::LMB, "connect");
+        actions.emplace_back(InToolActionID::RMB, "finish");
+        actions.emplace_back(InToolActionID::POSTURE);
+        actions.emplace_back(InToolActionID::TOGGLE_VIA);
+        actions.emplace_back(InToolActionID::ROUTER_SETTINGS);
+        const auto &sz = router->Sizes();
+        if (sz.WidthFromRules()) {
+            actions.emplace_back(InToolActionID::ENTER_WIDTH, "track width");
+        }
+        else {
+            actions.emplace_back(InToolActionID::ENTER_WIDTH, InToolActionID::TRACK_WIDTH_DEFAULT,
+                                 "track width / default");
+        }
+        if (iface->get_override_routing_offset() == -1) {
+            actions.emplace_back(InToolActionID::CLEARANCE_OFFSET, "set clearance offset");
+        }
+        else {
+            actions.emplace_back(InToolActionID::CLEARANCE_OFFSET, InToolActionID::CLEARANCE_OFFSET_DEFAULT,
+                                 "set / reset clearance offset");
+        }
+
         auto nets = router->GetCurrentNets();
         Net *net = nullptr;
         for (auto x : nets) {
@@ -828,7 +942,7 @@ void ToolRouteTrackInteractive::update_tip()
         }
         if (net) {
             if (net->name.size()) {
-                ss << "routing net \"" << net->name << "\"";
+                ss << "Net \"" << net->name << "\"";
             }
             else {
                 ss << "routing unnamed net";
@@ -838,17 +952,16 @@ void ToolRouteTrackInteractive::update_tip()
             ss << "routing no net";
         }
 
-        PNS::SIZES_SETTINGS sz(router->Sizes());
-        ss << "  track width " << dim_to_string(sz.TrackWidth());
+        ss << "  width " << format_length(sz.TrackWidth());
         if (sz.WidthFromRules()) {
             ss << " (default)";
         }
-        ss << "</i>";
     }
     else {
-        ss << "<b>LMB:</b>select starting junction/pad <b>RMB:</b>cancel "
-              "<b>s:</b>shove/walkaround ";
-        ss << "<i>";
+        actions.emplace_back(InToolActionID::LMB, "select staring point");
+        actions.emplace_back(InToolActionID::RMB, "cancel");
+        actions.emplace_back(InToolActionID::ROUTER_MODE);
+        actions.emplace_back(InToolActionID::ROUTER_SETTINGS);
         ss << "Mode: ";
         if (shove)
             ss << "shove";
@@ -862,8 +975,8 @@ void ToolRouteTrackInteractive::update_tip()
                 ss << " Current Net: " << net->name;
             ;
         }
-        ss << "</i>";
     }
+    imp->tool_bar_set_actions(actions);
     imp->tool_bar_set_tip(ss.str());
 }
 } // namespace horizon
