@@ -7,7 +7,7 @@
 namespace horizon {
 class EditKeepoutDialog : public Gtk::Dialog {
 public:
-    EditKeepoutDialog(Gtk::Window *parent, class Keepout *k, class Core *c, bool add_mode);
+    EditKeepoutDialog(Gtk::Window *parent, class Keepout *k, class IDocument *c, bool add_mode);
 
 private:
     class Keepout *keepout = nullptr;
