@@ -84,9 +84,18 @@ enum class ActionID {
     RESET_AIRWIRE_FILTER,
     FILTER_AIRWIRES,
     AIRWIRE_FILTER_WINDOW,
-    SELECT_POLYGON
+    SELECT_POLYGON,
+    FAB_OUTPUT_WINDOW,
+    GEN_FAB_OUTPUT,
+    FOOTPRINT_GENERATOR,
+    SET_GRID_ORIGIN,
+    DISTRACTION_FREE,
+    TOGGLE_SNAP_TO_TARGETS,
+    PARTS_WINDOW,
 };
 
+using ActionToolID = std::pair<ActionID, ToolID>;
+
 enum class ActionGroup {
     ALL,
     UNKNOWN,
@@ -105,12 +114,17 @@ enum class ActionGroup {
     VIEW,
     FRAME,
     GROUP_TAG,
-    SEARCH
+    SEARCH,
+    EXPORT_IMPORT,
+    TUNING,
 };
 
 typedef std::vector<std::pair<unsigned int, GdkModifierType>> KeySequence;
 
 std::string key_sequence_to_string(const KeySequence &keys);
+std::string key_sequence_to_string_short(const KeySequence &keys);
+
+std::string key_sequences_to_string(const std::vector<KeySequence> &seqs);
 
 class ActionConnection {
 public:
@@ -125,7 +139,7 @@ public:
     std::function<void(const ActionConnection &)> cb;
 };
 
-std::pair<ActionID, ToolID> make_action(ActionID id);
-std::pair<ActionID, ToolID> make_action(ToolID id);
+ActionToolID make_action(ActionID id);
+ActionToolID make_action(ToolID id);
 
 } // namespace horizon
