@@ -3,7 +3,11 @@
 #include "imp/action.hpp"
 
 namespace horizon {
-const std::map<std::pair<ActionID, ToolID>, ActionCatalogItem> action_catalog = {
+const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
+        {{ActionID::DISTRACTION_FREE, ToolID::NONE},
+         {"Toggle distraction free mode", ActionGroup::UNKNOWN, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
         {{ActionID::SELECTION_FILTER, ToolID::NONE},
          {"Selection filter", ActionGroup::UNKNOWN, ActionCatalogItem::AVAILABLE_EVERYWHERE,
           ActionCatalogItem::FLAGS_DEFAULT}},
@@ -151,15 +155,15 @@ const std::map<std::pair<ActionID, ToolID>, ActionCatalogItem> action_catalog =
           ActionCatalogItem::FLAGS_SPECIFIC}},
 
         {{ActionID::TUNING, ToolID::NONE},
-         {"Length tuning", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+         {"Length tuning", ActionGroup::TUNING, ActionCatalogItem::AVAILABLE_IN_BOARD,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TUNING_ADD_TRACKS, ToolID::NONE},
-         {"Measure track", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+         {"Measure track", ActionGroup::TUNING, ActionCatalogItem::AVAILABLE_IN_BOARD,
           ActionCatalogItem::FLAGS_SPECIFIC}},
 
         {{ActionID::TUNING_ADD_TRACKS_ALL, ToolID::NONE},
-         {"Measure all tracks", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+         {"Measure all tracks", ActionGroup::TUNING, ActionCatalogItem::AVAILABLE_IN_BOARD,
           ActionCatalogItem::FLAGS_SPECIFIC}},
 
         {{ActionID::HIGHLIGHT_NET, ToolID::NONE},
@@ -368,8 +372,20 @@ const std::map<std::pair<ActionID, ToolID>, ActionCatalogItem> action_catalog =
          {"Place hole", ActionGroup::PADSTACK, ActionCatalogItem::AVAILABLE_IN_PADSTACK,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
+        {{ActionID::TOOL, ToolID::PLACE_HOLE_SLOT},
+         {"Place slot hole", ActionGroup::PADSTACK, ActionCatalogItem::AVAILABLE_IN_PADSTACK,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
         {{ActionID::TOOL, ToolID::PLACE_SHAPE},
-         {"Place shape", ActionGroup::PADSTACK, ActionCatalogItem::AVAILABLE_IN_PADSTACK,
+         {"Place circular shape", ActionGroup::PADSTACK, ActionCatalogItem::AVAILABLE_IN_PADSTACK,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::PLACE_SHAPE_RECTANGLE},
+         {"Place rectangular shape", ActionGroup::PADSTACK, ActionCatalogItem::AVAILABLE_IN_PADSTACK,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::PLACE_SHAPE_OBROUND},
+         {"Place obround shape", ActionGroup::PADSTACK, ActionCatalogItem::AVAILABLE_IN_PADSTACK,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::EDIT_SHAPE},
@@ -377,7 +393,7 @@ const std::map<std::pair<ActionID, ToolID>, ActionCatalogItem> action_catalog =
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::IMPORT_DXF},
-         {"Import dxf", ActionGroup::UNKNOWN,
+         {"Import dxf", ActionGroup::EXPORT_IMPORT,
           ActionCatalogItem::AVAILABLE_IN_BOARD | ActionCatalogItem::AVAILABLE_IN_PACKAGE
                   | ActionCatalogItem::AVAILABLE_IN_FRAME,
           ActionCatalogItem::FLAGS_DEFAULT}},
@@ -443,10 +459,6 @@ const std::map<std::pair<ActionID, ToolID>, ActionCatalogItem> action_catalog =
         {{ActionID::TOOL, ToolID::EDIT_VIA},
          {"Edit via", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD, ActionCatalogItem::FLAGS_DEFAULT}},
 
-        {{ActionID::TOOL, ToolID::ROUTE_TRACK},
-         {"Route track (old)", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
-          ActionCatalogItem::FLAGS_DEFAULT}},
-
         {{ActionID::TOOL, ToolID::DRAW_TRACK},
          {"Draw track", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD, ActionCatalogItem::FLAGS_DEFAULT}},
 
@@ -462,17 +474,18 @@ const std::map<std::pair<ActionID, ToolID>, ActionCatalogItem> action_catalog =
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::DRAG_TRACK_INTERACTIVE},
-         {"Drag track", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD, ActionCatalogItem::FLAGS_DEFAULT}},
+         {"Drag track/via", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::TUNE_TRACK},
-         {"Tune track", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD, ActionCatalogItem::FLAGS_DEFAULT}},
+         {"Tune track", ActionGroup::TUNING, ActionCatalogItem::AVAILABLE_IN_BOARD, ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::TUNE_DIFFPAIR},
-         {"Tune diff. pair", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+         {"Tune diff. pair", ActionGroup::TUNING, ActionCatalogItem::AVAILABLE_IN_BOARD,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::TUNE_DIFFPAIR_SKEW},
-         {"Tune diff. pair skew", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+         {"Tune diff. pair skew", ActionGroup::TUNING, ActionCatalogItem::AVAILABLE_IN_BOARD,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::ADD_PLANE},
@@ -592,11 +605,11 @@ const std::map<std::pair<ActionID, ToolID>, ActionCatalogItem> action_catalog =
                   | ActionCatalogItem::FLAGS_NO_PREFERENCES}},
 
         {{ActionID::BOM_EXPORT_WINDOW, ToolID::NONE},
-         {"BOM export window", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+         {"BOM export window", ActionGroup::EXPORT_IMPORT, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::EXPORT_BOM, ToolID::NONE},
-         {"Export BOM", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+         {"Export BOM", ActionGroup::EXPORT_IMPORT, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::SWAP_NETS},
@@ -680,11 +693,11 @@ const std::map<std::pair<ActionID, ToolID>, ActionCatalogItem> action_catalog =
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::PDF_EXPORT_WINDOW, ToolID::NONE},
-         {"PDF export window", ActionGroup::UNKNOWN, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC_AND_BOARD,
+         {"PDF export window", ActionGroup::EXPORT_IMPORT, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC_AND_BOARD,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::EXPORT_PDF, ToolID::NONE},
-         {"Export PDF", ActionGroup::UNKNOWN, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC_AND_BOARD,
+         {"Export PDF", ActionGroup::EXPORT_IMPORT, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC_AND_BOARD,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::DRAW_CONNECTION_LINE},
@@ -701,7 +714,7 @@ const std::map<std::pair<ActionID, ToolID>, ActionCatalogItem> action_catalog =
                   | ActionCatalogItem::FLAGS_NO_PREFERENCES}},
 
         {{ActionID::TOOL, ToolID::IMPORT_KICAD_PACKAGE},
-         {"Import KiCad package", ActionGroup::PACKAGE, ActionCatalogItem::AVAILABLE_IN_PACKAGE,
+         {"Import KiCad package", ActionGroup::EXPORT_IMPORT, ActionCatalogItem::AVAILABLE_IN_PACKAGE,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::SMASH_SILKSCREEN_GRAPHICS},
@@ -732,23 +745,23 @@ const std::map<std::pair<ActionID, ToolID>, ActionCatalogItem> action_catalog =
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::STEP_EXPORT_WINDOW, ToolID::NONE},
-         {"STEP export window", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+         {"STEP export window", ActionGroup::EXPORT_IMPORT, ActionCatalogItem::AVAILABLE_IN_BOARD,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::EXPORT_STEP, ToolID::NONE},
-         {"Export STEP", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD, ActionCatalogItem::FLAGS_DEFAULT}},
+         {"Export STEP", ActionGroup::EXPORT_IMPORT, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::PNP_EXPORT_WINDOW, ToolID::NONE},
-         {"Pick & place export window", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+         {"Pick & place export window", ActionGroup::EXPORT_IMPORT, ActionCatalogItem::AVAILABLE_IN_BOARD,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::EXPORT_PNP, ToolID::NONE},
-         {"Export Pick & place", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+         {"Export Pick & place", ActionGroup::EXPORT_IMPORT, ActionCatalogItem::AVAILABLE_IN_BOARD,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::AIRWIRE_FILTER_WINDOW, ToolID::NONE},
-         {"Airwire filter", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
-          ActionCatalogItem::FLAGS_DEFAULT}},
+         {"Nets window", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD, ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::RESET_AIRWIRE_FILTER, ToolID::NONE},
          {"Reset airwire filter", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
@@ -793,18 +806,60 @@ const std::map<std::pair<ActionID, ToolID>, ActionCatalogItem> action_catalog =
         {{ActionID::TOOL, ToolID::ROUND_OFF_VERTEX},
          {"Round off vertex", ActionGroup::GRAPHICS, ActionCatalogItem::AVAILABLE_EVERYWHERE,
           ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::FAB_OUTPUT_WINDOW, ToolID::NONE},
+         {"Fab. output window", ActionGroup::EXPORT_IMPORT, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::GEN_FAB_OUTPUT, ToolID::NONE},
+         {"Generate fab. output", ActionGroup::EXPORT_IMPORT, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::SWAP_GATES},
+         {"Swap gates", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::FOOTPRINT_GENERATOR, ToolID::NONE},
+         {"Footprint generator", ActionGroup::PACKAGE, ActionCatalogItem::AVAILABLE_IN_PACKAGE,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::PLACE_PICTURE},
+         {"Place picture", ActionGroup::GRAPHICS,
+          ActionCatalogItem::AVAILABLE_IN_SCHEMATIC_AND_BOARD | ActionCatalogItem::AVAILABLE_IN_PACKAGE,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::SET_GRID_ORIGIN, ToolID::NONE},
+         {"Set grid origin", ActionGroup::UNKNOWN, ActionCatalogItem::AVAILABLE_IN_PACKAGE_AND_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOGGLE_SNAP_TO_TARGETS, ToolID::NONE},
+         {"Toggle snap to targets", ActionGroup::VIEW, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_IN_TOOL}},
+
+        {{ActionID::PARTS_WINDOW, ToolID::NONE},
+         {"Parts window", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD, ActionCatalogItem::FLAGS_DEFAULT}},
 };
 
 const std::vector<std::pair<ActionGroup, std::string>> action_group_catalog = {
-        {ActionGroup::CLIPBOARD, "Clipboard"}, {ActionGroup::GRAPHICS, "Graphics"},
-        {ActionGroup::MOVE, "Move"},           {ActionGroup::BOARD, "Board"},
-        {ActionGroup::SCHEMATIC, "Schematic"}, {ActionGroup::GROUP_TAG, "Group & Tag"},
-        {ActionGroup::SYMBOL, "Symbol"},       {ActionGroup::PADSTACK, "Padstack"},
-        {ActionGroup::PACKAGE, "Package"},     {ActionGroup::FRAME, "Frame"},
-        {ActionGroup::UNDO, "Undo"},           {ActionGroup::LAYER, "Layer"},
-        {ActionGroup::SELECTION, "Selection"}, {ActionGroup::RULES, "Rules"},
-        {ActionGroup::UNKNOWN, "Misc"},        {ActionGroup::VIEW, "View"},
+        {ActionGroup::CLIPBOARD, "Clipboard"},
+        {ActionGroup::GRAPHICS, "Graphics"},
+        {ActionGroup::MOVE, "Move"},
+        {ActionGroup::BOARD, "Board"},
+        {ActionGroup::SCHEMATIC, "Schematic"},
+        {ActionGroup::GROUP_TAG, "Group & Tag"},
+        {ActionGroup::SYMBOL, "Symbol"},
+        {ActionGroup::PADSTACK, "Padstack"},
+        {ActionGroup::PACKAGE, "Package"},
+        {ActionGroup::FRAME, "Frame"},
+        {ActionGroup::UNDO, "Undo"},
+        {ActionGroup::LAYER, "Layer"},
+        {ActionGroup::SELECTION, "Selection"},
+        {ActionGroup::RULES, "Rules"},
+        {ActionGroup::UNKNOWN, "Misc"},
+        {ActionGroup::VIEW, "View"},
+        {ActionGroup::EXPORT_IMPORT, "Export / import"},
         {ActionGroup::SEARCH, "Search"},
+        {ActionGroup::TUNING, "Tuning"},
 
 };
 
@@ -816,6 +871,7 @@ const std::vector<std::pair<ActionGroup, std::string>> action_group_catalog = {
 const LutEnumStr<ActionID> action_lut = {
         ACTION_LUT_ITEM(TOOL),
         ACTION_LUT_ITEM(SELECTION_FILTER),
+        ACTION_LUT_ITEM(DISTRACTION_FREE),
         ACTION_LUT_ITEM(SAVE),
         ACTION_LUT_ITEM(VIEW_3D),
         ACTION_LUT_ITEM(UNDO),
@@ -889,6 +945,12 @@ const LutEnumStr<ActionID> action_lut = {
         ACTION_LUT_ITEM(FILTER_AIRWIRES),
         ACTION_LUT_ITEM(AIRWIRE_FILTER_WINDOW),
         ACTION_LUT_ITEM(SELECT_POLYGON),
+        ACTION_LUT_ITEM(FAB_OUTPUT_WINDOW),
+        ACTION_LUT_ITEM(GEN_FAB_OUTPUT),
+        ACTION_LUT_ITEM(FOOTPRINT_GENERATOR),
+        ACTION_LUT_ITEM(SET_GRID_ORIGIN),
+        ACTION_LUT_ITEM(TOGGLE_SNAP_TO_TARGETS),
+        ACTION_LUT_ITEM(PARTS_WINDOW),
 };
 
 #define TOOL_LUT_ITEM(x)                                                                                               \
@@ -928,19 +990,21 @@ const LutEnumStr<ToolID> tool_lut = {
         TOOL_LUT_ITEM(ENTER_DATUM),
         TOOL_LUT_ITEM(MOVE_EXACTLY),
         TOOL_LUT_ITEM(PLACE_HOLE),
+        TOOL_LUT_ITEM(PLACE_HOLE_SLOT),
         TOOL_LUT_ITEM(PLACE_PAD),
         TOOL_LUT_ITEM(PASTE),
         TOOL_LUT_ITEM(ASSIGN_PART),
         TOOL_LUT_ITEM(MAP_PACKAGE),
         TOOL_LUT_ITEM(DRAW_TRACK),
         TOOL_LUT_ITEM(PLACE_VIA),
-        TOOL_LUT_ITEM(ROUTE_TRACK),
         TOOL_LUT_ITEM(DRAG_KEEP_SLOPE),
         TOOL_LUT_ITEM(ADD_PART),
         TOOL_LUT_ITEM(ANNOTATE),
         TOOL_LUT_ITEM(SMASH),
         TOOL_LUT_ITEM(UNSMASH),
         TOOL_LUT_ITEM(PLACE_SHAPE),
+        TOOL_LUT_ITEM(PLACE_SHAPE_RECTANGLE),
+        TOOL_LUT_ITEM(PLACE_SHAPE_OBROUND),
         TOOL_LUT_ITEM(EDIT_SHAPE),
         TOOL_LUT_ITEM(IMPORT_DXF),
         TOOL_LUT_ITEM(MANAGE_NET_CLASSES),
@@ -1031,5 +1095,7 @@ const LutEnumStr<ToolID> tool_lut = {
         TOOL_LUT_ITEM(SMASH_PACKAGE_OUTLINE),
         TOOL_LUT_ITEM(RESIZE_SYMBOL),
         TOOL_LUT_ITEM(ROUND_OFF_VERTEX),
+        TOOL_LUT_ITEM(SWAP_GATES),
+        TOOL_LUT_ITEM(PLACE_PICTURE),
 };
 } // namespace horizon
