@@ -1,9 +1,11 @@
 #include "footprint_generator_dual.hpp"
 #include "util/util.hpp"
 #include "widgets/pool_browser_button.hpp"
+#include "document/idocument_package.hpp"
+#include "pool/pool.hpp"
 
 namespace horizon {
-FootprintGeneratorDual::FootprintGeneratorDual(CorePackage *c)
+FootprintGeneratorDual::FootprintGeneratorDual(IDocumentPackage *c)
     : Glib::ObjectBase(typeid(FootprintGeneratorDual)),
       FootprintGeneratorBase("/org/horizon-eda/horizon/imp/footprint_generator/dual.svg", c)
 {
@@ -133,7 +135,7 @@ bool FootprintGeneratorDual::generate()
     int64_t y0 = (pad_count / 2 - 1) * (pitch / 2);
     int64_t pad_width = sp_pad_width->get_value_as_int();
     int64_t pad_height = sp_pad_height->get_value_as_int();
-    auto padstack = core->m_pool->get_padstack(browser_button->property_selected_uuid());
+    auto padstack = core->get_pool()->get_padstack(browser_button->property_selected_uuid());
     for (auto it : {-1, 1}) {
         for (unsigned int i = 0; i < pad_count / 2; i++) {
             auto uu = UUID::random();
@@ -169,8 +171,6 @@ bool FootprintGeneratorDual::generate()
             }
         }
     }
-
-    core->commit();
     return true;
 }
 
