@@ -2,9 +2,9 @@
 #include "footag/display.hpp"
 
 namespace horizon {
-FootprintGeneratorFootag::FootprintGeneratorFootag(CorePackage *c)
-    : Glib::ObjectBase(typeid(FootprintGeneratorFootag)), core(c), sidebar(),
-      separator(Gtk::Orientation::ORIENTATION_VERTICAL), stack()
+FootprintGeneratorFootag::FootprintGeneratorFootag(IDocumentPackage *c)
+    : Glib::ObjectBase(typeid(FootprintGeneratorFootag)), sidebar(), separator(Gtk::Orientation::ORIENTATION_VERTICAL),
+      stack()
 {
     add(sidebar);
     add(separator);
