@@ -1,14 +1,12 @@
 #pragma once
-#include "core/core_package.hpp"
 #include <gtkmm.h>
 namespace horizon {
 class FootprintGeneratorFootag : public Gtk::Box {
 public:
-    FootprintGeneratorFootag(CorePackage *c);
+    FootprintGeneratorFootag(class IDocumentPackage *c);
     bool generate();
 
 protected:
-    CorePackage *core;
     Gtk::Box *box_top = nullptr;
     Gtk::StackSidebar sidebar;
     Gtk::Separator separator;
