@@ -1,8 +1,10 @@
 #include "footprint_generator_single.hpp"
 #include "widgets/pool_browser_button.hpp"
+#include "document/idocument_package.hpp"
+#include "pool/pool.hpp"
 
 namespace horizon {
-FootprintGeneratorSingle::FootprintGeneratorSingle(CorePackage *c)
+FootprintGeneratorSingle::FootprintGeneratorSingle(IDocumentPackage *c)
     : Glib::ObjectBase(typeid(FootprintGeneratorSingle)), FootprintGeneratorBase(
                                                                   "/org/horizon-eda/horizon/imp/footprint_generator/"
                                                                   "single.svg",
@@ -66,7 +68,7 @@ bool FootprintGeneratorSingle::generate()
     int64_t pad_width = sp_pad_width->get_value_as_int();
     int64_t pad_height = sp_pad_height->get_value_as_int();
     int64_t y0 = (pad_count - 1) * (pitch / 2);
-    auto padstack = core->m_pool->get_padstack(browser_button->property_selected_uuid());
+    auto padstack = core->get_pool()->get_padstack(browser_button->property_selected_uuid());
     for (unsigned int i = 0; i < pad_count; i++) {
         auto uu = UUID::random();
         auto &pad = pkg->pads.emplace(uu, Pad(uu, padstack)).first->second;
@@ -81,8 +83,6 @@ bool FootprintGeneratorSingle::generate()
             pad.parameter_set[ParameterID::PAD_WIDTH] = pad_width;
         }
     }
-
-    core->commit();
     return true;
 }
 
