@@ -27,6 +27,9 @@
 #include "nlohmann/json.hpp"
 #include "core/tool_id.hpp"
 #include "imp/action.hpp"
+#include "preferences/preferences_util.hpp"
+#include "widgets/action_button.hpp"
+#include "in_tool_action_catalog.hpp"
 
 namespace horizon {
 
@@ -204,6 +207,7 @@ bool ImpBase::handle_close(GdkEventAny *ev)
     return false;
 }
 
+#undef GET_WIDGET
 #define GET_WIDGET(name)                                                                                               \
     do {                                                                                                               \
         main_window->builder->get_widget(#name, name);                                                                 \
@@ -294,6 +298,12 @@ void ImpBase::run(int argc, char *argv[])
             [this](ObjectType ty, UUID uu) { return core->get_display_name(ty, uu); });
 
     {
+        selection_qualifiers = {
+                {CanvasGL::SelectionTool::BOX, CanvasGL::SelectionQualifier::INCLUDE_ORIGIN},
+                {CanvasGL::SelectionTool::LASSO, CanvasGL::SelectionQualifier::INCLUDE_ORIGIN},
+                {CanvasGL::SelectionTool::PAINT, CanvasGL::SelectionQualifier::TOUCH_BOX},
+        };
+
         Gtk::RadioButton *selection_tool_box_button, *selection_tool_lasso_button, *selection_tool_paint_button;
         Gtk::RadioButton *selection_qualifier_include_origin_button, *selection_qualifier_touch_box_button,
                 *selection_qualifier_include_box_button, *selection_qualifier_auto_button;
@@ -313,34 +323,25 @@ void ImpBase::run(int argc, char *argv[])
                 {CanvasGL::SelectionQualifier::TOUCH_BOX, selection_qualifier_touch_box_button},
                 {CanvasGL::SelectionQualifier::INCLUDE_ORIGIN, selection_qualifier_include_origin_button},
                 {CanvasGL::SelectionQualifier::AUTO, selection_qualifier_auto_button}};
-        bind_widget<CanvasGL::SelectionQualifier>(qual_map, canvas->selection_qualifier,
-                                                  [this](auto v) { this->update_selection_label(); });
+        bind_widget<CanvasGL::SelectionQualifier>(qual_map, canvas->selection_qualifier, [this](auto v) {
+            selection_qualifiers.at(canvas->selection_tool) = v;
+            this->update_selection_label();
+        });
 
         std::map<CanvasGL::SelectionTool, Gtk::RadioButton *> tool_map = {
                 {CanvasGL::SelectionTool::BOX, selection_tool_box_button},
                 {CanvasGL::SelectionTool::LASSO, selection_tool_lasso_button},
                 {CanvasGL::SelectionTool::PAINT, selection_tool_paint_button},
         };
-        bind_widget<CanvasGL::SelectionTool>(tool_map, canvas->selection_tool,
-                                             [this, selection_qualifier_touch_box_button, selection_qualifier_box,
-                                              selection_qualifier_auto_button,
-                                              selection_qualifier_include_origin_button](auto v) {
-                                                 this->update_selection_label();
-                                                 auto is_paint = (v == CanvasGL::SelectionTool::PAINT);
-                                                 if (is_paint) {
-                                                     selection_qualifier_touch_box_button->set_active(true);
-                                                 }
-                                                 selection_qualifier_box->set_sensitive(!is_paint);
+        bind_widget<CanvasGL::SelectionTool>(
+                tool_map, canvas->selection_tool, [this, selection_qualifier_box, qual_map](auto v) {
+                    this->update_selection_label();
+                    selection_qualifier_box->set_sensitive(v != CanvasGL::SelectionTool::PAINT);
+                    qual_map.at(CanvasGL::SelectionQualifier::AUTO)->set_sensitive(v != CanvasGL::SelectionTool::LASSO);
 
-                                                 auto is_box = (v == CanvasGL::SelectionTool::BOX);
-                                                 selection_qualifier_auto_button->set_sensitive(is_box);
-                                                 if (is_box)
-                                                     selection_qualifier_auto_button->set_active(true);
-
-                                                 if (v == CanvasGL::SelectionTool::LASSO) {
-                                                     selection_qualifier_include_origin_button->set_active(true);
-                                                 }
-                                             });
+                    auto qual = selection_qualifiers.at(v);
+                    qual_map.at(qual)->set_active(true);
+                });
 
         connect_action(ActionID::SELECTION_TOOL_BOX,
                        [selection_tool_box_button](const auto &a) { selection_tool_box_button->set_active(true); });
@@ -408,6 +409,35 @@ void ImpBase::run(int argc, char *argv[])
 
     key_sequence_dialog = std::make_unique<KeySequenceDialog>(this->main_window);
 
+    distraction_free_action = main_window->add_action_bool("distraction_free", distraction_free);
+    distraction_free_action->signal_change_state().connect([this](const Glib::VariantBase &v) {
+        auto b = Glib::VariantBase::cast_dynamic<Glib::Variant<bool>>(v).get();
+        if (b != distraction_free) {
+            trigger_action(ActionID::DISTRACTION_FREE);
+        }
+    });
+
+    connect_action(ActionID::DISTRACTION_FREE, [this](const auto &a) {
+        distraction_free = !distraction_free;
+        g_simple_action_set_state(distraction_free_action->gobj(), g_variant_new_boolean(distraction_free));
+        if (distraction_free) {
+            left_panel_width = main_window->left_panel->get_allocated_width()
+                               + main_window->left_panel->get_margin_start()
+                               + main_window->left_panel->get_margin_end();
+        }
+
+        auto [scale, offset] = canvas->get_scale_and_offset();
+        if (distraction_free)
+            offset.x += left_panel_width;
+        else
+            offset.x -= left_panel_width;
+        canvas->set_scale_and_offset(scale, offset);
+
+        main_window->left_panel->set_visible(!distraction_free);
+        bool show_properties = panels->get_selection().size() > 0;
+        main_window->property_scrolled_window->set_visible(show_properties && !distraction_free);
+        this->update_view_hints();
+    });
     connect_action(ActionID::SELECTION_FILTER, [this](const auto &a) { selection_filter_dialog->present(); });
     connect_action(ActionID::SAVE, [this](const auto &a) {
         if (!read_only) {
@@ -453,7 +483,7 @@ void ImpBase::run(int argc, char *argv[])
         tool_popover->set_pointing_to(rect);
 
         this->update_action_sensitivity();
-        std::map<std::pair<ActionID, ToolID>, bool> can_begin;
+        std::map<ActionToolID, bool> can_begin;
         auto sel = canvas->get_selection();
         for (const auto &it : action_catalog) {
             if (it.first.first == ActionID::TOOL) {
@@ -490,18 +520,32 @@ void ImpBase::run(int argc, char *argv[])
         canvas->set_flip_view(false);
         this->canvas_update_from_pp();
         this->update_view_hints();
+        g_simple_action_set_state(bottom_view_action->gobj(), g_variant_new_boolean(canvas->get_flip_view()));
     });
 
     connect_action(ActionID::VIEW_BOTTOM, [this](const auto &a) {
         canvas->set_flip_view(true);
         this->canvas_update_from_pp();
         this->update_view_hints();
+        g_simple_action_set_state(bottom_view_action->gobj(), g_variant_new_boolean(canvas->get_flip_view()));
     });
 
     connect_action(ActionID::FLIP_VIEW, [this](const auto &a) {
         canvas->set_flip_view(!canvas->get_flip_view());
         this->canvas_update_from_pp();
         this->update_view_hints();
+        g_simple_action_set_state(bottom_view_action->gobj(), g_variant_new_boolean(canvas->get_flip_view()));
+    });
+
+    bottom_view_action = main_window->add_action_bool("bottom_view", false);
+    bottom_view_action->signal_change_state().connect([this](const Glib::VariantBase &v) {
+        auto b = Glib::VariantBase::cast_dynamic<Glib::Variant<bool>>(v).get();
+        if (b) {
+            trigger_action(ActionID::VIEW_BOTTOM);
+        }
+        else {
+            trigger_action(ActionID::VIEW_TOP);
+        }
     });
 
     connect_action(ActionID::SEARCH, [this](const auto &a) { this->set_search_mode(true); });
@@ -526,6 +570,10 @@ void ImpBase::run(int argc, char *argv[])
         this->handle_search();
         this->search_go(0);
     });
+    main_window->search_exact_cb->signal_toggled().connect([this] {
+        this->handle_search();
+        this->search_go(0);
+    });
     main_window->search_entry->signal_key_press_event().connect(
             [this](GdkEventKey *ev) {
                 if (ev->keyval == GDK_KEY_Escape) {
@@ -575,14 +623,11 @@ void ImpBase::run(int argc, char *argv[])
         search_go(0);
     }
 
-
-    grid_spin_button = Gtk::manage(new SpinButtonDim());
-    grid_spin_button->set_range(0.01_mm, 10_mm);
-    grid_spacing_binding = Glib::Binding::bind_property(grid_spin_button->property_value(),
-                                                        canvas->property_grid_spacing(), Glib::BINDING_BIDIRECTIONAL);
-    grid_spin_button->set_value(1.25_mm);
-    grid_spin_button->show_all();
-    main_window->grid_box->pack_start(*grid_spin_button, true, true, 0);
+    grid_controller.emplace(*main_window, *canvas);
+    connect_action(ActionID::SET_GRID_ORIGIN, [this](const auto &c) {
+        auto co = canvas->get_cursor_pos();
+        grid_controller->set_origin(co);
+    });
 
     auto save_button = create_action_button(make_action(ActionID::SAVE));
     save_button->show();
@@ -592,6 +637,7 @@ void ImpBase::run(int argc, char *argv[])
         json j;
         j["op"] = "needs-save";
         j["pid"] = getpid();
+        j["filename"] = core->get_filename();
         j["needs_save"] = core->get_needs_save();
         send_json(j);
     });
@@ -689,6 +735,20 @@ void ImpBase::run(int argc, char *argv[])
 
     add_hamburger_menu();
 
+    view_options_menu = Gio::Menu::create();
+    main_window->view_options_button->set_menu_model(view_options_menu);
+    {
+        Gdk::Rectangle rect;
+        rect.set_width(24);
+        main_window->view_options_button->get_popover()->set_pointing_to(rect);
+    }
+
+    view_options_menu->append("Distraction free mode", "win.distraction_free");
+    add_tool_action(ActionID::SELECTION_FILTER, "selection_filter");
+    view_options_menu->append("Selection filter", "win.selection_filter");
+
+    imp_interface = std::make_unique<ImpInterface>(this);
+
     construct();
 
     {
@@ -738,8 +798,6 @@ void ImpBase::run(int argc, char *argv[])
 
     context_menu = Gtk::manage(new Gtk::Menu());
 
-    imp_interface = std::make_unique<ImpInterface>(this);
-
     core->signal_tool_changed().connect([this](ToolID id) { s_signal_action_sensitive.emit(); });
 
     canvas->signal_hover_selection_changed().connect(sigc::mem_fun(*this, &ImpBase::hud_update));
@@ -1014,7 +1072,7 @@ void ImpBase::edit_pool_item(ObjectType type, const UUID &uu)
     send_json(j);
 }
 
-Gtk::Button *ImpBase::create_action_button(std::pair<ActionID, ToolID> action)
+Gtk::Button *ImpBase::create_action_button(ActionToolID action)
 {
     auto &catitem = action_catalog.at(action);
     auto button = Gtk::manage(new Gtk::Button(catitem.name));
@@ -1024,7 +1082,7 @@ Gtk::Button *ImpBase::create_action_button(std::pair<ActionID, ToolID> action)
     return button;
 }
 
-bool ImpBase::trigger_action(const std::pair<ActionID, ToolID> &action)
+bool ImpBase::trigger_action(const ActionToolID &action)
 {
     if (core->tool_is_active() && !(action_catalog.at(action).flags & ActionCatalogItem::FLAGS_IN_TOOL)) {
         return false;
@@ -1050,6 +1108,15 @@ void ImpBase::handle_tool_action(const ActionConnection &conn)
     tool_begin(conn.tool_id);
 }
 
+
+ToolID ImpBase::get_tool_for_drag_move(bool ctrl, const std::set<SelectableRef> &sel) const
+{
+    if (ctrl)
+        return ToolID::DUPLICATE;
+    else
+        return ToolID::MOVE;
+}
+
 void ImpBase::handle_drag(bool ctrl)
 {
     auto pos = canvas->get_cursor_pos_win();
@@ -1061,11 +1128,7 @@ void ImpBase::handle_drag(bool ctrl)
             ToolArgs args;
             args.coords = cursor_pos_grid_drag_begin;
             args.selection = selection_for_drag_move;
-            ToolID tool_id;
-            if (ctrl)
-                tool_id = ToolID::DUPLICATE;
-            else
-                tool_id = ToolID::MOVE;
+            ToolID tool_id = get_tool_for_drag_move(ctrl, selection_for_drag_move);
 
             ToolResponse r = core->tool_begin(tool_id, args, imp_interface.get(), true);
             tool_process(r);
@@ -1104,12 +1167,25 @@ void ImpBase::apply_preferences()
             }
         }
     }
+    in_tool_key_sequeces_preferences = preferences.in_tool_key_sequences;
+    in_tool_key_sequeces_preferences.keys.erase(InToolActionID::LMB);
+    in_tool_key_sequeces_preferences.keys.erase(InToolActionID::RMB);
+    in_tool_key_sequeces_preferences.keys.erase(InToolActionID::LMB_RELEASE);
     {
         const auto mod0 = static_cast<GdkModifierType>(0);
         action_connections.at(make_action(ToolID::MOVE_KEY_LEFT)).key_sequences = {{{GDK_KEY_Left, mod0}}};
         action_connections.at(make_action(ToolID::MOVE_KEY_RIGHT)).key_sequences = {{{GDK_KEY_Right, mod0}}};
         action_connections.at(make_action(ToolID::MOVE_KEY_UP)).key_sequences = {{{GDK_KEY_Up, mod0}}};
         action_connections.at(make_action(ToolID::MOVE_KEY_DOWN)).key_sequences = {{{GDK_KEY_Down, mod0}}};
+
+        in_tool_key_sequeces_preferences.keys[InToolActionID::CANCEL] = {{{GDK_KEY_Escape, mod0}}};
+        in_tool_key_sequeces_preferences.keys[InToolActionID::COMMIT] = {{{GDK_KEY_Return, mod0}},
+                                                                         {{GDK_KEY_KP_Enter, mod0}}};
+
+        in_tool_key_sequeces_preferences.keys[InToolActionID::MOVE_LEFT] = {{{GDK_KEY_Left, mod0}}};
+        in_tool_key_sequeces_preferences.keys[InToolActionID::MOVE_RIGHT] = {{{GDK_KEY_Right, mod0}}};
+        in_tool_key_sequeces_preferences.keys[InToolActionID::MOVE_UP] = {{{GDK_KEY_Up, mod0}}};
+        in_tool_key_sequeces_preferences.keys[InToolActionID::MOVE_DOWN] = {{{GDK_KEY_Down, mod0}}};
     }
     {
         switch (canvas_prefs->appearance.grid_fine_modifier) {
@@ -1128,6 +1204,11 @@ void ImpBase::apply_preferences()
                 {{GDK_KEY_Up, grid_fine_modifier}}};
         action_connections.at(make_action(ToolID::MOVE_KEY_FINE_DOWN)).key_sequences = {
                 {{GDK_KEY_Down, grid_fine_modifier}}};
+        in_tool_key_sequeces_preferences.keys[InToolActionID::MOVE_LEFT_FINE] = {{{GDK_KEY_Left, grid_fine_modifier}}};
+        in_tool_key_sequeces_preferences.keys[InToolActionID::MOVE_RIGHT_FINE] = {
+                {{GDK_KEY_Right, grid_fine_modifier}}};
+        in_tool_key_sequeces_preferences.keys[InToolActionID::MOVE_UP_FINE] = {{{GDK_KEY_Up, grid_fine_modifier}}};
+        in_tool_key_sequeces_preferences.keys[InToolActionID::MOVE_DOWN_FINE] = {{{GDK_KEY_Down, grid_fine_modifier}}};
     }
 
     key_sequence_dialog->clear();
@@ -1137,8 +1218,12 @@ void ImpBase::apply_preferences()
             tool_popover->set_key_sequences(it.first, it.second.key_sequences);
         }
     }
-    canvas->smooth_zoom = preferences.zoom.smooth_zoom_2d;
-    canvas->touchpad_pan = preferences.zoom.touchpad_pan;
+    preferences_apply_to_canvas(canvas, preferences);
+    for (auto it : action_buttons) {
+        it->update_key_sequences();
+        it->set_keep_primary_action(!preferences.action_bar.remember);
+    }
+    main_window->set_use_action_bar(preferences.action_bar.enable);
 }
 
 void ImpBase::canvas_update_from_pp()
@@ -1225,13 +1310,18 @@ void ImpBase::add_tool_action(ToolID tid, const std::string &action)
     auto tool_action = main_window->add_action(action, [this, tid] { tool_begin(tid); });
 }
 
-void ImpBase::set_action_sensitive(std::pair<ActionID, ToolID> action, bool v)
+void ImpBase::add_tool_action(ActionID aid, const std::string &action)
+{
+    auto tool_action = main_window->add_action(action, [this, aid] { trigger_action(aid); });
+}
+
+void ImpBase::set_action_sensitive(ActionToolID action, bool v)
 {
     action_sensitivity[action] = v;
     s_signal_action_sensitive.emit();
 }
 
-bool ImpBase::get_action_sensitive(std::pair<ActionID, ToolID> action) const
+bool ImpBase::get_action_sensitive(ActionToolID action) const
 {
     if (core->tool_is_active())
         return action_catalog.at(action).flags & ActionCatalogItem::FLAGS_IN_TOOL;
@@ -1305,34 +1395,16 @@ void ImpBase::goto_layer(int layer)
 
 bool ImpBase::handle_key_press(GdkEventKey *key_event)
 {
-    if (core->tool_is_active()) {
-        if (handle_action_key(key_event))
-            return true;
-
-        ToolArgs args;
-        args.coords = canvas->get_cursor_pos();
-        args.work_layer = canvas->property_work_layer();
-        if (!core->tool_handles_esc() && key_event->keyval == GDK_KEY_Escape) {
-            args.type = ToolEventType::CLICK;
-            args.button = 3;
-        }
-        else {
-            args.type = ToolEventType::KEY;
-            args.key = key_event->keyval;
-            if (key_event->state & grid_fine_modifier)
-                args.mod = ToolArgs::MOD_FINE;
-        }
-
-        ToolResponse r = core->tool_update(args);
-        tool_process(r);
-        return true;
-    }
-    else {
-        return handle_action_key(key_event);
-    }
+    return handle_action_key(key_event);
     return false;
 }
 
+bool ImpBase::keys_match(const KeySequence &keys) const
+{
+    auto minl = std::min(keys_current.size(), keys.size());
+    return minl && std::equal(keys_current.begin(), keys_current.begin() + minl, keys.begin());
+}
+
 bool ImpBase::handle_action_key(GdkEventKey *ev)
 {
     if (ev->is_modifier)
@@ -1343,6 +1415,17 @@ bool ImpBase::handle_action_key(GdkEventKey *ev)
             canvas->set_selection({});
             set_search_mode(false);
         }
+        else {
+            ToolArgs args;
+            args.coords = canvas->get_cursor_pos();
+            args.work_layer = canvas->property_work_layer();
+            args.type = ToolEventType::ACTION;
+            args.action = InToolActionID::CANCEL;
+            ToolResponse r = core->tool_update(args);
+            tool_process(r);
+            return true;
+        }
+
         if (keys_current.size() == 0) {
             return false;
         }
@@ -1365,6 +1448,8 @@ bool ImpBase::handle_action_key(GdkEventKey *ev)
                                                     & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK));
             keys_current.emplace_back(keyval, mod);
         }
+        auto in_tool_actions = core->get_tool_actions();
+        std::set<InToolActionID> in_tool_actions_matched;
         std::set<ActionConnection *> connections_matched;
         auto selection = canvas->get_selection();
         update_action_sensitivity();
@@ -1380,22 +1465,29 @@ bool ImpBase::handle_action_key(GdkEventKey *ev)
                 }
                 if (can_begin) {
                     for (const auto &it2 : it.second.key_sequences) {
-                        auto minl = std::min(keys_current.size(), it2.size());
-                        if (minl && std::equal(keys_current.begin(), keys_current.begin() + minl, it2.begin())) {
+                        if (keys_match(it2)) {
                             connections_matched.insert(&it.second);
                         }
                     }
                 }
             }
         }
-        if (connections_matched.size() == 0) {
-            main_window->tool_hint_label->set_text("Unknown key sequence");
-            keys_current.clear();
-            return false;
+        if (in_tool_actions.size()) {
+            for (const auto &[action, seqs] : in_tool_key_sequeces_preferences.keys) {
+                if (in_tool_actions.count(action)) {
+                    for (const auto &seq : seqs) {
+                        if (keys_match(seq))
+                            in_tool_actions_matched.insert(action);
+                    }
+                }
+            }
         }
-        else if (connections_matched.size() > 1) { // still ambigous
-            main_window->tool_hint_label->set_text(key_sequence_to_string(keys_current) + "?");
-            return true;
+
+        std::cout << "match " << connections_matched.size() << " " << in_tool_actions_matched.size() << std::endl;
+
+        if (connections_matched.size() == 1 && in_tool_actions_matched.size() == 1) {
+            main_window->tool_hint_label->set_text("Ambiguous");
+            keys_current.clear();
         }
         else if (connections_matched.size() == 1) {
             main_window->tool_hint_label->set_text(key_sequence_to_string(keys_current));
@@ -1407,8 +1499,34 @@ bool ImpBase::handle_action_key(GdkEventKey *ev)
             }
             return true;
         }
+        else if (in_tool_actions_matched.size() == 1) {
+            main_window->tool_hint_label->set_text(key_sequence_to_string(keys_current));
+            keys_current.clear();
+
+            ToolArgs args;
+            args.coords = canvas->get_cursor_pos();
+            args.work_layer = canvas->property_work_layer();
+            args.type = ToolEventType::ACTION;
+            args.action = *in_tool_actions_matched.begin();
+            ToolResponse r = core->tool_update(args);
+            tool_process(r);
+
+            return true;
+        }
+        else if (connections_matched.size() > 1 || in_tool_actions_matched.size() > 1) { // still ambigous
+            main_window->tool_hint_label->set_text(key_sequence_to_string(keys_current) + "?");
+            return true;
+        }
+        else if (connections_matched.size() == 0 || in_tool_actions_matched.size() == 0) {
+            main_window->tool_hint_label->set_text("Unknown key sequence");
+            keys_current.clear();
+            return false;
+        }
         else {
-            assert(false); // don't go here
+            Logger::log_warning("Key sequence??", Logger::Domain::IMP,
+                                std::to_string(connections_matched.size()) + " "
+                                        + std::to_string(in_tool_actions_matched.size()));
+            return false;
         }
     }
     return false;
@@ -1446,9 +1564,9 @@ bool ImpBase::handle_click_release(GdkEventButton *button_event)
 {
     if (core->tool_is_active() && button_event->button != 2 && !(button_event->state & Gdk::SHIFT_MASK)) {
         ToolArgs args;
-        args.type = ToolEventType::CLICK_RELEASE;
+        args.type = ToolEventType::ACTION;
+        args.action = InToolActionID::LMB_RELEASE;
         args.coords = canvas->get_cursor_pos();
-        args.button = button_event->button;
         args.target = canvas->get_current_target();
         args.work_layer = canvas->property_work_layer();
         ToolResponse r = core->tool_update(args);
@@ -1457,6 +1575,32 @@ bool ImpBase::handle_click_release(GdkEventButton *button_event)
     return false;
 }
 
+Gtk::MenuItem *ImpBase::create_context_menu_item(ActionToolID act)
+{
+    auto it = Gtk::manage(new Gtk::MenuItem);
+    auto box = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_HORIZONTAL, 8));
+
+    {
+        auto la = Gtk::manage(new Gtk::Label);
+        la->set_xalign(0);
+        la->set_label(action_catalog.at(act).name);
+        box->pack_start(*la, false, false, 0);
+    }
+
+    {
+        auto la = Gtk::manage(new Gtk::Label);
+        la->set_xalign(1);
+        la->get_style_context()->add_class("dim-label");
+        la->set_label(key_sequences_to_string(action_connections.at(act).key_sequences));
+        la->set_margin_end(4);
+        box->pack_start(*la, true, true, 0);
+    }
+
+    box->show_all();
+    it->add(*box);
+    return it;
+}
+
 void ImpBase::create_context_menu(Gtk::Menu *parent, const std::set<SelectableRef> &sel)
 {
     update_action_sensitivity();
@@ -1469,7 +1613,7 @@ void ImpBase::create_context_menu(Gtk::Menu *parent, const std::set<SelectableRe
                 if (it.first.first == ActionID::TOOL) {
                     auto r = core->tool_can_begin(it.first.second, {sel});
                     if (r.first && r.second) {
-                        auto la_sub = Gtk::manage(new Gtk::MenuItem(it.second.name));
+                        auto la_sub = create_context_menu_item(it.first);
                         ToolID tool_id = it.first.second;
                         std::set<SelectableRef> sr(sel);
                         la_sub->signal_activate().connect([this, tool_id, sr] {
@@ -1483,7 +1627,7 @@ void ImpBase::create_context_menu(Gtk::Menu *parent, const std::set<SelectableRe
                 }
                 else {
                     if (get_action_sensitive(it.first) && (it.second.flags & ActionCatalogItem::FLAGS_SPECIFIC)) {
-                        auto la_sub = Gtk::manage(new Gtk::MenuItem(it.second.name));
+                        auto la_sub = create_context_menu_item(it.first);
                         ActionID action_id = it.first.first;
                         std::set<SelectableRef> sr(sel);
                         la_sub->signal_activate().connect([this, action_id, sr] {
@@ -1532,9 +1676,12 @@ bool ImpBase::handle_click(GdkEventButton *button_event)
     if (core->tool_is_active() && button_event->button != 2 && !(button_event->state & Gdk::SHIFT_MASK)
         && button_event->type != GDK_2BUTTON_PRESS && button_event->type != GDK_3BUTTON_PRESS) {
         ToolArgs args;
-        args.type = ToolEventType::CLICK;
+        args.type = ToolEventType::ACTION;
+        if (button_event->button == 1)
+            args.action = InToolActionID::LMB;
+        else
+            args.action = InToolActionID::RMB;
         args.coords = canvas->get_cursor_pos();
-        args.button = button_event->button;
         args.target = canvas->get_current_target();
         args.work_layer = canvas->property_work_layer();
         ToolResponse r = core->tool_update(args);
@@ -1550,7 +1697,7 @@ bool ImpBase::handle_click(GdkEventButton *button_event)
             }
         }
     }
-    else if (!core->tool_is_active() && button_event->button == 1) {
+    else if (!core->tool_is_active() && button_event->button == 1 && !(button_event->state & Gdk::SHIFT_MASK)) {
         handle_maybe_drag();
     }
     else if (!core->tool_is_active() && button_event->button == 3) {
@@ -1653,14 +1800,13 @@ void ImpBase::tool_process(ToolResponse &resp)
         imp_interface->dialogs.close_nonmodal();
         main_window->tool_hint_label->set_text(">");
         canvas->set_cursor_external(false);
+        canvas->snap_filter.clear();
         no_update = false;
         highlights.clear();
         update_highlights();
     }
     if (!no_update) {
-        canvas->fast_draw = resp.fast_draw;
         canvas_update();
-        canvas->fast_draw = false;
         canvas->set_selection(core->get_tool_selection());
     }
     if (resp.next_tool != ToolID::NONE) {
@@ -1716,13 +1862,14 @@ void ImpBase::update_property_panels()
     sel.insert(sel_extra.begin(), sel_extra.end());
     panels->update_objects(sel);
     bool show_properties = panels->get_selection().size() > 0;
-    main_window->property_scrolled_window->set_visible(show_properties);
+    main_window->property_scrolled_window->set_visible(show_properties && !distraction_free);
 }
 
 void ImpBase::handle_tool_change(ToolID id)
 {
     panels->set_sensitive(id == ToolID::NONE);
     canvas->set_selection_allowed(id == ToolID::NONE);
+    main_window->tool_bar_set_use_actions(core->get_tool_actions().size());
     if (id != ToolID::NONE) {
         main_window->tool_bar_set_tool_name(action_catalog.at({ActionID::TOOL, id}).name);
         main_window->tool_bar_set_tool_tip("");
@@ -1733,6 +1880,8 @@ void ImpBase::handle_tool_change(ToolID id)
         canvas->set_cursor_size(get_canvas_preferences()->appearance.cursor_size);
     }
     main_window->tool_bar_set_visible(id != ToolID::NONE);
+    tool_bar_clear_actions();
+    main_window->action_bar_revealer->set_reveal_child(id == ToolID::NONE);
 }
 
 void ImpBase::handle_warning_selected(const Coordi &pos)
@@ -1821,6 +1970,7 @@ void ImpBase::handle_search()
             q.types.insert(it.first);
         }
     }
+    q.exact = main_window->search_exact_cb->get_active();
     auto min_c = canvas->screen2canvas({0, canvas->get_height()});
     auto max_c = canvas->screen2canvas({canvas->get_width(), 0});
     q.area_visible = {min_c, max_c};
@@ -1938,7 +2088,7 @@ void ImpBase::update_search_types_label()
     main_window->search_expander->set_label(la);
 }
 
-std::pair<ActionID, ToolID> ImpBase::get_doubleclick_action(ObjectType type, const UUID &uu)
+ActionToolID ImpBase::get_doubleclick_action(ObjectType type, const UUID &uu)
 {
     switch (type) {
     case ObjectType::TEXT:
@@ -2005,6 +2155,9 @@ void ImpBase::set_window_title_from_block()
 std::vector<std::string> ImpBase::get_view_hints()
 {
     std::vector<std::string> r;
+    if (distraction_free)
+        r.emplace_back("distraction free mode");
+
     if (canvas->get_flip_view())
         r.emplace_back("bottom view");
 
@@ -2041,6 +2194,7 @@ void ImpBase::handle_select_polygon(const ActionConnection &a)
             unsigned int v = 0;
             for (const auto &it_v : poly->vertices) {
                 new_sel.emplace(it.uuid, ObjectType::POLYGON_VERTEX, v);
+                new_sel.emplace(it.uuid, ObjectType::POLYGON_EDGE, v);
                 if (it_v.type == Polygon::Vertex::Type::ARC)
                     new_sel.emplace(it.uuid, ObjectType::POLYGON_ARC_CENTER, v);
                 v++;
@@ -2051,4 +2205,98 @@ void ImpBase::handle_select_polygon(const ActionConnection &a)
     canvas->set_selection_mode(CanvasGL::SelectionMode::NORMAL);
 }
 
+ActionButton &ImpBase::add_action_button(ActionToolID action)
+{
+    main_window->set_use_action_bar(true);
+    auto ab = Gtk::manage(new ActionButton(action, action_connections));
+    ab->show();
+    ab->signal_action().connect([this](auto act) { this->trigger_action(act); });
+    main_window->action_bar_box->pack_start(*ab, false, false, 0);
+    action_buttons.push_back(ab);
+    return *ab;
+}
+
+ActionButtonMenu &ImpBase::add_action_button_menu(const char *icon_name)
+{
+    main_window->set_use_action_bar(true);
+    auto ab = Gtk::manage(new ActionButtonMenu(icon_name, action_connections));
+    ab->show();
+    ab->signal_action().connect([this](auto act) { this->trigger_action(act); });
+    main_window->action_bar_box->pack_start(*ab, false, false, 0);
+    action_buttons.push_back(ab);
+    return *ab;
+}
+
+void ImpBase::tool_bar_set_actions(const std::vector<ActionLabelInfo> &labels)
+{
+    if (in_tool_action_label_infos != labels) {
+        tool_bar_clear_actions();
+        for (const auto &it : labels) {
+            tool_bar_append_action(it.action1, it.action2, it.label);
+        }
+
+        in_tool_action_label_infos = labels;
+    }
+}
+
+void ImpBase::tool_bar_append_action(InToolActionID action1, InToolActionID action2, const std::string &s)
+{
+    auto box = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_HORIZONTAL, 5));
+    if (any_of(action1, {InToolActionID::LMB, InToolActionID::RMB})) {
+        std::string icon_name = "action-";
+        if (action1 == InToolActionID::LMB) {
+            icon_name += "lmb";
+        }
+        else {
+            icon_name += "rmb";
+        }
+        icon_name += "-symbolic";
+        auto img = Gtk::manage(new Gtk::Image);
+        img->set_from_icon_name(icon_name, Gtk::ICON_SIZE_BUTTON);
+        img->show();
+        box->pack_start(*img, false, false, 0);
+    }
+    else {
+        auto key_box = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_HORIZONTAL, 0));
+        for (const auto action : {action1, action2}) {
+            if (action != InToolActionID::NONE) {
+                const auto &prefs = in_tool_key_sequeces_preferences.keys;
+                if (prefs.count(action)) {
+                    if (prefs.at(action).size()) {
+                        auto seq = prefs.at(action).front();
+                        auto kl = Gtk::manage(new Gtk::Label(key_sequence_to_string_short(seq)));
+                        kl->set_valign(Gtk::ALIGN_BASELINE);
+                        key_box->pack_start(*kl, false, false, 0);
+                    }
+                }
+            }
+        }
+        key_box->show_all();
+        key_box->get_style_context()->add_class("imp-key-box");
+
+        box->pack_start(*key_box, false, false, 0);
+    }
+    const auto &as = s.size() ? s : in_tool_action_catalog.at(action1).name;
+
+    auto la = Gtk::manage(new Gtk::Label(as));
+    la->set_valign(Gtk::ALIGN_BASELINE);
+
+    la->show();
+
+    box->pack_start(*la, false, false, 0);
+
+    main_window->tool_bar_append_action(*box);
+}
+
+void ImpBase::tool_bar_clear_actions()
+{
+    in_tool_action_label_infos.clear();
+    main_window->tool_bar_clear_actions();
+}
+
+ObjectType ImpBase::get_editor_type() const
+{
+    return core->get_object_type();
+}
+
 } // namespace horizon
