@@ -20,12 +20,14 @@
 #include "pnp_export_window.hpp"
 #include "airwire_filter_window.hpp"
 #include "core/tool_id.hpp"
+#include "widgets/action_button.hpp"
+#include "parts_window.hpp"
 
 namespace horizon {
 ImpBoard::ImpBoard(const std::string &board_filename, const std::string &block_filename, const std::string &via_dir,
-                   const PoolParams &pool_params)
-    : ImpLayer(pool_params), core_board(board_filename, block_filename, via_dir, *pool),
-      project_dir(Glib::path_get_dirname(board_filename)), searcher(core_board), airwire_filter(*core_board.get_board())
+                   const std::string &pictures_dir, const PoolParams &pool_params)
+    : ImpLayer(pool_params), core_board(board_filename, block_filename, via_dir, pictures_dir, *pool),
+      project_dir(Glib::path_get_dirname(board_filename)), searcher(core_board)
 {
     core = &core_board;
     core_board.signal_tool_changed().connect(sigc::mem_fun(*this, &ImpBase::handle_tool_change));
@@ -36,41 +38,54 @@ void ImpBoard::canvas_update()
 {
     canvas->update(*core_board.get_canvas_data());
     warnings_box->update(core_board.get_board()->warnings);
+    apply_net_colors();
     update_highlights();
     tuning_window->update();
     update_text_owner_annotation();
 }
 
+void ImpBoard::update_airwire_annotation()
+{
+    airwire_annotation->clear();
+    for (const auto &[net, airwires] : core_board.get_board()->airwires) {
+        if (!airwire_filter_window->airwire_is_visible(net))
+            continue;
+        const bool highlight = highlights.count(ObjectRef(ObjectType::NET, net));
+        uint8_t color2 = 0;
+        if (net_color_map.count(net))
+            color2 = net_color_map.at(net);
+        if (core_board.tool_is_active() && !highlight && highlights.size())
+            continue;
+        for (const auto &airwire : airwires) {
+            airwire_annotation->draw_line(airwire.from.get_position(), airwire.to.get_position(), ColorP::AIRWIRE, 0,
+                                          highlight, color2);
+        }
+    }
+}
+
 void ImpBoard::update_highlights()
 {
-    canvas->set_flags_all(0, Triangle::FLAG_HIGHLIGHT);
+    canvas->set_flags_all(0, TriangleInfo::FLAG_HIGHLIGHT);
     canvas->set_highlight_enabled(highlights.size());
     for (const auto &it : highlights) {
         if (it.type == ObjectType::NET) {
             for (const auto &it_track : core_board.get_board()->tracks) {
                 if (it_track.second.net.uuid == it.uuid) {
                     ObjectRef ref(ObjectType::TRACK, it_track.first);
-                    canvas->set_flags(ref, Triangle::FLAG_HIGHLIGHT, 0);
-                }
-            }
-            {
-                const auto &airwires = core_board.get_board()->airwires;
-                if (airwires.count(it.uuid)) {
-                    ObjectRef ref(ObjectType::AIRWIRE, it.uuid);
-                    canvas->set_flags(ref, Triangle::FLAG_HIGHLIGHT, 0);
+                    canvas->set_flags(ref, TriangleInfo::FLAG_HIGHLIGHT, 0);
                 }
             }
             for (const auto &it_via : core_board.get_board()->vias) {
                 if (it_via.second.junction->net.uuid == it.uuid) {
                     ObjectRef ref(ObjectType::VIA, it_via.first);
-                    canvas->set_flags(ref, Triangle::FLAG_HIGHLIGHT, 0);
+                    canvas->set_flags(ref, TriangleInfo::FLAG_HIGHLIGHT, 0);
                 }
             }
             for (const auto &it_pkg : core_board.get_board()->packages) {
                 for (const auto &it_pad : it_pkg.second.package.pads) {
                     if (it_pad.second.net.uuid == it.uuid) {
                         ObjectRef ref(ObjectType::PAD, it_pad.first, it_pkg.first);
-                        canvas->set_flags(ref, Triangle::FLAG_HIGHLIGHT, 0);
+                        canvas->set_flags(ref, TriangleInfo::FLAG_HIGHLIGHT, 0);
                     }
                 }
             }
@@ -79,12 +94,46 @@ void ImpBoard::update_highlights()
             for (const auto &it_pkg : core_board.get_board()->packages) {
                 if (it_pkg.second.component->uuid == it.uuid) {
                     ObjectRef ref(ObjectType::BOARD_PACKAGE, it_pkg.first);
-                    canvas->set_flags(ref, Triangle::FLAG_HIGHLIGHT, 0);
+                    canvas->set_flags(ref, TriangleInfo::FLAG_HIGHLIGHT, 0);
                 }
             }
         }
         else {
-            canvas->set_flags(it, Triangle::FLAG_HIGHLIGHT, 0);
+            canvas->set_flags(it, TriangleInfo::FLAG_HIGHLIGHT, 0);
+        }
+    }
+    update_airwire_annotation();
+}
+
+void ImpBoard::apply_net_colors()
+{
+    canvas->reset_color2();
+    for (const auto &[net, color] : net_color_map) {
+        for (const auto &it_track : core_board.get_board()->tracks) {
+            if (it_track.second.net.uuid == net) {
+                ObjectRef ref(ObjectType::TRACK, it_track.first);
+                canvas->set_color2(ref, color);
+            }
+        }
+        for (const auto &it_via : core_board.get_board()->vias) {
+            if (it_via.second.junction->net.uuid == net) {
+                ObjectRef ref(ObjectType::VIA, it_via.first);
+                canvas->set_color2(ref, color);
+            }
+        }
+        for (const auto &it_pkg : core_board.get_board()->packages) {
+            for (const auto &it_pad : it_pkg.second.package.pads) {
+                if (it_pad.second.net.uuid == net) {
+                    ObjectRef ref(ObjectType::PAD, it_pad.first, it_pkg.first);
+                    canvas->set_color2(ref, color);
+                }
+            }
+        }
+        for (const auto &it_plane : core_board.get_board()->planes) {
+            if (it_plane.second.net.uuid == net) {
+                ObjectRef ref(ObjectType::PLANE, it_plane.first);
+                canvas->set_color2(ref, color);
+            }
         }
     }
 }
@@ -123,6 +172,18 @@ bool ImpBoard::handle_broadcast(const json &j)
             main_window->present(timestamp);
             trigger_action(ActionID::RELOAD_NETLIST);
         }
+        else if (op == "reload-netlist-hint") {
+            reload_netlist_delay_conn = Glib::signal_timeout().connect(
+                    [this] {
+#if GTK_CHECK_VERSION(3, 22, 0)
+                        reload_netlist_popover->popup();
+#else
+                        reload_netlist_popover->show();
+#endif
+                        return false;
+                    },
+                    500);
+        }
     }
     return true;
 }
@@ -231,6 +292,7 @@ void ImpBoard::apply_preferences()
     }
     canvas->set_highlight_on_top(preferences.board.highlight_on_top);
     canvas->show_text_in_tracks = preferences.board.show_text_in_tracks;
+    canvas->show_text_in_vias = preferences.board.show_text_in_vias;
     ImpLayer::apply_preferences();
     canvas_update_from_pp();
 }
@@ -357,7 +419,7 @@ void ImpBoard::construct()
     });
 
     hamburger_menu->append("Fabrication output", "win.fab_out");
-    main_window->add_action("fab_out", [this] { fab_output_window->present(); });
+    main_window->add_action("fab_out", [this] { trigger_action(ActionID::FAB_OUTPUT_WINDOW); });
 
     hamburger_menu->append("PDF Export", "win.export_pdf");
     main_window->add_action("export_pdf", [this] { trigger_action(ActionID::PDF_EXPORT_WINDOW); });
@@ -383,6 +445,11 @@ void ImpBoard::construct()
     hamburger_menu->append("Length tuning", "win.tuning");
     main_window->add_action("tuning", [this] { trigger_action(ActionID::TUNING); });
 
+    add_tool_action(ActionID::AIRWIRE_FILTER_WINDOW, "airwire_filter");
+    view_options_menu->append("Nets…", "win.airwire_filter");
+
+    view_options_menu->append("Bottom view", "win.bottom_view");
+
     if (sockets_connected) {
         hamburger_menu->append("Cross probing", "win.cross_probing");
         auto cp_action = main_window->add_action_bool("cross_probing", true);
@@ -448,24 +515,47 @@ void ImpBoard::construct()
         });
     }
 
-    add_tool_button(ToolID::MAP_PACKAGE, "Place package", false);
-
     connect_action(ActionID::RELOAD_NETLIST, [this](const ActionConnection &c) {
+        reload_netlist_delay_conn.disconnect();
+#if GTK_CHECK_VERSION(3, 22, 0)
+        reload_netlist_popover->popdown();
+#else
+        reload_netlist_popover->hide();
+#endif
         core_board.reload_netlist();
         core_board.set_needs_save();
         canvas_update();
+        airwire_filter_window->update_nets();
+        parts_window->update();
     });
 
     {
         auto button = create_action_button(make_action(ActionID::RELOAD_NETLIST));
         button->show();
         main_window->header->pack_end(*button);
+
+        reload_netlist_popover = Gtk::manage(new Gtk::Popover);
+        reload_netlist_popover->set_modal(false);
+        reload_netlist_popover->set_relative_to(*button);
+
+        auto la = Gtk::manage(
+                new Gtk::Label("Netlist has changed.\nReload the netlist to update the board to the latest netlist."));
+        la->show();
+        reload_netlist_popover->add(*la);
+        la->set_line_wrap(true);
+        la->set_max_width_chars(20);
+        la->property_margin() = 10;
     }
 
     fab_output_window = FabOutputWindow::create(main_window, &core_board, project_dir);
     core->signal_tool_changed().connect([this](ToolID t) { fab_output_window->set_can_generate(t == ToolID::NONE); });
     core->signal_rebuilt().connect([this] { fab_output_window->reload_layers(); });
     fab_output_window->signal_changed().connect([this] { core_board.set_needs_save(); });
+    connect_action(ActionID::FAB_OUTPUT_WINDOW, [this](const auto &c) { fab_output_window->present(); });
+    connect_action(ActionID::GEN_FAB_OUTPUT, [this](const auto &c) {
+        fab_output_window->present();
+        fab_output_window->generate();
+    });
 
     pdf_export_window =
             PDFExportWindow::create(main_window, &core_board, *core_board.get_pdf_export_settings(), project_dir);
@@ -485,6 +575,13 @@ void ImpBoard::construct()
         core_board.get_colors()->substrate = color_from_rgba(view_3d_window->get_substrate_color());
         core_board.set_needs_save();
     });
+    view_3d_window->signal_key_press_event().connect([this](GdkEventKey *ev) {
+        if (ev->keyval == GDK_KEY_Escape) {
+            main_window->present();
+            return true;
+        }
+        return false;
+    });
 
     step_export_window = StepExportWindow::create(main_window, &core_board, project_dir);
     step_export_window->signal_changed().connect([this] { core_board.set_needs_save(); });
@@ -494,8 +591,9 @@ void ImpBoard::construct()
         step_export_window->generate();
     });
 
-    tuning_window = new TuningWindow(core_board.get_board());
+    tuning_window = new TuningWindow(*core_board.get_board());
     tuning_window->set_transient_for(*main_window);
+    imp_interface->signal_request_length_tuning_ref().connect([this] { return tuning_window->get_ref_length(); });
 
     rules_window->signal_goto().connect([this](Coordi location, UUID sheet) { canvas->center_and_zoom(location); });
 
@@ -508,6 +606,25 @@ void ImpBoard::construct()
     connect_action(ActionID::TUNING_ADD_TRACKS, sigc::mem_fun(*this, &ImpBoard::handle_measure_tracks));
     connect_action(ActionID::TUNING_ADD_TRACKS_ALL, sigc::mem_fun(*this, &ImpBoard::handle_measure_tracks));
 
+    parts_window = new PartsWindow(*core_board.get_board());
+    parts_window->set_transient_for(*main_window);
+    connect_action(ActionID::PARTS_WINDOW, [this](const auto &a) { parts_window->present(); });
+    parts_window->update();
+    parts_window->signal_selected().connect([this](const auto &comps) {
+        highlights.clear();
+        std::set<UUID> pkgs;
+        for (const auto &[uu, pkg] : core_board.get_board()->packages) {
+            if (comps.count(pkg.component->uuid)) {
+                highlights.emplace(ObjectType::BOARD_PACKAGE, uu);
+                pkgs.insert(uu);
+            }
+        }
+        update_highlights();
+        view_3d_window->set_highlights(pkgs);
+    });
+    if (m_meta.count("parts"))
+        parts_window->load_from_json(m_meta.at("parts"));
+
     connect_action(ActionID::HIGHLIGHT_NET, [this](const auto &a) {
         highlights.clear();
         for (const auto &it : canvas->get_selection()) {
@@ -556,10 +673,8 @@ void ImpBoard::construct()
 
     board_display_options_box->signal_set_layer_display().connect([this](int index, const LayerDisplay &lda) {
         LayerDisplay ld = canvas->get_layer_display(index);
-        ld.types_force_outline = lda.types_force_outline;
         ld.types_visible = lda.types_visible;
         canvas->set_layer_display(index, ld);
-        canvas->queue_draw();
     });
     canvas->set_layer_display(10000, LayerDisplay(true, LayerDisplay::Mode::OUTLINE));
     core->signal_rebuilt().connect([this] { board_display_options_box->update(); });
@@ -582,6 +697,11 @@ void ImpBoard::construct()
     text_owner_annotation->set_visible(true);
     text_owner_annotation->set_display(LayerDisplay(true, LayerDisplay::Mode::OUTLINE));
 
+    airwire_annotation = canvas->create_annotation();
+    airwire_annotation->set_visible(true);
+    airwire_annotation->set_display(LayerDisplay(true, LayerDisplay::Mode::OUTLINE));
+    airwire_annotation->use_highlight = true;
+
     core_board.signal_rebuilt().connect(sigc::mem_fun(*this, &ImpBoard::update_text_owners));
     canvas->signal_hover_selection_changed().connect(sigc::mem_fun(*this, &ImpBoard::update_text_owner_annotation));
     canvas->signal_selection_changed().connect(sigc::mem_fun(*this, &ImpBoard::update_text_owner_annotation));
@@ -620,16 +740,24 @@ void ImpBoard::construct()
     core->signal_tool_changed().connect([this](ToolID t) { pnp_export_window->set_can_export(t == ToolID::NONE); });
     core->signal_rebuilt().connect([this] { pnp_export_window->update(); });
 
-    airwire_filter_window = AirwireFilterWindow::create(main_window, *core_board.get_block(), airwire_filter);
-    connect_action(ActionID::AIRWIRE_FILTER_WINDOW, [this](const auto &a) { airwire_filter_window->present(); });
-    core_board.signal_rebuilt().connect(sigc::mem_fun(*this, &ImpBoard::update_airwires));
-    update_airwires();
-    canvas->set_airwire_filter(&airwire_filter);
-    airwire_filter.signal_changed().connect([this] {
-        canvas_update_from_pp();
+    airwire_filter_window = AirwireFilterWindow::create(main_window, *core_board.get_board());
+    airwire_filter_window->update_nets();
+    airwire_filter_window->signal_changed().connect([this] {
+        update_net_colors();
+        apply_net_colors();
+        update_airwire_annotation();
         update_view_hints();
     });
-    connect_action(ActionID::RESET_AIRWIRE_FILTER, [this](const auto &a) { airwire_filter.set_all(true); });
+    airwire_filter_window->signal_selection_changed().connect([this](auto nets) {
+        highlights.clear();
+        for (const auto &net : nets) {
+            highlights.emplace(ObjectType::NET, net);
+        }
+        update_highlights();
+    });
+    connect_action(ActionID::AIRWIRE_FILTER_WINDOW, [this](const auto &a) { airwire_filter_window->present(); });
+    core_board.signal_rebuilt().connect(sigc::mem_fun(*this, &ImpBoard::update_airwires));
+    connect_action(ActionID::RESET_AIRWIRE_FILTER, [this](const auto &a) { airwire_filter_window->set_all(true); });
     connect_action(ActionID::FILTER_AIRWIRES, [this](const auto &a) {
         std::set<UUID> nets;
         const auto board = core_board.get_board();
@@ -665,16 +793,40 @@ void ImpBoard::construct()
             }
         }
 
-        airwire_filter.set_only(nets);
+        airwire_filter_window->set_only(nets);
     });
 
+    if (m_meta.count("nets"))
+        airwire_filter_window->load_from_json(m_meta.at("nets"));
+
+    {
+        auto &b = add_action_button(make_action(ToolID::ROUTE_TRACK_INTERACTIVE));
+        b.add_action(make_action(ToolID::ROUTE_DIFFPAIR_INTERACTIVE));
+    }
+    {
+        auto &b = add_action_button(make_action(ToolID::DRAW_POLYGON));
+        b.add_action(make_action(ToolID::DRAW_POLYGON_RECTANGLE));
+        b.add_action(make_action(ToolID::DRAW_POLYGON_CIRCLE));
+    }
+    {
+        auto &b = add_action_button(make_action(ToolID::PLACE_BOARD_HOLE));
+        b.add_action(make_action(ToolID::PLACE_VIA));
+    }
+    {
+        auto &b = add_action_button(make_action(ToolID::DRAW_LINE));
+        b.add_action(make_action(ToolID::DRAW_LINE_RECTANGLE));
+        b.add_action(make_action(ToolID::DRAW_ARC));
+    }
+
+    add_action_button(make_action(ToolID::PLACE_TEXT));
+    add_action_button(make_action(ToolID::DRAW_DIMENSION));
+
     display_control_notebook->show();
 }
 
 void ImpBoard::update_airwires()
 {
-    airwire_filter.update_from_board();
-    airwire_filter_window->update_from_filter();
+    airwire_filter_window->update_from_board();
 }
 
 void ImpBoard::update_text_owners()
@@ -777,6 +929,23 @@ std::string ImpBoard::get_hud_text(std::set<SelectableRef> &sel)
         sel_erase_type(sel, ObjectType::BOARD_PACKAGE);
     }
     trim(s);
+    if (sel_count_type(sel, ObjectType::POLYGON_VERTEX) == 1 || sel_count_type(sel, ObjectType::POLYGON_EDGE) == 1) {
+        const auto &brd = *core_board.get_board();
+        const Polygon *poly = nullptr;
+        if (sel_count_type(sel, ObjectType::POLYGON_VERTEX))
+            poly = &brd.polygons.at(sel_find_one(sel, ObjectType::POLYGON_VERTEX).uuid);
+        if (sel_count_type(sel, ObjectType::POLYGON_EDGE))
+            poly = &brd.polygons.at(sel_find_one(sel, ObjectType::POLYGON_EDGE).uuid);
+        if (poly) {
+            if (auto plane = dynamic_cast<const Plane *>(poly->usage.ptr)) {
+                s += "\n\n<b>Plane " + plane->net->name + "</b>\n";
+                s += "Priority: " + std::to_string(plane->priority) + "\n";
+                s += "Layer: ";
+                s += core_board.get_layer_provider()->get_layers().at(poly->layer).name + "\n";
+            }
+        }
+    }
+    trim(s);
     return s;
 }
 
@@ -793,6 +962,14 @@ void ImpBoard::handle_measure_tracks(const ActionConnection &a)
     tuning_window->present();
 }
 
+ToolID ImpBoard::get_tool_for_drag_move(bool ctrl, const std::set<SelectableRef> &sel) const
+{
+    if (preferences.board.move_using_router && sel.size() == 1 && sel_count_type(sel, ObjectType::TRACK) == 1)
+        return ToolID::DRAG_TRACK_INTERACTIVE;
+    else
+        return ImpBase::get_tool_for_drag_move(ctrl, sel);
+}
+
 void ImpBoard::handle_maybe_drag()
 {
     if (!preferences.board.drag_start_track) {
@@ -800,13 +977,21 @@ void ImpBoard::handle_maybe_drag()
         return;
     }
     auto target = canvas->get_current_target();
-    auto brd = core_board.get_board();
+    const auto brd = core_board.get_board();
     if (target.type == ObjectType::PAD) {
-        auto pkg = brd->packages.at(target.path.at(0));
-        auto pad = pkg.package.pads.at(target.path.at(1));
+        auto &pkg = brd->packages.at(target.path.at(0));
+        auto &pad = pkg.package.pads.at(target.path.at(1));
+        if (pad.padstack.type == Padstack::Type::MECHANICAL || pad.is_nc) {
+            ImpBase::handle_maybe_drag();
+            return;
+        }
     }
     else if (target.type == ObjectType::JUNCTION) {
-        auto ju = brd->junctions.at(target.path.at(0));
+        const auto &ju = brd->junctions.at(target.path.at(0));
+        if (!ju.net) {
+            ImpBase::handle_maybe_drag();
+            return;
+        }
     }
     else {
         ImpBase::handle_maybe_drag();
@@ -844,9 +1029,9 @@ void ImpBoard::handle_drag()
             }
             {
                 ToolArgs args;
-                args.type = ToolEventType::CLICK;
+                args.type = ToolEventType::ACTION;
                 args.coords = target_drag_begin.p;
-                args.button = 1;
+                args.action = InToolActionID::LMB;
                 args.target = target_drag_begin;
                 args.work_layer = canvas->property_work_layer();
                 ToolResponse r = core->tool_update(args);
@@ -864,9 +1049,9 @@ void ImpBoard::handle_drag()
             }
             {
                 ToolArgs args;
-                args.type = ToolEventType::CLICK;
+                args.type = ToolEventType::ACTION;
                 args.coords = target_drag_begin.p;
-                args.button = 1;
+                args.action = InToolActionID::LMB;
                 args.target = target_drag_begin;
                 args.work_layer = canvas->property_work_layer();
                 ToolResponse r = core->tool_update(args);
@@ -877,7 +1062,7 @@ void ImpBoard::handle_drag()
     }
 }
 
-std::pair<ActionID, ToolID> ImpBoard::get_doubleclick_action(ObjectType type, const UUID &uu)
+ActionToolID ImpBoard::get_doubleclick_action(ObjectType type, const UUID &uu)
 {
     auto a = ImpBase::get_doubleclick_action(type, uu);
     if (a.first != ActionID::NONE)
@@ -963,6 +1148,7 @@ std::map<ObjectType, ImpBase::SelectionFilterInfo> ImpBoard::get_selection_filte
             {ObjectType::BOARD_HOLE, {}},
             {ObjectType::CONNECTION_LINE, {}},
             {ObjectType::BOARD_PANEL, {}},
+            {ObjectType::PICTURE, {}},
     };
     return r;
 }
@@ -987,17 +1173,52 @@ void ImpBoard::get_save_meta(json &j)
 {
     ImpLayer::get_save_meta(j);
     j["board_display_options"] = board_display_options_box->serialize();
+    j["nets"] = airwire_filter_window->serialize();
+    j["parts"] = parts_window->serialize();
 }
 
 std::vector<std::string> ImpBoard::get_view_hints()
 {
     auto r = ImpBase::get_view_hints();
 
-    const auto &aws = airwire_filter.get_airwires();
-    if (std::any_of(aws.begin(), aws.end(), [](const auto &x) { return x.second.visible == false; }))
+    if (airwire_filter_window->get_filtered())
         r.emplace_back("airwires filtered");
 
     return r;
 }
 
+void ImpBoard::update_net_colors()
+{
+    auto &net_colors = airwire_filter_window->get_net_colors();
+    net_color_map.clear();
+    if (net_colors.size()) {
+        std::vector<ColorI> t;
+        t.emplace_back(ColorI{0, 0, 0});
+
+        std::map<ColorI, int> colors_idxs;
+        auto get_or_create_color = [&t, &colors_idxs](ColorI c) -> int {
+            if (colors_idxs.count(c)) {
+                return colors_idxs.at(c);
+            }
+            else {
+                t.emplace_back(c);
+                const auto i = t.size() - 1;
+                colors_idxs.emplace(c, i);
+                return i;
+            }
+        };
+
+        for (const auto &[net, color] : net_colors) {
+            net_color_map[net] = get_or_create_color(color);
+        }
+        canvas->set_colors2(t);
+    }
+}
+
+ImpBoard::~ImpBoard()
+{
+    reload_netlist_delay_conn.disconnect();
+    delete view_3d_window;
+}
+
 } // namespace horizon
