@@ -1,20 +1,21 @@
 #pragma once
 #include "core/core_board.hpp"
 #include "imp_layer.hpp"
-#include "airwire_filter.hpp"
 #include "search/searcher_board.hpp"
 
 namespace horizon {
 class ImpBoard : public ImpLayer {
 public:
     ImpBoard(const std::string &board_filename, const std::string &block_filename, const std::string &via_dir,
-             const PoolParams &params);
+             const std::string &pictures_dir, const PoolParams &params);
 
     const std::map<int, Layer> &get_layers();
     void update_highlights() override;
 
     std::map<ObjectType, SelectionFilterInfo> get_selection_filter_info() const override;
 
+    ~ImpBoard();
+
 protected:
     void construct() override;
     bool handle_broadcast(const json &j) override;
@@ -26,13 +27,9 @@ protected:
     {
         return ActionCatalogItem::AVAILABLE_IN_BOARD;
     };
-    ObjectType get_editor_type() const override
-    {
-        return ObjectType::BOARD;
-    }
 
     std::string get_hud_text(std::set<SelectableRef> &sel) override;
-    std::pair<ActionID, ToolID> get_doubleclick_action(ObjectType type, const UUID &uu) override;
+    ActionToolID get_doubleclick_action(ObjectType type, const UUID &uu) override;
 
     void get_save_meta(json &j) override;
     std::vector<std::string> get_view_hints() override;
@@ -42,6 +39,8 @@ protected:
         return &searcher;
     }
 
+    ToolID get_tool_for_drag_move(bool ctrl, const std::set<SelectableRef> &sel) const override;
+
 private:
     void canvas_update() override;
     void handle_selection_cross_probe();
@@ -58,6 +57,7 @@ private:
     class BoardDisplayOptionsBox *board_display_options_box = nullptr;
     class PnPExportWindow *pnp_export_window = nullptr;
     class AirwireFilterWindow *airwire_filter_window = nullptr;
+    class PartsWindow *parts_window = nullptr;
     bool cross_probing_enabled = false;
 
     Coordf cursor_pos_drag_begin;
@@ -80,6 +80,13 @@ private:
 
     int get_schematic_pid();
 
-    AirwireFilter airwire_filter;
+    class CanvasAnnotation *airwire_annotation = nullptr;
+    void update_airwire_annotation();
+    void update_net_colors();
+    void apply_net_colors();
+    std::map<UUID, uint8_t> net_color_map;
+
+    Gtk::Popover *reload_netlist_popover = nullptr;
+    sigc::connection reload_netlist_delay_conn;
 };
 } // namespace horizon
