@@ -5,6 +5,7 @@
 #include "widgets/layer_box.hpp"
 #include "pool/part.hpp"
 #include "nlohmann/json.hpp"
+#include "util/util.hpp"
 
 namespace horizon {
 ImpInterface::ImpInterface(ImpBase *i) : imp(i)
@@ -48,9 +49,9 @@ class CanvasGL *ImpInterface::get_canvas()
     return imp->canvas;
 }
 
-uint64_t ImpInterface::get_grid_spacing()
+Coordi ImpInterface::get_grid_spacing() const
 {
-    return imp->canvas->property_grid_spacing();
+    return imp->canvas->get_grid_spacing();
 }
 
 void ImpInterface::set_no_update(bool v)
@@ -90,5 +91,36 @@ const LayerDisplay &ImpInterface::get_layer_display(int layer) const
     return imp->canvas->get_layer_display(layer);
 }
 
+void ImpInterface::set_snap_filter(const std::set<SnapFilter> &filter)
+{
+    imp->canvas->snap_filter = filter;
+}
+
+void ImpInterface::set_snap_filter_from_selection(const std::set<SelectableRef> &sel)
+{
+    std::set<SnapFilter> sf;
+    for (const auto &it : sel) {
+        sf.emplace(it.type, it.uuid, it.vertex);
+        if (it.type == ObjectType::BOARD_PACKAGE)
+            sf.emplace(ObjectType::PAD, it.uuid);
+        else if (it.type == ObjectType::SCHEMATIC_SYMBOL)
+            sf.emplace(ObjectType::SYMBOL_PIN, it.uuid);
+        else if (it.type == ObjectType::POLYGON_VERTEX)
+            sf.emplace(ObjectType::POLYGON_EDGE, it.uuid, it.vertex);
+        else if (it.type == ObjectType::DIMENSION)
+            sf.emplace(ObjectType::DIMENSION, it.uuid, -1);
+    }
+    imp->canvas->snap_filter = sf;
+}
+
+uint64_t ImpInterface::get_length_tuning_ref() const
+{
+    return s_signal_request_length_tuning_ref.emit();
+}
+
+void ImpInterface::tool_bar_set_actions(const std::vector<ActionLabelInfo> &labels)
+{
+    imp->tool_bar_set_actions(labels);
+}
 
 } // namespace horizon
