@@ -21,6 +21,9 @@
 #include <gtkmm.h>
 #include <iostream>
 #include <memory>
+#ifdef G_OS_WIN32
+#include <windows.h>
+#endif
 
 using horizon::Canvas;
 using horizon::LutEnumStr;
@@ -31,6 +34,10 @@ using json = nlohmann::json;
 
 int main(int argc, char *argv[])
 {
+#ifdef G_OS_WIN32
+    SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOGPFAULTERRORBOX | SEM_NOOPENFILEERRORBOX);
+#endif
+
     gtk_disable_setlocale();
     Gio::init();
     horizon::PoolManager::init();
@@ -108,7 +115,8 @@ int main(int argc, char *argv[])
 
     std::unique_ptr<horizon::ImpBase> imp = nullptr;
     if (mode_sch) {
-        imp.reset(new horizon::ImpSchematic(filenames.at(0), filenames.at(1), {pool_base_path, pool_cache_path}));
+        imp.reset(new horizon::ImpSchematic(filenames.at(0), filenames.at(1), filenames.at(2),
+                                            {pool_base_path, pool_cache_path}));
     }
     else if (mode_symbol) {
         imp.reset(new horizon::ImpSymbol(filenames.at(0), pool_base_path));
@@ -120,7 +128,7 @@ int main(int argc, char *argv[])
         imp.reset(new horizon::ImpPackage(filenames.at(0), pool_base_path));
     }
     else if (mode_board) {
-        imp.reset(new horizon::ImpBoard(filenames.at(0), filenames.at(1), filenames.at(2),
+        imp.reset(new horizon::ImpBoard(filenames.at(0), filenames.at(1), filenames.at(2), filenames.at(3),
                                         {pool_base_path, pool_cache_path}));
     }
     else if (mode_frame) {
