@@ -14,11 +14,7 @@ protected:
     {
         return ActionCatalogItem::AVAILABLE_IN_PADSTACK;
     };
-    ObjectType get_editor_type() const override
-    {
-        return ObjectType::PADSTACK;
-    }
-    std::pair<ActionID, ToolID> get_doubleclick_action(ObjectType type, const UUID &uu) override;
+    ActionToolID get_doubleclick_action(ObjectType type, const UUID &uu) override;
 
     std::map<ObjectType, SelectionFilterInfo> get_selection_filter_info() const override;
 
