@@ -16,11 +16,12 @@
 #include "core/tools/tool_map_symbol.hpp"
 #include "widgets/unplaced_box.hpp"
 #include "core/tool_id.hpp"
+#include "widgets/action_button.hpp"
 
 namespace horizon {
 ImpSchematic::ImpSchematic(const std::string &schematic_filename, const std::string &block_filename,
-                           const PoolParams &pool_params)
-    : ImpBase(pool_params), core_schematic(schematic_filename, block_filename, *pool),
+                           const std::string &pictures_dir, const PoolParams &pool_params)
+    : ImpBase(pool_params), core_schematic(schematic_filename, block_filename, pictures_dir, *pool),
       project_dir(Glib::path_get_dirname(schematic_filename)), searcher(core_schematic)
 {
     core = &core_schematic;
@@ -42,12 +43,21 @@ void ImpSchematic::handle_select_sheet(Sheet *sh)
 
     auto v = canvas->get_scale_and_offset();
     sheet_views[core_schematic.get_sheet()->uuid] = v;
+    sheet_selections[core_schematic.get_sheet()->uuid] = canvas->get_selection();
+    auto highlights_saved = highlights;
     core_schematic.set_sheet(sh->uuid);
     canvas_update();
     if (sheet_views.count(sh->uuid)) {
         auto v2 = sheet_views.at(sh->uuid);
         canvas->set_scale_and_offset(v2.first, v2.second);
     }
+    if (sheet_selections.count(sh->uuid)) {
+        canvas->set_selection(sheet_selections.at(sh->uuid));
+    }
+    else {
+        canvas->set_selection({});
+    }
+    highlights = highlights_saved;
     update_highlights();
 }
 
@@ -120,10 +130,10 @@ void ImpSchematic::update_highlights()
     }
 
     auto sheet = core_schematic.get_sheet();
-    canvas->set_flags_all(0, Triangle::FLAG_HIGHLIGHT);
+    canvas->set_flags_all(0, TriangleInfo::FLAG_HIGHLIGHT);
     canvas->set_highlight_enabled(highlights_for_sheet[sheet->uuid].size());
     for (const auto &it : highlights_for_sheet[sheet->uuid]) {
-        canvas->set_flags(it, Triangle::FLAG_HIGHLIGHT, 0);
+        canvas->set_flags(it, TriangleInfo::FLAG_HIGHLIGHT, 0);
     }
 }
 
@@ -301,14 +311,6 @@ void ImpSchematic::construct()
 
     canvas->set_highlight_mode(CanvasGL::HighlightMode::DIM);
 
-    {
-        auto button = Gtk::manage(new Gtk::Button("Place part"));
-        button->signal_clicked().connect([this] { trigger_action(ActionID::PLACE_PART); });
-        button->show();
-        core->signal_tool_changed().connect([button](ToolID t) { button->set_sensitive(t == ToolID::NONE); });
-        main_window->header->pack_end(*button);
-    }
-
     connect_action(ActionID::PLACE_PART, [this](const auto &conn) {
         if (!sockets_connected) {
             this->tool_begin(ToolID::ADD_PART);
@@ -343,10 +345,6 @@ void ImpSchematic::construct()
             this->send_json(j);
         });
         set_action_sensitive(make_action(ActionID::TO_BOARD), false);
-        auto button = create_action_button(make_action(ActionID::TO_BOARD));
-        button->set_tooltip_text("Place selected components on board");
-        button->show();
-        main_window->header->pack_end(*button);
 
         connect_action(ActionID::SHOW_IN_BROWSER, [this](const auto &conn) {
             for (const auto &it : canvas->get_selection()) {
@@ -482,9 +480,25 @@ void ImpSchematic::construct()
     core_schematic.signal_tool_changed().connect(
             [this](ToolID tool_id) { unplaced_box->set_sensitive(tool_id == ToolID::NONE); });
 
-    grid_spin_button->set_sensitive(false);
+    grid_controller->disable();
     canvas->snap_to_targets = false;
 
+    connect_action(ActionID::TOGGLE_SNAP_TO_TARGETS, [this](const ActionConnection &c) {
+        canvas->snap_to_targets = !canvas->snap_to_targets;
+        g_simple_action_set_state(toggle_snap_to_targets_action->gobj(),
+                                  g_variant_new_boolean(canvas->snap_to_targets));
+    });
+    view_options_menu->append("Snap to targets", "win.snap_to_targets");
+
+    toggle_snap_to_targets_action = main_window->add_action_bool("snap_to_targets", canvas->snap_to_targets);
+    toggle_snap_to_targets_action->signal_change_state().connect([this](const Glib::VariantBase &v) {
+        auto b = Glib::VariantBase::cast_dynamic<Glib::Variant<bool>>(v).get();
+        if (b != canvas->snap_to_targets) {
+            trigger_action(ActionID::TOGGLE_SNAP_TO_TARGETS);
+        }
+    });
+
+
     rules_window->signal_goto().connect([this](Coordi location, UUID sheet) {
         auto sch = core_schematic.get_schematic();
         if (sch->sheets.count(sheet)) {
@@ -505,9 +519,39 @@ void ImpSchematic::construct()
         return false;
     });
 
+    main_window->signal_activate_hud_link().connect(
+            [this](const std::string &url) {
+                if (url.find("s:") == 0) {
+                    std::string sheet = url.substr(2);
+                    auto uuid = UUID(sheet);
+                    auto sch = core_schematic.get_schematic();
+                    if (sch->sheets.count(uuid)) {
+                        sheet_box->select_sheet(uuid);
+                    }
+                    return true;
+                }
+                return false;
+            },
+            false);
+
+    add_action_button(make_action(ActionID::PLACE_PART));
+    add_action_button(make_action(ToolID::DRAW_NET));
+    {
+        auto &b = add_action_button(make_action(ToolID::PLACE_NET_LABEL));
+        b.add_action(make_action(ToolID::PLACE_BUS_LABEL));
+        b.add_action(make_action(ToolID::PLACE_BUS_RIPPER));
+    }
+    add_action_button(make_action(ToolID::PLACE_POWER_SYMBOL));
+    {
+        auto &b = add_action_button(make_action(ToolID::DRAW_LINE));
+        b.add_action(make_action(ToolID::DRAW_LINE_RECTANGLE));
+        b.add_action(make_action(ToolID::DRAW_ARC));
+    }
+    add_action_button(make_action(ToolID::PLACE_TEXT));
+
     if (!core_schematic.get_project_meta_loaded_from_block())
         core_schematic.set_needs_save();
-} // namespace horizon
+}
 
 void ImpSchematic::update_action_sensitivity()
 {
@@ -574,6 +618,27 @@ std::string ImpSchematic::get_hud_text(std::set<SelectableRef> &sel)
         s += get_hud_text_for_component(sym.component);
         sel_erase_type(sel, ObjectType::SCHEMATIC_SYMBOL);
     }
+
+    if (sel_count_type(sel, ObjectType::TEXT) == 1) {
+        const auto text = core->get_text(sel_find_one(sel, ObjectType::TEXT).uuid);
+        const auto txt = Glib::ustring(text->text);
+        Glib::MatchInfo ma;
+        if (core_schematic.get_schematic()->get_sheetref_regex()->match(txt, ma)) {
+            s += "\n\n<b>Text with sheet references</b>\n";
+            do {
+                auto uuid = ma.fetch(1);
+                std::string url = "s:" + uuid;
+                if (core_schematic.get_schematic()->sheets.count(UUID(uuid))) {
+                    s += make_link_markup(url, core_schematic.get_schematic()->sheets.at(UUID(uuid)).name);
+                }
+                else {
+                    s += "Unknown Sheet";
+                }
+                s += "\n";
+            } while (ma.next());
+            sel_erase_type(sel, ObjectType::TEXT);
+        }
+    }
     trim(s);
     return s;
 }
@@ -678,9 +743,9 @@ void ImpSchematic::handle_drag()
         }
         {
             ToolArgs args;
-            args.type = ToolEventType::CLICK;
+            args.type = ToolEventType::ACTION;
             args.coords = target_drag_begin.p;
-            args.button = 1;
+            args.action = InToolActionID::LMB;
             args.target = target_drag_begin;
             args.work_layer = canvas->property_work_layer();
             ToolResponse r = core->tool_update(args);
@@ -959,7 +1024,7 @@ void ImpSchematic::handle_move_to_other_sheet(const ActionConnection &conn)
     tool_begin(ToolID::MOVE);
 }
 
-std::pair<ActionID, ToolID> ImpSchematic::get_doubleclick_action(ObjectType type, const UUID &uu)
+ActionToolID ImpSchematic::get_doubleclick_action(ObjectType type, const UUID &uu)
 {
     auto a = ImpBase::get_doubleclick_action(type, uu);
     if (a.first != ActionID::NONE)
@@ -979,6 +1044,19 @@ void ImpSchematic::update_unplaced()
     unplaced_box->update(core_schematic.get_schematic()->get_unplaced_gates());
 }
 
+ToolID ImpSchematic::get_tool_for_drag_move(bool ctrl, const std::set<SelectableRef> &sel) const
+{
+    if (preferences.schematic.bend_non_ortho && sel.size() == 1 && sel.begin()->type == ObjectType::LINE_NET) {
+        const auto &ln = core_schematic.get_sheet()->net_lines.at(sel.begin()->uuid);
+        auto from = ln.from.get_position();
+        auto to = ln.to.get_position();
+        if (!((from.x == to.x) || (from.y == to.y))) {
+            return ToolID::BEND_LINE_NET;
+        }
+    }
+    return ImpBase::get_tool_for_drag_move(ctrl, sel);
+}
+
 void ImpSchematic::expand_selection_for_property_panel(std::set<SelectableRef> &sel_extra,
                                                        const std::set<SelectableRef> &sel)
 {
