@@ -7,7 +7,8 @@ namespace horizon {
 class ImpSchematic : public ImpBase {
 
 public:
-    ImpSchematic(const std::string &schematic_filename, const std::string &block_filename, const PoolParams &params);
+    ImpSchematic(const std::string &schematic_filename, const std::string &block_filename,
+                 const std::string &pictures_dir, const PoolParams &params);
     void update_highlights() override;
 
 protected:
@@ -20,14 +21,10 @@ protected:
     {
         return ActionCatalogItem::AVAILABLE_IN_SCHEMATIC;
     };
-    ObjectType get_editor_type() const override
-    {
-        return ObjectType::SCHEMATIC;
-    }
 
     std::string get_hud_text(std::set<SelectableRef> &sel) override;
     void search_center(const Searcher::SearchResult &res) override;
-    std::pair<ActionID, ToolID> get_doubleclick_action(ObjectType type, const UUID &uu) override;
+    ActionToolID get_doubleclick_action(ObjectType type, const UUID &uu) override;
     void expand_selection_for_property_panel(std::set<SelectableRef> &sel_extra,
                                              const std::set<SelectableRef> &sel) override;
 
@@ -36,6 +33,9 @@ protected:
         return &searcher;
     }
 
+    ToolID get_tool_for_drag_move(bool ctrl, const std::set<SelectableRef> &sel) const override;
+
+
 private:
     void canvas_update() override;
     CoreSchematic core_schematic;
@@ -53,6 +53,7 @@ private:
     std::string last_pdf_filename;
 
     std::map<UUID, std::pair<float, Coordf>> sheet_views;
+    std::map<UUID, std::set<SelectableRef>> sheet_selections;
     class SheetBox *sheet_box;
     void handle_selection_cross_probe();
     bool cross_probing_enabled = false;
@@ -67,6 +68,8 @@ private:
 
     void handle_drag();
 
+    Glib::RefPtr<Gio::SimpleAction> toggle_snap_to_targets_action;
+
     int get_board_pid();
 };
 } // namespace horizon
