@@ -5,6 +5,7 @@
 #include "symbol_preview/symbol_preview_window.hpp"
 #include "widgets/unplaced_box.hpp"
 #include "core/tool_id.hpp"
+#include "widgets/action_button.hpp"
 
 namespace horizon {
 ImpSymbol::ImpSymbol(const std::string &symbol_filename, const std::string &pool_path)
@@ -92,10 +93,18 @@ void ImpSymbol::construct()
     name_entry->signal_changed().connect([this] { core_symbol.set_needs_save(); });
     name_entry->signal_activate().connect(sigc::mem_fun(*this, &ImpSymbol::update_header));
 
-    unit_label = Gtk::manage(new Gtk::Label(core_symbol.get_symbol()->unit->name));
-    unit_label->set_xalign(0);
-    unit_label->set_selectable(true);
-    header_button->add_widget("Unit", unit_label);
+    {
+        auto box = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_HORIZONTAL, 10));
+        unit_label = Gtk::manage(new Gtk::Label(core_symbol.get_symbol()->unit->name));
+        unit_label->set_xalign(0);
+        unit_label->set_selectable(true);
+        box->pack_start(*unit_label, true, true, 0);
+        auto button = create_action_button(make_action(ToolID::CHANGE_UNIT));
+        button->set_label("Change");
+        box->pack_start(*button, false, false, 0);
+        box->show_all();
+        header_button->add_widget("Unit", box);
+    }
 
     can_expand_switch = Gtk::manage(new Gtk::Switch);
     can_expand_switch->set_active(core_symbol.get_symbol()->can_expand);
@@ -150,7 +159,7 @@ void ImpSymbol::construct()
         sym->text_placements = symbol_preview_window->get_text_placements();
         sym->can_expand = can_expand_switch->get_active();
     });
-    grid_spin_button->set_sensitive(false);
+    grid_controller->disable();
     update_monitor();
 
     connect_action(ActionID::EDIT_UNIT, [this](const auto &a) {
@@ -172,6 +181,19 @@ void ImpSymbol::construct()
     bbox_annotation->set_visible(true);
     bbox_annotation->on_top = false;
     bbox_annotation->set_display(LayerDisplay(true, LayerDisplay::Mode::OUTLINE));
+
+    if (sockets_connected) {
+        canvas->signal_selection_changed().connect(sigc::mem_fun(*this, &ImpSymbol::handle_selection_cross_probe));
+    }
+
+    add_action_button(make_action(ToolID::DRAW_LINE_RECTANGLE));
+    add_action_button(make_action(ToolID::PLACE_REFDES_AND_VALUE));
+    {
+        auto &x = add_action_button(make_action(ToolID::DRAW_LINE));
+        x.add_action(make_action(ToolID::DRAW_ARC));
+    }
+    add_action_button(make_action(ToolID::PLACE_TEXT));
+    add_action_button(make_action(ToolID::RESIZE_SYMBOL));
 }
 
 void ImpSymbol::update_header()
@@ -209,4 +231,21 @@ void ImpSymbol::update_bbox_annotation()
     bbox_annotation->draw_line(Coordf(a.x, b.y), a, ColorP::PIN_HIDDEN, 0);
 }
 
+
+void ImpSymbol::handle_selection_cross_probe()
+{
+    if (core_symbol.tool_is_active())
+        return;
+    json j;
+    j["op"] = "symbol-select";
+    j["unit"] = (std::string)core_symbol.get_symbol()->unit->uuid;
+    auto pins = json::array();
+    for (const auto &it : canvas->get_selection()) {
+        if (it.type == ObjectType::SYMBOL_PIN) {
+            pins.push_back((std::string)it.uuid);
+        }
+    }
+    j["pins"] = pins;
+    send_json(j);
+}
 } // namespace horizon
