@@ -15,10 +15,6 @@ protected:
     {
         return ActionCatalogItem::AVAILABLE_IN_SYMBOL;
     };
-    ObjectType get_editor_type() const override
-    {
-        return ObjectType::SYMBOL;
-    }
 
     void update_monitor() override;
 
@@ -41,6 +37,7 @@ private:
     class UnplacedBox *unplaced_box = nullptr;
     void update_unplaced();
     void update_header();
+    void handle_selection_cross_probe();
 
     class CanvasAnnotation *bbox_annotation = nullptr;
     void update_bbox_annotation();
