@@ -1,13 +1,13 @@
 #include "pdf_export_window.hpp"
 #include "util/util.hpp"
-#include "core/core_schematic.hpp"
-#include "core/core_board.hpp"
+#include "document/idocument_board.hpp"
+#include "document/idocument_schematic.hpp"
 #include "export_pdf/export_pdf.hpp"
 #include "export_pdf/export_pdf_board.hpp"
 #include "util/gtk_util.hpp"
 #include "util/util.hpp"
 #include "widgets/spin_button_dim.hpp"
-#include "core/core.hpp"
+#include "common/layer_provider.hpp"
 #include <podofo/podofo.h>
 #ifdef G_OS_WIN32
 #undef DELETE
@@ -83,7 +83,7 @@ void PDFExportWindow::MyExportFileChooser::prepare_filename(std::string &filenam
     }
 }
 
-PDFExportWindow *PDFExportWindow::create(Gtk::Window *p, Core *c, PDFExportSettings &s, const std::string &pd)
+PDFExportWindow *PDFExportWindow::create(Gtk::Window *p, IDocument *c, PDFExportSettings &s, const std::string &pd)
 {
     PDFExportWindow *w;
     Glib::RefPtr<Gtk::Builder> x = Gtk::Builder::create();
@@ -93,7 +93,7 @@ PDFExportWindow *PDFExportWindow::create(Gtk::Window *p, Core *c, PDFExportSetti
     return w;
 }
 
-PDFExportWindow::PDFExportWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, Core *c,
+PDFExportWindow::PDFExportWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, IDocument *c,
                                  PDFExportSettings &s, const std::string &pd)
     : Gtk::Window(cobject), core(c), settings(s)
 {
@@ -122,7 +122,7 @@ PDFExportWindow::PDFExportWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk
         grid_attach_label_and_widget(grid, "Min. line width", sp, top);
     }
 
-    if (dynamic_cast<CoreSchematic *>(core)) {
+    if (dynamic_cast<IDocumentSchematic *>(core)) {
         Gtk::ScrolledWindow *sc;
         x->get_widget("layers_sc", sc);
         sc->set_visible(false);
@@ -147,7 +147,10 @@ PDFExportWindow::PDFExportWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk
     export_filechooser.attach(filename_entry, filename_button, this);
     export_filechooser.set_project_dir(pd);
     export_filechooser.bind_filename(settings.output_filename);
-    export_filechooser.signal_changed().connect([this] { s_signal_changed.emit(); });
+    export_filechooser.signal_changed().connect([this] {
+        s_signal_changed.emit();
+        update_export_button();
+    });
 
     status_dispatcher.attach(progress_label);
     status_dispatcher.attach(progress_bar);
@@ -155,16 +158,22 @@ PDFExportWindow::PDFExportWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk
     status_dispatcher.attach(progress_revealer);
     status_dispatcher.signal_notified().connect([this](const StatusDispatcher::Notification &n) {
         is_busy = n.status == StatusDispatcher::Status::BUSY;
-        export_button->set_sensitive(!is_busy);
+        update_export_button();
         header->set_show_close_button(!is_busy);
     });
 
     signal_delete_event().connect([this](GdkEventAny *ev) { return is_busy; });
     reload_layers();
+    update_export_button();
 }
 
 void PDFExportWindow::reload_layers()
 {
+    if (n_layers == settings.layers.size())
+        return;
+    else
+        n_layers = settings.layers.size();
+
     {
         auto children = layers_box->get_children();
         for (auto ch : children)
@@ -198,16 +207,30 @@ void PDFExportWindow::generate()
     thr.detach();
 }
 
+void PDFExportWindow::update_export_button()
+{
+    std::string txt;
+    if (!is_busy) {
+        if (settings.output_filename.size() == 0) {
+            txt = "output filename not set";
+        }
+    }
+    else {
+        txt = "busy";
+    }
+    widget_set_insensitive_tooltip(*export_button, txt);
+}
+
 void PDFExportWindow::export_thread(PDFExportSettings s)
 {
     try {
-        if (auto core_sch = dynamic_cast<CoreSchematic *>(core)) {
+        if (auto core_sch = dynamic_cast<IDocumentSchematic *>(core)) {
             export_pdf(*core_sch->get_schematic(), s, [this](std::string st, double p) {
                 status_dispatcher.set_status(StatusDispatcher::Status::BUSY, st, p);
             });
             status_dispatcher.set_status(StatusDispatcher::Status::DONE, "Done", 1);
         }
-        else if (auto core_brd = dynamic_cast<CoreBoard *>(core)) {
+        else if (auto core_brd = dynamic_cast<IDocumentBoard *>(core)) {
             export_pdf(*core_brd->get_board(), s, [this](std::string st, double p) {
                 status_dispatcher.set_status(StatusDispatcher::Status::BUSY, st, p);
             });
