@@ -69,6 +69,7 @@ void RuleEditorClearanceCopperOther::populate()
 
     auto sp_routing_offset = Gtk::manage(new SpinButtonDim);
     sp_routing_offset->set_range(0, 10_mm);
+    sp_routing_offset->set_width_chars(8);
     sp_routing_offset->set_value(rule2->routing_offset);
     sp_routing_offset->signal_value_changed().connect([this, sp_routing_offset] {
         rule2->routing_offset = sp_routing_offset->get_value_as_int();
@@ -133,6 +134,7 @@ void RuleEditorClearanceCopperOther::populate()
             for (const auto it_row : patch_types_ncu) {
                 auto *sp = Gtk::manage(new SpinButtonDim());
                 sp->set_range(0, 100_mm);
+                sp->set_width_chars(8);
                 sp->set_value(rule2->get_clearance(it_col, it_row));
                 sp->signal_value_changed().connect([this, sp, it_row, it_col] {
                     rule2->set_clearance(it_col, it_row, sp->get_value_as_int());
@@ -158,12 +160,11 @@ void RuleEditorClearanceCopperOther::set_some(int row, int col)
 {
     Dialogs dias;
     dias.set_parent(dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW)));
-    auto res = dias.ask_datum("Enter clearance", 0.1_mm);
-    if (!res.first)
-        return;
-    for (auto &it : spin_buttons) {
-        if ((it.first.first == row || row == -1) && (it.first.second == col || col == -1)) {
-            it.second->set_value(res.second);
+    if (auto r = dias.ask_datum("Enter clearance", 0.1_mm)) {
+        for (auto &it : spin_buttons) {
+            if ((it.first.first == row || row == -1) && (it.first.second == col || col == -1)) {
+                it.second->set_value(*r);
+            }
         }
     }
 }
