@@ -13,6 +13,7 @@
 #include "rule_editor_track_width.hpp"
 #include "rule_editor_via.hpp"
 #include "rule_editor_clearance_copper_keepout.hpp"
+#include "rule_editor_layer_pair.hpp"
 #include "rules/cache.hpp"
 #include "rules/rule_descr.hpp"
 #include "rules/rules.hpp"
@@ -73,7 +74,8 @@ private:
 
 RulesWindow::RulesWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, CanvasGL *ca, Rules *ru,
                          class Core *c)
-    : Gtk::Window(cobject), canvas(ca), rules(ru), core(c), state_store(this, "imp-rules")
+    : Gtk::Window(cobject), canvas(ca), rules(ru), core(c),
+      state_store(this, "imp-rules-" + object_type_lut.lookup_reverse(core->get_object_type()))
 {
     x->get_widget("lb_rules", lb_rules);
     x->get_widget("lb_multi", lb_multi);
@@ -211,6 +213,7 @@ RulesWindow::RulesWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builde
             auto mcr = dynamic_cast<CellRendererColorBox *>(tcr);
             auto co = rules_check_error_level_to_color(row[tree_columns.result]);
             Gdk::RGBA va;
+            va.set_alpha(1);
             va.set_red(co.r);
             va.set_green(co.g);
             va.set_blue(co.b);
@@ -394,6 +397,7 @@ void RulesWindow::rule_selected(RuleID id)
     if (multi) {
         show_editor(nullptr);
         update_rule_instances(id);
+        update_warning();
     }
     else {
         auto rule = rules->get_rule(id);
@@ -479,6 +483,10 @@ RuleEditor *RulesWindow::create_editor(Rule *r)
         e = new RuleEditorClearanceCopperKeepout(r, core);
         break;
 
+    case RuleID::LAYER_PAIR:
+        e = new RuleEditorLayerPair(r, core);
+        break;
+
     default:
         e = new RuleEditor(r, core);
     }
@@ -503,8 +511,8 @@ void RulesWindow::update_rule_instances(RuleID id)
         delete it;
     }
     for (const auto &it : inst) {
-        auto la =
-                Gtk::manage(new RuleLabel(sg_order, it.second->get_brief(get_block()), id, it.first, it.second->order));
+        auto la = Gtk::manage(
+                new RuleLabel(sg_order, it.second->get_brief(get_block()), id, it.first, it.second->get_order()));
         la->set_sensitive(it.second->enabled);
         la->show();
         lb_multi->append(*la);
@@ -526,7 +534,7 @@ void RulesWindow::update_rule_instance_labels()
         auto rule = rules->get_rule(la->id, la->uuid);
         la->set_text(rule->get_brief(get_block()));
         la->set_sensitive(rule->enabled);
-        la->set_order(rule->order);
+        la->set_order(rule->get_order());
     }
     lb_multi->invalidate_sort();
 }
@@ -558,6 +566,11 @@ void RulesWindow::set_enabled(bool enable)
 
 void RulesWindow::update_warning()
 {
+    if (!rule_descriptions.at(rule_current).needs_match_all) {
+        rev_warn->set_reveal_child(false);
+        return;
+    }
+
     auto sorted = rules->get_rules_sorted(rule_current);
     if (sorted.size() == 0) {
         rev_warn->set_reveal_child(true);
