@@ -1,5 +1,6 @@
 #pragma once
 #include "util/pool_goto_provider.hpp"
+#include "util/item_set.hpp"
 
 namespace horizon {
 class PoolEditorInterface : public PoolGotoProvider {
@@ -11,10 +12,13 @@ public:
     {
         needs_save = false;
     }
-    bool get_needs_save()
+    bool get_needs_save() const
     {
         return needs_save;
     }
+    virtual void select(const ItemSet &items)
+    {
+    }
 
 protected:
     bool needs_save = false;
