@@ -421,8 +421,10 @@ PartEditor::PartEditor(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>
         }
     }
     update_parametric_editor();
-    w_parametric_table_combo->signal_changed().connect(sigc::mem_fun(*this, &PartEditor::update_parametric_editor));
-
+    w_parametric_table_combo->signal_changed().connect([this] {
+        needs_save = true;
+        update_parametric_editor();
+    });
     w_parametric_from_base->set_sensitive(part->base);
     w_parametric_from_base->signal_clicked().connect([this] {
         if (part->base) {
@@ -468,11 +470,11 @@ void PartEditor::update_map_buttons()
                 w_tv_pads->get_selection()->count_selected_rows() && w_tv_pins->get_selection()->count_selected_rows();
         w_button_map->set_sensitive(can_map);
         w_button_unmap->set_sensitive(w_tv_pads->get_selection()->count_selected_rows());
-        w_button_automap->set_sensitive(true);
+        w_button_automap->set_sensitive(w_tv_pads->get_selection()->count_selected_rows());
         w_button_copy_from_other->set_sensitive(true);
     }
     w_button_select_pads->set_sensitive(w_tv_pins->get_selection()->count_selected_rows());
-    w_button_select_pin->set_sensitive(w_tv_pads->get_selection()->count_selected_rows());
+    w_button_select_pin->set_sensitive(w_tv_pads->get_selection()->count_selected_rows() == 1);
 }
 
 void PartEditor::update_orderable_MPNs_label()
@@ -661,6 +663,9 @@ void PartEditor::save()
     if (parametric_editor) {
         part->parametric = parametric_editor->get_values();
     }
+    else {
+        part->parametric.clear();
+    }
 
     part->orderable_MPNs.clear();
 
