@@ -6,6 +6,8 @@
 #include <iomanip>
 #include "util/pool_completion.hpp"
 #include "util/gtk_util.hpp"
+#include "widgets/help_button.hpp"
+#include "help_texts.hpp"
 
 namespace horizon {
 
@@ -21,7 +23,6 @@ private:
     Gtk::Entry *name_entry = nullptr;
     Gtk::Entry *names_entry = nullptr;
     Gtk::ComboBoxText *dir_combo = nullptr;
-    Gtk::SpinButton *swap_group_spin_button = nullptr;
 };
 
 PinEditor::PinEditor(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, Pin *p, UnitEditor *pa)
@@ -30,9 +31,12 @@ PinEditor::PinEditor(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &
     x->get_widget("pin_name", name_entry);
     x->get_widget("pin_names", names_entry);
     x->get_widget("pin_direction", dir_combo);
-    x->get_widget("pin_swap_group", swap_group_spin_button);
     entry_add_sanitizer(name_entry);
     entry_add_sanitizer(names_entry);
+    parent->sg_name->add_widget(*name_entry);
+    parent->sg_direction->add_widget(*dir_combo);
+    parent->sg_names->add_widget(*names_entry);
+    widget_remove_scroll_events(*dir_combo);
 
     for (const auto &it : Pin::direction_names) {
         dir_combo->append(std::to_string(static_cast<int>(it.first)), it.second);
@@ -90,19 +94,14 @@ PinEditor::PinEditor(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &
         propagate([this](PinEditor *ed) { ed->dir_combo->set_active_id(dir_combo->get_active_id()); });
         parent->needs_save = true;
     });
-    swap_group_spin_button->set_value(pin->swap_group);
-    swap_group_spin_button->signal_value_changed().connect([this, propagate] {
-        pin->swap_group = swap_group_spin_button->get_value_as_int();
-        propagate([this](PinEditor *ed) { ed->swap_group_spin_button->set_value(pin->swap_group); });
-        parent->needs_save = true;
-    });
 }
 
 PinEditor *PinEditor::create(Pin *p, UnitEditor *pa)
 {
     PinEditor *w;
     Glib::RefPtr<Gtk::Builder> x = Gtk::Builder::create();
-    x->add_from_resource("/org/horizon-eda/horizon/pool-prj-mgr/pool-mgr/editors/unit_editor.ui");
+    static const std::vector<Glib::ustring> widgets = {"pin_editor"};
+    x->add_from_resource("/org/horizon-eda/horizon/pool-prj-mgr/pool-mgr/editors/unit_editor.ui", widgets);
     x->get_widget_derived("pin_editor", w, p, pa);
     w->reference();
     return w;
@@ -133,9 +132,16 @@ UnitEditor::UnitEditor(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>
     x->get_widget("unit_pins_refresh", refresh_button);
     x->get_widget("pin_add", add_button);
     x->get_widget("pin_delete", delete_button);
+    x->get_widget("cross_probing", cross_probing_cb);
     entry_add_sanitizer(name_entry);
     entry_add_sanitizer(manufacturer_entry);
 
+    sg_name = decltype(sg_name)::cast_dynamic(x->get_object("sg_name"));
+    sg_direction = decltype(sg_direction)::cast_dynamic(x->get_object("sg_direction"));
+    sg_names = decltype(sg_names)::cast_dynamic(x->get_object("sg_names"));
+
+    HelpButton::pack_into(x, "box_names", HelpTexts::UNIT_PIN_ALT_NAMES);
+
     name_entry->set_text(unit->name);
     name_entry->signal_changed().connect([this] {
         unit->name = name_entry->get_text();
@@ -263,6 +269,7 @@ void UnitEditor::handle_add()
     int index = 0;
     {
         auto children = pins_listbox->get_children();
+        index = children.size();
         int i = 0;
         for (auto &ch : children) {
             auto row = dynamic_cast<Gtk::ListBoxRow *>(ch);
@@ -323,6 +330,21 @@ void UnitEditor::handle_activate(PinEditor *ed)
     }
 }
 
+void UnitEditor::select(const ItemSet &items)
+{
+    if (!cross_probing_cb->get_active())
+        return;
+
+    pins_listbox->unselect_all();
+    for (auto &ch : pins_listbox->get_children()) {
+        auto row = dynamic_cast<Gtk::ListBoxRow *>(ch);
+        auto ed_row = dynamic_cast<PinEditor *>(row->get_child());
+        if (items.count({ObjectType::SYMBOL_PIN, ed_row->pin->uuid})) {
+            pins_listbox->select_row(*row);
+        }
+    }
+}
+
 UnitEditor *UnitEditor::create(Unit *u, class Pool *p)
 {
     UnitEditor *w;
