@@ -22,6 +22,7 @@
 #include "widgets/part_preview.hpp"
 #include <thread>
 #include "nlohmann/json.hpp"
+#include "widgets/where_used_box.hpp"
 
 #ifdef G_OS_WIN32
 #undef ERROR
@@ -38,6 +39,19 @@ void PoolNotebook::pool_updated(bool success)
         dia.run();
     }
     appwin->set_pool_updating(false, success);
+    reload();
+    if (success && pool_update_done_cb) {
+        pool_update_done_cb();
+        pool_update_done_cb = nullptr;
+    }
+    if (success)
+        Glib::RefPtr<PoolProjectManagerApplication>::cast_dynamic(appwin->get_application())
+                ->signal_pool_updated()
+                .emit(pool_uuid);
+}
+
+void PoolNotebook::reload()
+{
     pool.clear();
     for (auto &br : browsers) {
         if (widget_is_visible(br.second))
@@ -55,24 +69,16 @@ void PoolNotebook::pool_updated(bool success)
     for (auto &it : procs) {
         it.second->reload();
     }
-    if (success && pool_update_done_cb) {
-        pool_update_done_cb();
-        pool_update_done_cb = nullptr;
-    }
     if (settings_box)
         settings_box->pool_updated();
     if (part_wizard)
         part_wizard->reload();
-    if (git_box && success && git_box->refreshed_once) {
+    if (git_box && git_box->refreshed_once) {
         if (widget_is_visible(git_box))
             git_box->refresh();
         else
             git_box->refreshed_once = false;
     }
-    if (success)
-        Glib::RefPtr<PoolProjectManagerApplication>::cast_dynamic(appwin->get_application())
-                ->signal_pool_updated()
-                .emit(pool_uuid);
 }
 
 bool PoolNotebook::widget_is_visible(Gtk::Widget *widget)
@@ -152,13 +158,16 @@ PoolNotebook::PoolNotebook(const std::string &bp, class PoolProjectManagerAppWin
 {
     appwin->pool = &pool;
     appwin->pool_parametric = &pool_parametric;
-    appwin->signal_process_exited().connect(sigc::track_obj(
-            [this](std::string filename, int status, bool modified) {
+
+    appwin->signal_process_saved().connect(sigc::track_obj(
+            [this](std::string filename) {
                 auto in_pool = Gio::File::create_for_path(base_path)
                                        ->get_relative_path(Gio::File::create_for_path(filename))
                                        .size();
-                if (modified && in_pool)
+                if (in_pool)
                     pool_update(nullptr, {filename});
+                else
+                    reload();
             },
             *this));
 
@@ -309,18 +318,62 @@ void PoolNotebook::add_context_menu(PoolBrowser *br)
 void PoolNotebook::handle_delete(ObjectType ty, const UUID &uu)
 {
     auto top = dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW));
-    Gtk::MessageDialog md(*top, "Permanently delete " + object_descriptions.at(ty).name + "?", false /* use_markup */,
-                          Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE);
-    md.add_button("Cancel", Gtk::RESPONSE_CANCEL);
-    md.add_button("Delete", Gtk::RESPONSE_OK)->get_style_context()->add_class("destructive-action");
-    if (md.run() == Gtk::RESPONSE_OK) {
-        auto filename = pool.get_filename(ty, uu);
-        if (ty == ObjectType::PACKAGE) {
-            auto dir = Glib::path_get_dirname(filename);
-            rmdir_recursive(dir);
+
+    std::string item_name;
+    {
+        SQLite::Query q(pool.db, "SELECT name FROM all_items_view WHERE type = ? and uuid = ?");
+        q.bind(1, object_type_lut.lookup_reverse(ty));
+        q.bind(2, uu);
+        if (q.step()) {
+            item_name = q.get<std::string>(0);
         }
-        else {
-            Gio::File::create_for_path(filename)->remove();
+    }
+
+    Gtk::MessageDialog md(*top, "Permanently delete " + object_descriptions.at(ty).name + " \"" + item_name + "\" ?",
+                          false /* use_markup */, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE);
+    auto box = Gtk::manage(new WhereUsedBox(pool));
+    const auto cnt = box->load(ty, uu);
+    ItemSet items_del = {{ty, uu}};
+    if (cnt) {
+        box->property_margin() = 10;
+        md.get_content_area()->pack_start(*box, true, true, 0);
+        md.get_content_area()->show_all();
+        md.get_content_area()->set_spacing(0);
+    }
+    else {
+        delete box;
+    }
+
+
+    static constexpr int RESP_DEL_THIS = 1;
+    static constexpr int RESP_DEL_ALL = 2;
+
+    md.add_button("Cancel", Gtk::RESPONSE_CANCEL);
+    if (cnt == 0) {
+        md.add_button("Delete", RESP_DEL_THIS)->get_style_context()->add_class("destructive-action");
+    }
+    else {
+        md.add_button("Delete and break dependents", RESP_DEL_THIS)
+                ->get_style_context()
+                ->add_class("destructive-action");
+        md.add_button("Delete dependents as well", RESP_DEL_ALL)->get_style_context()->add_class("destructive-action");
+    }
+
+    const auto resp = md.run();
+    if (any_of(resp, {RESP_DEL_THIS, RESP_DEL_ALL})) {
+        if (resp == RESP_DEL_ALL) {
+            auto deps = box->get_items();
+            items_del.insert(deps.begin(), deps.end());
+        }
+        for (const auto &[it_ty, it_uu] : items_del) {
+            auto filename = pool.get_filename(it_ty, it_uu);
+            if (it_ty == ObjectType::PACKAGE) {
+                auto dir = Glib::path_get_dirname(filename);
+                rmdir_recursive(dir);
+            }
+            else {
+                Gio::File::create_for_path(filename)->remove();
+            }
         }
         pool_update();
     }
@@ -371,7 +424,7 @@ void PoolNotebook::show_duplicate_window(ObjectType ty, const UUID &uu)
 
 bool PoolNotebook::get_close_prohibited() const
 {
-    return part_wizard || pool_updating || duplicate_window;
+    return part_wizard || pool_updating || duplicate_window || kicad_symbol_import_wizard;
 }
 
 void PoolNotebook::prepare_close()
