@@ -2,7 +2,6 @@
 #include "editors/editor_window.hpp"
 #include "duplicate/duplicate_unit.hpp"
 #include "duplicate/duplicate_part.hpp"
-#include "part_wizard/part_wizard.hpp"
 #include "util/util.hpp"
 #include "nlohmann/json.hpp"
 #include "pool-prj-mgr/pool-prj-mgr-app_win.hpp"
@@ -139,26 +138,6 @@ void PoolNotebook::handle_duplicate_package(const UUID &uu)
     }
 }
 
-void PoolNotebook::handle_part_wizard(const UUID &uu)
-{
-    if (!part_wizard) {
-        auto pkg = pool.get_package(uu);
-        part_wizard = PartWizard::create(pkg, base_path, &pool, appwin);
-        part_wizard->present();
-        part_wizard->signal_hide().connect([this] {
-            auto files_saved = part_wizard->get_files_saved();
-            if (files_saved.size()) {
-                pool_update(nullptr, files_saved);
-            }
-            delete part_wizard;
-            part_wizard = nullptr;
-        });
-    }
-    else {
-        part_wizard->present();
-    }
-}
-
 void PoolNotebook::construct_packages()
 {
     auto br = Gtk::manage(new PoolBrowserPackage(&pool));
@@ -193,9 +172,6 @@ void PoolNotebook::construct_packages()
             }
         });
     }
-    add_action_button("Part Wizard...", bbox, br, sigc::mem_fun(*this, &PoolNotebook::handle_part_wizard))
-            ->get_style_context()
-            ->add_class("suggested-action");
 
     auto stack = Gtk::manage(new Gtk::Stack);
     add_preview_stack_switcher(bbox, stack);
