@@ -1,7 +1,6 @@
 #include "pool_remote_box.hpp"
 #include <git2.h>
 #include <git2/cred_helpers.h>
-#include <git2/sys/repository.h>
 #include "util/autofree_ptr.hpp"
 #include "util/gtk_util.hpp"
 #include "pool_notebook.hpp"
@@ -348,9 +347,9 @@ bool PoolRemoteBox::exists_in_pool(Pool &pool, ObjectType ty, const UUID &uu)
     }
 }
 
-std::set<std::pair<ObjectType, UUID>> PoolRemoteBox::get_referenced(ObjectType ty, const UUID &uu)
+ItemSet PoolRemoteBox::get_referenced(ObjectType ty, const UUID &uu)
 {
-    std::set<std::pair<ObjectType, UUID>> items;
+    ItemSet items;
     items.emplace(ty, uu);
     Pool pool_remote(notebook->remote_repo);
     bool added = true;
