@@ -9,7 +9,7 @@
 #include "nlohmann/json.hpp"
 #include "close_utils.hpp"
 #include "pool/pool_manager.hpp"
-#include "preferences_window.hpp"
+#include "preferences/preferences_window.hpp"
 #include "preferences/preferences_provider.hpp"
 #include "widgets/about_dialog.hpp"
 
@@ -89,6 +89,7 @@ void PoolProjectManagerApplication::load_from_config(const std::string &config_f
             part_favorites.push_back(it.value().get<std::string>());
         }
     }
+    pool_doc_info_bar_dismissed = j.value("pool_doc_info_bar_dismissed", false);
     recent_from_json(recent_items, j);
 }
 
@@ -148,7 +149,7 @@ void PoolProjectManagerApplication::on_startup()
 PreferencesWindow *PoolProjectManagerApplication::show_preferences_window(guint32 timestamp)
 {
     if (!preferences_window) {
-        preferences_window = new PreferencesWindow(&preferences);
+        preferences_window = new PreferencesWindow(preferences);
 
         preferences_window->signal_hide().connect([this] {
             delete preferences_window;
@@ -175,6 +176,7 @@ void PoolProjectManagerApplication::on_shutdown()
     for (const auto &it : recent_items) {
         j["recent"][it.first] = it.second.to_unix();
     }
+    j["pool_doc_info_bar_dismissed"] = pool_doc_info_bar_dismissed;
     save_json_to_file(config_filename, j);
 }
 
