@@ -1,6 +1,6 @@
 #include "pool-prj-mgr-app_win.hpp"
 #include "pool-prj-mgr-app.hpp"
-#include "preferences_window.hpp"
+#include "preferences/preferences_window.hpp"
 #include <iostream>
 #include <thread>
 #include "util/util.hpp"
@@ -67,6 +67,8 @@ PoolProjectManagerAppWindow::PoolProjectManagerAppWindow(BaseObjectType *cobject
     builder->get_widget("menu_new_project", menu_new_project);
     builder->get_widget("hamburger_menu_button", hamburger_menu_button);
     builder->get_widget("info_bar_pool_not_added", info_bar_pool_not_added);
+    builder->get_widget("info_bar_pool_doc", info_bar_pool_doc);
+
     set_view_mode(ViewMode::OPEN);
 
     {
@@ -119,6 +121,14 @@ PoolProjectManagerAppWindow::PoolProjectManagerAppWindow(BaseObjectType *cobject
     });
     info_bar_hide(info_bar_pool_not_added);
 
+    info_bar_pool_doc->signal_response().connect([this](int resp) {
+        if (resp == Gtk::RESPONSE_OK) {
+            info_bar_hide(info_bar_pool_doc);
+            app->pool_doc_info_bar_dismissed = true;
+        }
+    });
+    info_bar_hide(info_bar_pool_doc);
+
     show_output_button->signal_clicked().connect([this] {
         output_window->present();
         info_bar_hide(info_bar);
@@ -380,6 +390,19 @@ json PoolProjectManagerAppWindow::handle_req(const json &j)
         pids_need_save[pid] = needs_save;
         if (project && !needs_save)
             view_project.update_meta();
+        if (!needs_save) {
+            s_signal_process_saved.emit(j.at("filename"));
+            if (auto proc = find_top_schematic_process()) {
+                if (proc->proc->get_pid() == pid) { // schematic got saved
+                    if (auto proc_board = find_board_process()) {
+                        auto board_pid = proc_board->proc->get_pid();
+                        json tx;
+                        tx["op"] = "reload-netlist-hint";
+                        app->send_json(board_pid, tx);
+                    }
+                }
+            }
+        }
     }
     else if (op == "ready") {
         int pid = j.at("pid");
@@ -408,7 +431,8 @@ json PoolProjectManagerAppWindow::handle_req(const json &j)
             default:
                 return nullptr;
             }
-            spawn(ptype, {path}, {}, this_pool_uuid && (item_pool_uuid != this_pool_uuid));
+            spawn(ptype, {path}, {},
+                  (item_pool_uuid != Pool::tmp_pool_uuid) && this_pool_uuid && (item_pool_uuid != this_pool_uuid));
         }
         catch (const std::exception &e) {
             Gtk::MessageDialog md(*this, "Can't open editor", false /* use_markup */, Gtk::MESSAGE_ERROR,
@@ -453,6 +477,20 @@ json PoolProjectManagerAppWindow::handle_req(const json &j)
             app->send_json(pid, tx);
         }
     }
+    else if (op == "symbol-select") {
+        UUID unit_uuid = j.at("unit").get<std::string>();
+        for (auto &it_proc : processes) {
+            if (it_proc.second.win && it_proc.second.win->get_object_type() == ObjectType::UNIT
+                && it_proc.second.win->get_uuid() == unit_uuid) {
+                ItemSet sel;
+                for (const auto &pin : j.at("pins")) {
+                    sel.emplace(ObjectType::SYMBOL_PIN, pin.get<std::string>());
+                }
+                it_proc.second.win->select(sel);
+                break;
+            }
+        }
+    }
     return nullptr;
 }
 
@@ -818,6 +856,8 @@ void PoolProjectManagerAppWindow::set_view_mode(ViewMode mode)
         button_update->show();
         hamburger_menu_button->show();
         header->set_title("Pool manager");
+        if (!app->pool_doc_info_bar_dismissed)
+            info_bar_show(info_bar_pool_doc);
         break;
 
     case ViewMode::DOWNLOAD:
@@ -1057,7 +1097,9 @@ void PoolProjectManagerAppWindow::open_file_view(const Glib::RefPtr<Gio::File> &
         view_project.label_project_directory->set_text(Glib::path_get_dirname(project_filename));
 
         view_project.pool_info_bar->hide();
-        view_project.label_pool_name->set_text(prj_pool->name);
+        view_project.label_pool_name->set_markup("<a href=\"" + prj_pool->base_path + "\""
+        		"title=\"Open in pool manager\">"
+                                                 + Glib::Markup::escape_text(prj_pool->name) + "</a>");
         view_project.label_pool_path->set_text(prj_pool->base_path);
         view_project.reset_pool_cache_status();
 
@@ -1104,7 +1146,8 @@ void PoolProjectManagerAppWindow::open_file_view(const Glib::RefPtr<Gio::File> &
 void PoolProjectManagerAppWindow::open_pool(const std::string &pool_json, ObjectType type, const UUID &uu)
 {
     open_file_view(Gio::File::create_for_path(pool_json));
-    pool_notebook_go_to(type, uu);
+    if (type != ObjectType::INVALID)
+        pool_notebook_go_to(type, uu);
 }
 
 
@@ -1193,6 +1236,9 @@ PoolProjectManagerProcess *PoolProjectManagerAppWindow::spawn(PoolProjectManager
                 output_window->handle_output("[" + real_filename + "] " + line, err);
             }
         });
+        if (proc.win)
+            proc.win->signal_saved().connect([this](auto fn) { s_signal_process_saved.emit(fn); });
+
         return &proc;
     }
     else { // present imp
