@@ -16,7 +16,6 @@ private:
 
 public:
     Entity(const UUID &uu);
-    Entity(const UUID &uu, const YAML::Node &n, Pool &pool);
 
     static Entity new_from_file(const std::string &filename, Pool &pool);
     UUID uuid;
@@ -25,7 +24,6 @@ public:
     std::string prefix;
     std::set<std::string> tags;
     std::map<UUID, Gate> gates;
-    void serialize_yaml(YAML::Emitter &em) const;
     json serialize() const;
     void update_refs(Pool &pool);
     UUID get_uuid() const;
