@@ -3,6 +3,7 @@
 #include "util/util.hpp"
 #include "nlohmann/json.hpp"
 #include "board/board_layers.hpp"
+#include "util/picture_load.hpp"
 
 namespace horizon {
 
@@ -61,35 +62,35 @@ Package::Package(const UUID &uu, const json &j, Pool &pool)
       parameter_program(this, j.value("parameter_program", ""))
 {
     {
-        const json &o = j["junctions"];
+        const json &o = j.at("junctions");
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID(it.key());
             junctions.emplace(std::make_pair(u, Junction(u, it.value())));
         }
     }
     {
-        const json &o = j["lines"];
+        const json &o = j.at("lines");
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID(it.key());
             lines.emplace(std::make_pair(u, Line(u, it.value(), *this)));
         }
     }
     {
-        const json &o = j["arcs"];
+        const json &o = j.at("arcs");
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID(it.key());
             arcs.emplace(std::make_pair(u, Arc(u, it.value(), *this)));
         }
     }
     {
-        const json &o = j["texts"];
+        const json &o = j.at("texts");
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID(it.key());
             texts.emplace(std::make_pair(u, Text(u, it.value())));
         }
     }
     {
-        const json &o = j["pads"];
+        const json &o = j.at("pads");
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID(it.key());
             pads.emplace(std::make_pair(u, Pad(u, it.value(), pool)));
@@ -119,6 +120,13 @@ Package::Package(const UUID &uu, const json &j, Pool &pool)
                                std::forward_as_tuple(u, it.value()));
         }
     }
+    if (j.count("pictures")) {
+        const json &o = j["pictures"];
+        for (auto it = o.cbegin(); it != o.cend(); ++it) {
+            auto u = UUID(it.key());
+            pictures.emplace(std::piecewise_construct, std::forward_as_tuple(u), std::forward_as_tuple(u, it.value()));
+        }
+    }
     for (auto &it : keepouts) {
         it.second.polygon.update(polygons);
         it.second.polygon->usage = &it.second;
@@ -175,7 +183,7 @@ Polygon *Package::get_polygon(const UUID &uu)
 Package::Package(const Package &pkg)
     : uuid(pkg.uuid), name(pkg.name), manufacturer(pkg.manufacturer), tags(pkg.tags), junctions(pkg.junctions),
       lines(pkg.lines), arcs(pkg.arcs), texts(pkg.texts), pads(pkg.pads), polygons(pkg.polygons),
-      keepouts(pkg.keepouts), dimensions(pkg.dimensions), parameter_set(pkg.parameter_set),
+      keepouts(pkg.keepouts), dimensions(pkg.dimensions), pictures(pkg.pictures), parameter_set(pkg.parameter_set),
       parameter_program(pkg.parameter_program), models(pkg.models), default_model(pkg.default_model),
       alternate_for(pkg.alternate_for), warnings(pkg.warnings)
 {
@@ -196,6 +204,7 @@ void Package::operator=(Package const &pkg)
     polygons = pkg.polygons;
     keepouts = pkg.keepouts;
     dimensions = pkg.dimensions;
+    pictures = pkg.pictures;
     parameter_set = pkg.parameter_set;
     parameter_program = pkg.parameter_program;
     models = pkg.models;
@@ -275,7 +284,6 @@ void Package::expand()
 {
     map_erase_if(keepouts, [this](auto &it) { return polygons.count(it.second.polygon.uuid) == 0; });
     for (auto &it : junctions) {
-        it.second.temp = false;
         it.second.layer = 10000;
         it.second.has_via = false;
         it.second.needs_via = false;
@@ -420,6 +428,12 @@ json Package::serialize() const
     for (const auto &it : dimensions) {
         j["dimensions"][(std::string)it.first] = it.second.serialize();
     }
+    if (pictures.size()) {
+        j["pictures"] = json::object();
+        for (const auto &it : pictures) {
+            j["pictures"][(std::string)it.first] = it.second.serialize();
+        }
+    }
     return j;
 }
 
@@ -459,4 +473,15 @@ int Package::get_max_pad_name() const
     }
     return -1;
 }
+
+void Package::save_pictures(const std::string &dir) const
+{
+    pictures_save({&pictures}, dir, "pkg");
+}
+
+void Package::load_pictures(const std::string &dir)
+{
+    pictures_load({&pictures}, dir, "pkg");
+}
+
 } // namespace horizon
