@@ -10,6 +10,7 @@ using json = nlohmann::json;
 
 enum class ActionID;
 enum class ToolID;
+enum class InToolActionID;
 
 class CanvasPreferences {
 public:
@@ -24,6 +25,7 @@ class SchematicPreferences {
 public:
     bool show_all_junctions = false;
     bool drag_start_net_line = true;
+    bool bend_non_ortho = true;
 
     void load_from_json(const json &j);
     json serialize() const;
@@ -34,6 +36,8 @@ public:
     bool drag_start_track = true;
     bool highlight_on_top = true;
     bool show_text_in_tracks = true;
+    bool show_text_in_vias = true;
+    bool move_using_router = true;
 
     void load_from_json(const json &j);
     json serialize() const;
@@ -41,7 +45,16 @@ public:
 
 class KeySequencesPreferences {
 public:
-    std::map<std::pair<ActionID, ToolID>, std::map<ActionCatalogItem::Availability, std::vector<KeySequence>>> keys;
+    std::map<ActionToolID, std::map<ActionCatalogItem::Availability, std::vector<KeySequence>>> keys;
+
+    void load_from_json(const json &j);
+    void append_from_json(const json &j);
+    json serialize() const;
+};
+
+class InToolKeySequencesPreferences {
+public:
+    std::map<InToolActionID, std::vector<KeySequence>> keys;
 
     void load_from_json(const json &j);
     void append_from_json(const json &j);
@@ -72,6 +85,15 @@ public:
     json serialize() const;
 };
 
+class ActionBarPreferences {
+public:
+    bool enable = true;
+    bool remember = true;
+
+    void load_from_json(const json &j);
+    json serialize() const;
+};
+
 class Preferences {
 public:
     Preferences();
@@ -91,6 +113,9 @@ public:
     ZoomPreferences zoom;
     bool capture_output = false;
     PartInfoPreferences partinfo;
+    ActionBarPreferences action_bar;
+    InToolKeySequencesPreferences in_tool_key_sequences;
+
 
     typedef sigc::signal<void> type_signal_changed;
     type_signal_changed signal_changed()
