@@ -5,11 +5,13 @@
 #include "canvas/canvas_gl.hpp"
 #include "clipper/clipper.hpp"
 #include "geometry/shape_simple.h"
+#include "geometry/shape_rect.h"
 #include "router/pns_debug_decorator.h"
 #include "router/pns_solid.h"
 #include "router/pns_topology.h"
 #include "router/pns_via.h"
 #include "util/util.hpp"
+#include "logger/logger.hpp"
 
 namespace PNS {
 
@@ -69,23 +71,24 @@ int PNS_HORIZON_IFACE::layer_from_router(int l)
 
 class PNS_HORIZON_RULE_RESOLVER : public PNS::RULE_RESOLVER {
 public:
-    PNS_HORIZON_RULE_RESOLVER(const horizon::Board *aBoard, horizon::BoardRules *aRules, PNS::ROUTER *aRouter);
+    PNS_HORIZON_RULE_RESOLVER(const horizon::Board *aBoard, const horizon::BoardRules *aRules, PNS::ROUTER *aRouter);
     virtual ~PNS_HORIZON_RULE_RESOLVER();
 
-    virtual int Clearance(const PNS::ITEM *aA, const PNS::ITEM *aB) const override;
-    virtual int Clearance(int aNetCode) const override;
-    virtual int DpCoupledNet(int aNet) override;
-    virtual int DpNetPolarity(int aNet) override;
-    virtual bool DpNetPair(PNS::ITEM *aItem, int &aNetP, int &aNetN) override;
+    int Clearance(const PNS::ITEM *aA, const PNS::ITEM *aB) const override;
+    int Clearance(int aNetCode) const override;
+    int DpCoupledNet(int aNet) override;
+    int DpNetPolarity(int aNet) override;
+    bool DpNetPair(PNS::ITEM *aItem, int &aNetP, int &aNetN) override;
+    std::string NetName(int aNet);
 
 private:
     PNS::ROUTER *m_router;
-    horizon::BoardRules *m_rules;
+    const horizon::BoardRules *m_rules;
     PNS_HORIZON_IFACE *m_iface = nullptr;
-    std::vector<horizon::RuleClearanceCopperKeepout *> m_rules_keepout;
+    std::vector<const horizon::RuleClearanceCopperKeepout *> m_rules_keepout;
 };
 
-PNS_HORIZON_RULE_RESOLVER::PNS_HORIZON_RULE_RESOLVER(const horizon::Board *aBoard, horizon::BoardRules *rules,
+PNS_HORIZON_RULE_RESOLVER::PNS_HORIZON_RULE_RESOLVER(const horizon::Board *aBoard, const horizon::BoardRules *rules,
                                                      PNS::ROUTER *aRouter)
     : m_router(aRouter), m_rules(rules)
 {
@@ -94,8 +97,8 @@ PNS_HORIZON_RULE_RESOLVER::PNS_HORIZON_RULE_RESOLVER(const horizon::Board *aBoar
     PNS::TOPOLOGY topo(world);
 
     m_iface = static_cast<PNS_HORIZON_IFACE *>(m_router->GetInterface());
-    m_rules_keepout = horizon::dynamic_cast_vector<horizon::RuleClearanceCopperKeepout *>(
-            m_rules->get_rules_sorted(horizon::RuleID::CLEARANCE_COPPER_KEEPOUT));
+    m_rules_keepout =
+            m_rules->get_rules_sorted<horizon::RuleClearanceCopperKeepout>(horizon::RuleID::CLEARANCE_COPPER_KEEPOUT);
 }
 
 PNS_HORIZON_RULE_RESOLVER::~PNS_HORIZON_RULE_RESOLVER()
@@ -267,6 +270,15 @@ bool PNS_HORIZON_RULE_RESOLVER::DpNetPair(PNS::ITEM *aItem, int &aNetP, int &aNe
     return false;
 }
 
+std::string PNS_HORIZON_RULE_RESOLVER::NetName(int aNet)
+{
+    auto net = m_iface->get_net_for_code(aNet);
+    if (net)
+        return net->name;
+    else
+        return "";
+}
+
 class PNS_HORIZON_DEBUG_DECORATOR : public PNS::DEBUG_DECORATOR {
 public:
     PNS_HORIZON_DEBUG_DECORATOR(horizon::CanvasGL *canvas) : PNS::DEBUG_DECORATOR(), m_canvas(canvas)
@@ -275,7 +287,13 @@ public:
 
     ~PNS_HORIZON_DEBUG_DECORATOR()
     {
-        Clear();
+        try {
+            Clear();
+        }
+        catch (const std::exception &e) {
+            horizon::Logger::log_critical("exception thrown in ~PNS_HORIZON_DEBUG_DECORATOR",
+                                          horizon::Logger::Domain::TOOL, e.what());
+        }
     }
 
     void AddPoint(VECTOR2I aP, int aColor) override
@@ -372,7 +390,7 @@ void PNS_HORIZON_IFACE::SetCanvas(horizon::CanvasGL *ca)
     canvas = ca;
 }
 
-void PNS_HORIZON_IFACE::SetRules(horizon::BoardRules *ru)
+void PNS_HORIZON_IFACE::SetRules(const horizon::BoardRules *ru)
 {
     rules = ru;
 }
@@ -388,10 +406,10 @@ int PNS_HORIZON_IFACE::get_net_code(const horizon::UUID &uu)
         return net_code_map.at(uu);
     }
     else {
-        net_code_max++;
-        net_code_map.emplace(uu, net_code_max);
-        net_code_map_r.emplace(net_code_max, uu);
-        return net_code_max;
+        net_code_map_r.emplace_back(&board->block->nets.at(uu));
+        auto nc = net_code_map_r.size() - 1;
+        net_code_map.emplace(uu, nc);
+        return nc;
     }
 }
 
@@ -399,9 +417,11 @@ horizon::Net *PNS_HORIZON_IFACE::get_net_for_code(int code)
 {
     if (code == PNS::ITEM::UnusedNet)
         return nullptr;
-    if (!net_code_map_r.count(code))
+    if (code < 0)
         return nullptr;
-    return &board->block->nets.at(net_code_map_r.at(code));
+    if (code >= (int)net_code_map_r.size())
+        return nullptr;
+    return net_code_map_r.at(code);
 }
 
 const PNS_HORIZON_PARENT_ITEM *PNS_HORIZON_IFACE::get_parent(const horizon::Track *track)
@@ -545,6 +565,21 @@ std::unique_ptr<PNS::SOLID> PNS_HORIZON_IFACE::syncHole(const horizon::BoardHole
     return solid;
 }
 
+bool shape_is_at_origin(const horizon::Shape &sh)
+{
+    return sh.placement.shift == horizon::Coordi();
+}
+
+bool angle_is_rect(int angle)
+{
+    return (angle % 16384) == 0; // multiple of 90°
+}
+
+bool angle_needs_swap(int angle)
+{
+    return angle == 16384 || angle == 49152; // 90° or 270°
+}
+
 std::unique_ptr<PNS::SOLID> PNS_HORIZON_IFACE::syncPadstack(const horizon::Padstack *padstack,
                                                             const horizon::Placement &tr)
 {
@@ -554,6 +589,57 @@ std::unique_ptr<PNS::SOLID> PNS_HORIZON_IFACE::syncPadstack(const horizon::Padst
     ClipperLib::Clipper clipper;
 
     if (padstack->type != horizon::Padstack::Type::MECHANICAL) { // normal pad
+
+        auto n_shapes_cu = std::count_if(padstack->shapes.begin(), padstack->shapes.end(), [](const auto &it) {
+            return horizon::BoardLayers::is_copper(it.second.layer);
+        });
+
+        auto n_polys_cu = std::count_if(padstack->polygons.begin(), padstack->polygons.end(), [](const auto &it) {
+            return horizon::BoardLayers::is_copper(it.second.layer);
+        });
+
+        if (n_shapes_cu == 1 && n_polys_cu == 0) { // single SMD pad
+            const auto &shape = std::find_if(padstack->shapes.begin(), padstack->shapes.end(), [](const auto &it) {
+                                    return horizon::BoardLayers::is_copper(it.second.layer);
+                                })->second;
+            if (shape_is_at_origin(shape)) {
+                const int angle = tr.get_angle() + shape.placement.get_angle();
+                if (shape.form == horizon::Shape::Form::CIRCLE) {
+                    std::unique_ptr<PNS::SOLID> solid(new PNS::SOLID);
+
+                    solid->SetLayers(LAYER_RANGE(layer_to_router(shape.layer)));
+
+                    solid->SetOffset(VECTOR2I(0, 0));
+                    solid->SetPos(VECTOR2I(tr.shift.x, tr.shift.y));
+
+                    auto *sshape = new SHAPE_CIRCLE(VECTOR2I(tr.shift.x, tr.shift.y), shape.params.at(0) / 2);
+
+                    solid->SetShape(sshape);
+
+                    return solid;
+                }
+                else if (shape.form == horizon::Shape::Form::RECTANGLE && angle_is_rect(angle)) {
+                    std::unique_ptr<PNS::SOLID> solid(new PNS::SOLID);
+
+                    solid->SetLayers(LAYER_RANGE(layer_to_router(shape.layer)));
+
+                    solid->SetOffset(VECTOR2I(0, 0));
+                    const auto c = VECTOR2I(tr.shift.x, tr.shift.y);
+                    solid->SetPos(c);
+                    auto sz = VECTOR2I(shape.params.at(0), shape.params.at(1));
+                    if (angle_needs_swap(angle))
+                        std::swap(sz.x, sz.y);
+
+                    auto *sshape = new SHAPE_RECT(c - sz / 2, sz.x, sz.y);
+
+                    solid->SetShape(sshape);
+
+                    return solid;
+                }
+            }
+        }
+
+
         auto add_polygon = [&clipper, &tr](const auto &poly) {
             ClipperLib::Path path;
             if (poly.vertices.size() == 0) {
@@ -719,7 +805,7 @@ void PNS_HORIZON_IFACE::EraseView()
         m_debugDecorator->Clear();
 }
 
-void PNS_HORIZON_IFACE::DisplayItem(const PNS::ITEM *aItem, int aColor, int aClearance)
+void PNS_HORIZON_IFACE::DisplayItem(const PNS::ITEM *aItem, int aColor, int aClearance, bool aEdit)
 {
     wxLogTrace("PNS", "DisplayItem %p %s", aItem, aItem->KindStr().c_str());
     if (aItem->Kind() == PNS::ITEM::LINE_T) {
@@ -732,7 +818,7 @@ void PNS_HORIZON_IFACE::DisplayItem(const PNS::ITEM *aItem, int aColor, int aCle
         }
         int la = line_item->Layer();
         m_preview_items.insert(
-                canvas->add_line(pts, line_item->Width(), horizon::ColorP::FROM_LAYER, layer_from_router(la)));
+                canvas->add_line(pts, line_item->Width(), horizon::ColorP::LAYER_HIGHLIGHT, layer_from_router(la)));
     }
     else if (aItem->Kind() == PNS::ITEM::SEGMENT_T) {
         auto seg_item = dynamic_cast<const PNS::SEGMENT *>(aItem);
@@ -742,7 +828,7 @@ void PNS_HORIZON_IFACE::DisplayItem(const PNS::ITEM *aItem, int aColor, int aCle
         pts.emplace_back(seg.B.x, seg.B.y);
         int la = seg_item->Layer();
         m_preview_items.insert(
-                canvas->add_line(pts, seg_item->Width(), horizon::ColorP::FROM_LAYER, layer_from_router(la)));
+                canvas->add_line(pts, seg_item->Width(), horizon::ColorP::LAYER_HIGHLIGHT, layer_from_router(la)));
     }
     else if (aItem->Kind() == PNS::ITEM::VIA_T) {
         auto via_item = dynamic_cast<const PNS::VIA *>(aItem);
@@ -753,10 +839,10 @@ void PNS_HORIZON_IFACE::DisplayItem(const PNS::ITEM *aItem, int aColor, int aCle
         pts.emplace_back(pos.x + 10, pos.y);
         int la = via_item->Layers().Start();
         m_preview_items.insert(
-                canvas->add_line(pts, via_item->Diameter(), horizon::ColorP::FROM_LAYER, layer_from_router(la)));
+                canvas->add_line(pts, via_item->Diameter(), horizon::ColorP::LAYER_HIGHLIGHT, layer_from_router(la)));
         la = via_item->Layers().End();
         m_preview_items.insert(
-                canvas->add_line(pts, via_item->Diameter(), horizon::ColorP::FROM_LAYER, layer_from_router(la)));
+                canvas->add_line(pts, via_item->Diameter(), horizon::ColorP::LAYER_HIGHLIGHT, layer_from_router(la)));
     }
     else {
         assert(false);
@@ -940,4 +1026,18 @@ void PNS_HORIZON_IFACE::SetRouter(PNS::ROUTER *aRouter)
 {
     m_router = aRouter;
 }
+
+bool PNS_HORIZON_IFACE::IsAnyLayerVisible(const LAYER_RANGE &aLayer)
+{
+    throw std::runtime_error("IsAnyLayerVisible not implemented");
+    return true;
+}
+
+bool PNS_HORIZON_IFACE::IsItemVisible(const PNS::ITEM *aItem)
+{
+    throw std::runtime_error("IsItemVisible not implemented");
+    return true;
+}
+
+
 } // namespace PNS
