@@ -21,20 +21,26 @@ enum class RuleID {
     DIFFPAIR,
     PACKAGE_CHECKS,
     PREFLIGHT_CHECKS,
-    CLEARANCE_COPPER_KEEPOUT
+    CLEARANCE_COPPER_KEEPOUT,
+    LAYER_PAIR
 };
 
 extern const LutEnumStr<RuleID> rule_id_lut;
 
 class Rule {
+    friend class Rules;
+
 public:
     Rule(const UUID &uu);
     Rule(const json &j);
     Rule(const UUID &uu, const json &j);
     UUID uuid;
-    int order = 0;
     RuleID id = RuleID::NONE;
     bool enabled = true;
+    int get_order() const
+    {
+        return order;
+    }
 
     virtual json serialize() const;
     virtual std::string get_brief(const class Block *block = nullptr) const = 0;
@@ -47,5 +53,8 @@ public:
 
 protected:
     Rule();
+
+private:
+    int order = -1;
 };
 } // namespace horizon
