@@ -50,10 +50,41 @@ public:
     virtual void load_from_json(const json &j) = 0;
     virtual json serialize() const = 0;
     virtual std::set<RuleID> get_rule_ids() const = 0;
-    virtual Rule *get_rule(RuleID id) = 0;
-    virtual Rule *get_rule(RuleID id, const UUID &uu) = 0;
-    virtual std::map<UUID, Rule *> get_rules(RuleID id) = 0;
-    std::vector<Rule *> get_rules_sorted(RuleID id);
+
+    virtual const Rule *get_rule(RuleID id) const = 0;
+    Rule *get_rule(RuleID id);
+
+    virtual const Rule *get_rule(RuleID id, const UUID &uu) const = 0;
+    Rule *get_rule(RuleID id, const UUID &uu);
+
+    virtual std::map<UUID, const Rule *> get_rules(RuleID id) const = 0;
+    std::map<UUID, Rule *> get_rules(RuleID id);
+    std::map<UUID, Rule *> get_rules_nc(RuleID id)
+    {
+        return get_rules(id);
+    }
+
+    template <typename T = Rule> std::vector<const T *> get_rules_sorted(RuleID id) const
+    {
+        auto rs = get_rules(id);
+        std::vector<const T *> rv;
+        rv.reserve(rs.size());
+        for (auto &it : rs) {
+            rv.push_back(dynamic_cast<const T *>(it.second));
+        }
+        std::sort(rv.begin(), rv.end(), [](auto a, auto b) { return a->order < b->order; });
+        return rv;
+    }
+
+    template <typename T = Rule> std::vector<T *> get_rules_sorted(RuleID id)
+    {
+        std::vector<T *> r;
+        auto rs = static_cast<const Rules *>(this)->get_rules_sorted<T>(id);
+        r.reserve(rs.size());
+        std::transform(rs.begin(), rs.end(), std::back_inserter(r), [](auto x) { return const_cast<T *>(x); });
+        return r;
+    }
+
     virtual void remove_rule(RuleID id, const UUID &uu) = 0;
     virtual Rule *add_rule(RuleID id) = 0;
     void move_rule(RuleID id, const UUID &uu, int dir);
