@@ -6,6 +6,7 @@
 #include "nlohmann/json.hpp"
 #include "util/util.hpp"
 #include "logger/logger.hpp"
+#include "util/picture_load.hpp"
 
 namespace horizon {
 
@@ -238,7 +239,6 @@ void Schematic::smash_symbol(Sheet *sheet, SchematicSymbol *sym)
         auto uu = UUID::random();
         auto &x = sheet->texts.emplace(uu, uu).first->second;
         x.from_smash = true;
-        x.overridden = true;
         x.placement = sym->placement;
         Placement placement = it.second.placement;
         if (sym->symbol.texts.count(it.first)) {
@@ -471,6 +471,9 @@ void Schematic::expand(bool careful)
             it_sym.second.net->is_power = true;
             it_sym.second.net->is_power_forced = true;
         }
+        for (auto &it : sheet.texts) {
+            it.second.overridden = false;
+        }
     }
 
     if (!careful) {
@@ -547,7 +550,6 @@ void Schematic::expand(bool careful)
         for (auto &it_junc : sheet.junctions) {
             it_junc.second.net = nullptr;
             it_junc.second.bus = nullptr;
-            it_junc.second.temp = false;
         }
         for (auto &it_line : sheet.net_lines) {
             it_line.second.net = nullptr;
@@ -795,6 +797,33 @@ void Schematic::expand(bool careful)
             }
         }
     }
+
+    if (!careful) {
+        for (auto &it_sheet : sheets) {
+            Sheet &sheet = it_sheet.second;
+            for (auto &it_text : sheet.texts) {
+                auto &text = it_text.second;
+                if (!text.overridden) {
+                    Glib::ustring txt = text.text;
+                    Glib::MatchInfo ma;
+                    int start, end;
+                    while (get_sheetref_regex()->match(txt, ma) && ma.fetch_pos(0, start, end)) {
+                        auto ref_uuid = UUID(ma.fetch(1));
+                        if (sheets.count(ref_uuid)) {
+                            std::stringstream ss;
+                            ss << "[" << sheets.at(ref_uuid).index << "]";
+                            txt.replace(start, end - start, ss.str());
+                        }
+                        else {
+                            txt.replace(start, end - start, "[?]");
+                        }
+                        text.overridden = true;
+                    }
+                    text.text_override = txt;
+                }
+            }
+        }
+    }
 }
 
 void Schematic::annotate()
@@ -883,7 +912,7 @@ void Schematic::annotate()
                     if (annotation.fill_gaps && v.size() >= 2) {
                         bool hole = false;
                         for (auto it = v.begin(); it < v.end() - 1; it++) {
-                            if (*(it + 1) != (*it) + 1) {
+                            if (*it > sheet_offset && *(it + 1) != (*it) + 1) {
                                 n = (*it) + 1;
                                 hole = true;
                                 break;
@@ -941,6 +970,30 @@ std::map<UUIDPath<2>, std::string> Schematic::get_unplaced_gates() const
     return unplaced;
 }
 
+void Schematic::swap_gates(const UUID &comp_uu, const UUID &g1_uu, const UUID &g2_uu)
+{
+    block->swap_gates(comp_uu, g1_uu, g2_uu);
+    auto entity = block->components.at(comp_uu).entity;
+    for (auto &it_sheet : sheets) {
+        for (auto &it_sym : it_sheet.second.symbols) {
+            if (it_sym.second.gate->uuid == g1_uu) {
+                it_sym.second.gate = &entity->gates.at(g2_uu);
+            }
+            else if (it_sym.second.gate->uuid == g2_uu) {
+                it_sym.second.gate = &entity->gates.at(g1_uu);
+            }
+        }
+    }
+}
+
+Glib::RefPtr<Glib::Regex> Schematic::get_sheetref_regex()
+{
+    static auto regex =
+            Glib::Regex::create(R"(\$sheetref:([a-f0-9]{8}-[a-f0-9]{4}-4[a-f0-9]{3}-[89aAbB][a-f0-9]{3}-[a-f0-9]{12}))",
+                                Glib::RegexCompileFlags::REGEX_CASELESS | Glib::RegexCompileFlags::REGEX_OPTIMIZE);
+    return regex;
+}
+
 Schematic::Schematic(const Schematic &sch)
     : uuid(sch.uuid), block(sch.block), name(sch.name), sheets(sch.sheets), rules(sch.rules),
       group_tag_visible(sch.group_tag_visible), annotation(sch.annotation), pdf_export_settings(sch.pdf_export_settings)
@@ -1022,4 +1075,24 @@ json Schematic::serialize() const
 
     return j;
 }
+
+
+void Schematic::save_pictures(const std::string &dir) const
+{
+    std::list<const std::map<UUID, Picture> *> pictures;
+    for (const auto &it : sheets) {
+        pictures.push_back(&it.second.pictures);
+    }
+    pictures_save(pictures, dir, "sch");
+}
+
+void Schematic::load_pictures(const std::string &dir)
+{
+    std::list<std::map<UUID, Picture> *> pictures;
+    for (auto &it : sheets) {
+        pictures.push_back(&it.second.pictures);
+    }
+    pictures_load(pictures, dir, "sch");
+}
+
 } // namespace horizon
