@@ -12,19 +12,19 @@ public:
     SchematicRules();
 
     void load_from_json(const json &j);
-    RulesCheckResult check(RuleID id, const class Schematic *sch, class RulesCheckCache &cache);
+    RulesCheckResult check(RuleID id, const class Schematic *sch, class RulesCheckCache &cache) const;
     void apply(RuleID id, class Schematic *sch);
     json serialize() const;
-    std::set<RuleID> get_rule_ids() const;
-    Rule *get_rule(RuleID id);
-    Rule *get_rule(RuleID id, const UUID &uu);
-    std::map<UUID, Rule *> get_rules(RuleID id);
+    std::set<RuleID> get_rule_ids() const override;
+    const Rule *get_rule(RuleID id) const override;
+    const Rule *get_rule(RuleID id, const UUID &uu) const override;
+    std::map<UUID, const Rule *> get_rules(RuleID id) const;
     void remove_rule(RuleID id, const UUID &uu);
     Rule *add_rule(RuleID id);
 
 private:
     RuleSinglePinNet rule_single_pin_net;
 
-    RulesCheckResult check_single_pin_net(const class Schematic *sch, class RulesCheckCache &cache);
+    RulesCheckResult check_single_pin_net(const class Schematic *sch, class RulesCheckCache &cache) const;
 };
 } // namespace horizon
