@@ -12,6 +12,7 @@
 #include "nlohmann/json.hpp"
 #include "alphanum/alphanum.hpp"
 #include <gdk/gdkkeysyms.h>
+#include "imp/in_tool_action.hpp"
 
 namespace horizon {
 
@@ -333,6 +334,25 @@ Color color_from_json(const json &j)
     return c;
 }
 
+json colori_to_json(const ColorI &c)
+{
+    json j;
+    j["r"] = c.r;
+    j["g"] = c.g;
+    j["b"] = c.b;
+    return j;
+}
+
+
+ColorI colori_from_json(const json &j)
+{
+    ColorI c;
+    c.r = j.at("r");
+    c.g = j.at("g");
+    c.b = j.at("b");
+    return c;
+}
+
 static std::locale the_locale = std::locale::classic();
 
 void setup_locale()
@@ -530,21 +550,43 @@ std::string replace_placeholders(const std::string &s, std::function<std::string
     return r;
 }
 
-
-Coordi dir_from_arrow_key(unsigned int key)
+std::pair<Coordi, bool> dir_from_action(InToolActionID a)
 {
-    switch (key) {
-    case GDK_KEY_Up:
-        return {0, 1};
-    case GDK_KEY_Down:
-        return {0, -1};
-    case GDK_KEY_Left:
-        return {-1, 0};
-    case GDK_KEY_Right:
-        return {1, 0};
+    using I = InToolActionID;
+    switch (a) {
+    case I::MOVE_UP_FINE:
+        return {{0, 1}, true};
+    case I::MOVE_UP:
+        return {{0, 1}, false};
+
+    case I::MOVE_DOWN_FINE:
+        return {{0, -1}, true};
+    case I::MOVE_DOWN:
+        return {{0, -1}, false};
+
+    case I::MOVE_LEFT_FINE:
+        return {{-1, 0}, true};
+    case I::MOVE_LEFT:
+        return {{-1, 0}, false};
+
+    case I::MOVE_RIGHT_FINE:
+        return {{1, 0}, true};
+    case I::MOVE_RIGHT:
+        return {{1, 0}, false};
+
     default:
-        return {0, 0};
+        return {{0, 0}, false};
     }
 }
 
+Coordd project_onto_perp_bisector(const Coordd &a, const Coordd &b, const Coordd &p)
+{
+    const auto c = (a + b) / 2;
+    const auto d = b - a;
+    if (d.mag_sq() == 0)
+        return p;
+    const auto u = (d.dot(c) - d.dot(p)) / d.mag_sq();
+    return p + d * u;
+}
+
 } // namespace horizon
