@@ -9,6 +9,7 @@
 
 namespace horizon {
 using json = nlohmann::json;
+enum class InToolActionID;
 
 std::ifstream make_ifstream(const std::string &filename_utf8, std::ios_base::openmode mode = std::ios_base::in);
 std::ofstream make_ofstream(const std::string &filename_utf8, std::ios_base::openmode mode = std::ios_base::out);
@@ -61,6 +62,9 @@ void find_files_recursive(const std::string &base_path, std::function<void(const
 Color color_from_json(const json &j);
 json color_to_json(const Color &c);
 
+ColorI colori_from_json(const json &j);
+json colori_to_json(const ColorI &c);
+
 std::string format_m_of_n(unsigned int m, unsigned int n);
 std::string format_digits(unsigned int m, unsigned int digits_max);
 double parse_si(const std::string &inps);
@@ -69,6 +73,22 @@ void rmdir_recursive(const std::string &dir_name);
 std::string replace_placeholders(const std::string &s, std::function<std::string(const std::string &)> fn,
                                  bool keep_empty);
 
-Coordi dir_from_arrow_key(unsigned int key);
+std::pair<Coordi, bool> dir_from_action(InToolActionID a);
+
+template <typename T> std::pair<Coord<T>, Coord<T>> pad_bbox(std::pair<Coord<T>, Coord<T>> bb, T pad)
+{
+    bb.first.x -= pad;
+    bb.first.y -= pad;
+
+    bb.second.x += pad;
+    bb.second.y += pad;
+    return bb;
+}
+Coordd project_onto_perp_bisector(const Coordd &a, const Coordd &b, const Coordd &p);
+
+template <typename T> constexpr bool any_of(T value, std::initializer_list<T> choices)
+{
+    return std::count(choices.begin(), choices.end(), value);
+}
 
 } // namespace horizon
