@@ -1,6 +1,6 @@
 major = 1
-minor = 1
+minor = 2
 micro = 0
-name = "Blue sky"
+name = "Canopus"
 
 string = ".".join(str(x) for x in (major, minor, micro))
